@@ -250,10 +250,10 @@ action_proc_eor( Reduce *rc, Compile *compile,
 		vipsva( rc, out, 
 			"im_eorimage", PEGETII( a ), PEGETII( b ) ); 
 	else if( PEISIMAGE( a ) && PEISREAL( b ) ) 
-		vipsva( rc, out, "im_eorconst", 
+		vipsva( rc, out, "im_eorimageconst", 
 			PEGETII( a ), (int) PEGETREAL( b ) ); 
 	else if( PEISREAL( a ) && PEISIMAGE( b ) ) 
-		vipsva( rc, out, "im_eorconst", 
+		vipsva( rc, out, "im_eorimageconst", 
 			PEGETII( b ), (int) PEGETREAL( a ) ); 
 	else
 		action_boperror( rc, compile, NULL, op, name, a, b );
@@ -279,10 +279,10 @@ action_proc_bor( Reduce *rc, Compile *compile,
 		vipsva( rc, out, "im_orimage", 
 			PEGETII( a ), PEGETII( b ) ); 
 	else if( PEISIMAGE( a ) && PEISREAL( b ) ) 
-		vipsva( rc, out, "im_orconst", 
+		vipsva( rc, out, "im_orimageconst", 
 			PEGETII( a ), (int) PEGETREAL( b ) ); 
 	else if( PEISREAL( a ) && PEISIMAGE( b ) ) 
-		vipsva( rc, out, "im_orconst", 
+		vipsva( rc, out, "im_orimageconst", 
 			PEGETII( b ), (int) PEGETREAL( a ) ); 
 	else
 		action_boperror( rc, compile, NULL, op, name, a, b );
@@ -308,10 +308,10 @@ action_proc_band( Reduce *rc, Compile *compile,
 				op, name, a, b );
 	}
 	else if( PEISIMAGE( a ) && PEISREAL( b ) ) 
-		vipsva( rc, out, "im_andconst", 
+		vipsva( rc, out, "im_andimageconst", 
 			PEGETII( a ), (int) PEGETREAL( b ) ); 
 	else if( PEISREAL( a ) && PEISIMAGE( b ) ) 
-		vipsva( rc, out, "im_andconst", 
+		vipsva( rc, out, "im_andimageconst", 
 			PEGETII( b ), (int) PEGETREAL( a ) ); 
 	else
 		action_boperror( rc, compile, NULL, op, name, a, b );
@@ -323,56 +323,72 @@ action_proc_dot_add_link( Expr *expr, Symbol *child )
 	return( link_add( child, expr, TRUE ) );
 }
 
-/* Extract field from object. Be careful, obj can be equal to out. Compile is
- * a dummy parameter we accpt for compatibility ... we actually pick up our
- * context from the graph ourselves.
+/* Extract field from object. Be careful, a can be equal to out. 
  */
-void
-action_proc_dot( Reduce *rc, Compile *dummy,
-	int op, const char *name, HeapNode **arg, PElement *out )
+static void
+action_proc_dot( Reduce *rc, Compile *compile,
+	int op, const char *name, PElement *a, PElement *b, PElement *out )
 {
-	Compile *compile;
-	HeapNode *dotnode;
-	PElement obj;
-	PElement mem;
-	PElement c;
-	PElement pe;
+	char tag[256];
 
-	PEPOINTLEFT( arg[0], &pe );
-	assert( PEISNODE( &pe ) );
-	dotnode = PEGETVAL( &pe );
-	assert( dotnode->type == TAG_DOT );
+	if( PEISCLASS( a ) ) {
+		PElement c;
 
-	PEPOINTLEFT( dotnode, &mem );
-	PEPOINTRIGHT( dotnode, &c );
-	compile = PEGETCOMPILE( &c );
-	PEPOINTRIGHT( arg[0], &obj );
+		if( PEISTAG( b ) ) {
+			if( !class_get_member( a, PEGETTAG( b ), NULL, &c ) )
+				action_nomerror( rc, compile, a, b );
 
-	if( PEISSYMREF( &obj ) ) {
-		Symbol *sym = PEGETSYMREF( &obj );
+			PEPUTPE( out, &c );
+		}
+		else if( PEISSYMREF( b ) ) {
+			if( !class_get_symbol( a, PEGETSYMREF( b ), &c ) )
+				action_nomerror( rc, compile, a, b );
+
+			PEPUTPE( out, &c );
+		}
+		else if( heap_get_string( b, tag, 256 ) ) {
+			if( !class_get_member( a, tag, NULL, &c ) )
+				action_nomerror( rc, compile, a, b );
+
+			PEPUTPE( out, &c );
+		}
+		else
+			action_boperror( rc, compile, 
+				_( "Bad right hand side of '.'." ),
+				op, name, a, b );
+	}
+
+	else if( PEISSYMREF( a ) ) {
+		Symbol *sym = PEGETSYMREF( a );
 		Symbol *child;
 
 		if( !is_scope( sym ) )
 			action_boperror( rc, compile,
 				_( "Symbol on left hand side of '.' "
 				"is not scope" ), 
-				op, name, &obj, &mem );
-		if( !PEISTAG( &mem ) ) 
-			action_boperror( rc, compile,  
-				_( "Right hand side of '.' is not tag." ), 
-				op, name, &obj, &mem );
+				op, name, a, b );
 
 		assert( sym->expr );
-		if( !(child = compile_lookup( sym->expr->compile, 
-			PEGETTAG( &mem ) )) )
-			action_nomerror( rc, compile, &obj, &mem );
+
+		if( PEISTAG( b ) ) 
+			child = compile_lookup( sym->expr->compile, 
+				PEGETTAG( b ) );
+		else if( heap_get_string( b, tag, 256 ) ) 
+			child = compile_lookup( sym->expr->compile, tag );
+		else
+			action_boperror( rc, compile, 
+				_( "Bad right hand side of '.'." ),
+				op, name, a, b );
+
+		if( !child )
+			action_nomerror( rc, compile, a, b );
 
 		/* Add all exprs which use compile to dynamic link graph.
 		 */
 		if( slist_map( compile->exprs,
 			(SListMapFn) action_proc_dot_add_link, child ) )
 			action_boperror( rc, compile, error_get_sub(), 
-				op, name, &obj, &mem );
+				op, name, a, b );
 
 		/* Don't check for dirty here ... wait for
 		 * link.
@@ -384,30 +400,9 @@ action_proc_dot( Reduce *rc, Compile *dummy,
 			PEPUTP( out, ELEMENT_SYMREF, child );
 		}
 	}
-
-	else if( PEISCLASS( &obj ) ) {
-		if( PEISTAG( &mem ) ) {
-			if( !class_get_member( &obj, PEGETTAG( &mem ), 
-				NULL, &c ) )
-				action_nomerror( rc, compile, &obj, &mem );
-
-			PEPUTPE( out, &c );
-		}
-		else if( PEISSYMREF( &mem ) ) {
-			if( !class_get_symbol( &obj, PEGETSYMREF( &mem ), &c ) )
-				action_nomerror( rc, compile, &obj, &mem );
-
-			PEPUTPE( out, &c );
-		}
-		else
-			action_boperror( rc, compile, 
-				_( "Right hand side of '.' is not tag." ), 
-				op, name, &obj, &mem );
-	}
-
 	else
 		action_boperror( rc, compile, _( "Bad left hand side of '.'." ),
-			op, name, &obj, &mem );
+			op, name, a, b );
 }
 
 /* Less than or equal to.
@@ -513,14 +508,12 @@ action_node_equal( Reduce *rc, HeapNode *a, HeapNode *b )
 	case TAG_DOUBLE:
 		return( a->body.num == b->body.num );
 
-	case TAG_DOT:
-		return( strcmp( a->body.ptrs.left, b->body.ptrs.left ) == 0 );
-
 	case TAG_COMPLEX:
 		return( GETLEFT( a )->body.num == GETLEFT( b )->body.num &&
 			GETRIGHT( a )->body.num == GETRIGHT( b )->body.num );
 
 	case TAG_GEN:
+	case TAG_FILE:
 	case TAG_FREE:
 	case TAG_SHARED:
 	case TAG_REFERENCE:
@@ -533,6 +526,63 @@ action_node_equal( Reduce *rc, HeapNode *a, HeapNode *b )
 	}
 }
 
+static gboolean
+action_image_equal( Reduce *rc, Imageinfo *a, Imageinfo *b )
+{
+	Imageinfo *ii[2];
+	IMAGE *t1;
+	IMAGE *ai, *bi;
+	gboolean use_luts;
+	double mn;
+
+	/* Easy tests first.
+	 */
+	ii[0] = a;
+	ii[1] = b;
+	assert( !ii[0] && !ii[1] );
+	if( ii[0] == ii[1] )
+		/* Trivial!
+		 */
+		return( TRUE );
+
+	/* Extract images ... get LUTs if the underlying image is the
+	 * same.
+	 */
+	use_luts = imageinfo_same_underlying( ii, 2 );
+	if( !(ai = imageinfo_get( use_luts, ii[0] )) || 
+		!(bi = imageinfo_get( use_luts, ii[1] )) ) {
+		reduce_throw( rc );
+
+		/* Never get here, but keeps gcc happy.
+		 */
+		return( FALSE );
+	}
+
+	/* Size and bands must be the same.
+	 */
+	if( ai->Xsize != bi->Xsize ||
+		ai->Ysize != bi->Ysize ||
+		ai->Bands != bi->Bands ||
+		ai->Coding != bi->Coding )
+		return( FALSE );
+
+	/* Exhaustive test.
+	 */
+	if( !(t1 = im_open( "equals:1", "p" )) ) {
+		error_vips_all();
+		reduce_throw( rc );
+	}
+	if( im_equal( ai, bi, t1 ) ||
+		im_min( t1, &mn ) ) {
+		im_close( t1 );
+		error_vips_all();
+		reduce_throw( rc );
+	}
+	im_close( t1 );
+
+	return( mn == 255 );
+}
+
 /* Test two elements for equality. Force computation as required.
  */
 static gboolean
@@ -573,61 +623,12 @@ action_element_equal( Reduce *rc, PElement *p1, PElement *p2 )
 	case ELEMENT_BOOL:
 		return( PEGETBOOL( p1 ) == PEGETBOOL( p2 ) );
 
-	case ELEMENT_IMAGE:
-	{
-		Imageinfo *ii[2];
-		IMAGE *t1;
-		IMAGE *ai, *bi;
-		gboolean use_luts;
-		double mn;
-
-		/* Easy tests first.
-		 */
-		ii[0] = PEGETII( p1 );
-		ii[1] = PEGETII( p2 );
-		assert( !ii[0] && !ii[1] );
-		if( ii[0] == ii[1] )
-			/* Trivial!
-			 */
-			return( TRUE );
-
-		/* Extract images ... get LUTs if the underlying image is the
-		 * same.
-		 */
-		use_luts = imageinfo_same_underlying( ii, 2 );
-		if( !(ai = imageinfo_get( use_luts, ii[0] )) || 
-			!(bi = imageinfo_get( use_luts, ii[1] )) ) {
-			reduce_throw( rc );
-
-			/* Never get here, but keeps gcc happy.
-			 */
+	case ELEMENT_MANAGED:
+		if( PEISIMAGE( p1 ) && PEISIMAGE( p2 ) )
+			return( action_image_equal( rc,
+				PEGETII( p1 ), PEGETII( p2 ) ) );
+		else
 			return( FALSE );
-		}
-
-		/* Size and bands must be the same.
-		 */
-		if( ai->Xsize != bi->Xsize ||
-			ai->Ysize != bi->Ysize ||
-			ai->Bands != bi->Bands ||
-			ai->Coding != bi->Coding )
-			return( FALSE );
-
-		/* Exhaustive test.
-		 */
-		if( !(t1 = im_open( "equals:1", "p" )) ) {
-			error_vips_all();
-			reduce_throw( rc );
-		}
-		if( im_equal( ai, bi, t1 ) ||
-			im_min( t1, &mn ) ) {
-			im_close( t1 );
-			error_vips_all();
-			reduce_throw( rc );
-		}
-		im_close( t1 );
-
-		return( mn == 255 );
-	}
 
 	case ELEMENT_ELIST:
 		return( TRUE );
@@ -673,7 +674,7 @@ action_proc_equal( Reduce *rc, Compile *compile,
 			vipsva( rc, out, "im_equal", 
 				PEGETII( a ), PEGETII( b ) ); 
 		else
-			action_boperror( rc, compile, NULL, op, name, a, b );
+			PEPUTP( out, ELEMENT_BOOL, FALSE ); 
 	}
 	else {
 		/* Lazy form.
@@ -713,7 +714,7 @@ action_proc_notequal( Reduce *rc, Compile *compile,
 			vipsva( rc, out, "im_notequal", 
 				PEGETII( a ), PEGETII( b ) ); 
 		else
-			action_boperror( rc, compile, NULL, op, name, a, b );
+			PEPUTP( out, ELEMENT_BOOL, TRUE ); 
 	}
 	else {
 		res = action_element_equal( rc, a, b );
@@ -1178,6 +1179,10 @@ action_proc_bop_strict( Reduce *rc, Compile *compile,
 		action_proc_div( rc, compile, op, name, a, b, out );
 		break;
 
+	case BI_DOT:
+		action_proc_dot( rc, compile, op, name, a, b, out );
+		break;
+
 	case BI_POW:
 		action_proc_exp( rc, compile, op, name, a, b, out );
 		break;
@@ -1306,7 +1311,7 @@ action_proc_uop( Reduce *rc, Compile *compile,
 				reduce_throw( rc );
 		}
 		else if( PEISIMAGE( a ) ) 
-			vipsva( rc, out, "im_eorconst", 
+			vipsva( rc, out, "im_eorimageconst", 
 				PEGETII( a ), -1 );
 		else
 			action_uoperror( rc, compile, NULL, op, name, a ); 
@@ -1811,11 +1816,14 @@ action_if( Reduce *rc, Compile *compile,
 void
 action_proc_bop( Reduce *rc, Compile *compile, BinOp bop, HeapNode **arg )
 {
-	/* Special ninja magic :-( we need to handle reduce carefully here.
-	 */
-	if( bop == BI_LAND || bop == BI_LOR ) {
-		PElement a, b, out;
+	PElement a, b, out;
 
+	switch( bop ) {
+	case BI_LAND:
+	case BI_LOR:
+		/* Special ninja magic :-( we need to handle reduce carefully 
+		 * here.
+		 */
 		PEPOINTRIGHT( arg[0], &b );
 		PEPOINTRIGHT( arg[1], &a );
 		PEPOINTRIGHT( arg[0], &out );
@@ -1827,13 +1835,12 @@ action_proc_bop( Reduce *rc, Compile *compile, BinOp bop, HeapNode **arg )
 		 * shared node.
 		 */
 		PPUTLEFT( arg[0], ELEMENT_COMB, COMB_I );
-	}
 
-	/* If is lazy-ish in it's 2nd argument too.
-	 */
-	else if( bop == BI_IF ) {
-		PElement a, b, out;
+		break;
 
+	case BI_IF:
+		/* If is lazy-ish in it's 2nd argument too.
+		 */
 		PEPOINTRIGHT( arg[0], &b );
 		PEPOINTRIGHT( arg[1], &a );
 		PEPOINTRIGHT( arg[0], &out );
@@ -1845,21 +1852,27 @@ action_proc_bop( Reduce *rc, Compile *compile, BinOp bop, HeapNode **arg )
 		 * shared node.
 		 */
 		PPUTLEFT( arg[0], ELEMENT_COMB, COMB_I );
-	}
 
-	/* Strict, not overrideable.
-	 */
-	else if( bop == BI_PEQ || bop == BI_PNOTEQ ) 
+		break;
+
+	case BI_DOT:
+	case BI_PEQ:
+	case BI_PNOTEQ:
+		/* Strict, not overrideable.
+		 */
 		action_dispatch( rc, compile, reduce_spine, 
 			bop, OPERATOR_NAME( bop ), FALSE,
 			(ActionFn) action_proc_bop_strict, 2, arg, NULL );
 
-	/* Strict, overrideable.
-	 */
-	else
+		break;
+
+	default:
+		/* Strict, overrideable.
+		 */
 		action_dispatch( rc, compile, reduce_spine, 
 			bop, OPERATOR_NAME( bop ), TRUE,
 			(ActionFn) action_proc_bop_strict, 2, arg, NULL );
+	}
 }
 
 static void *
@@ -1925,6 +1938,11 @@ action_dispatch( Reduce *rc, Compile *compile, ReduceFunction rfn,
 	PElement a, b;
 	int i;
 
+	/* Don't allow nargs == 0. We rely on having a bit of graph we can
+	 * replace with (I result) for caching.
+	 */
+	assert( nargs > 0 );
+
 	/* Reduce all the args.
 	 */
 	for( i = 0; i < nargs; i++ ) {
