@@ -43,8 +43,10 @@ static GtkWidget *
 box_pick_parent( GtkWidget *par )
 {
 	if( !par ) {
-		if( mainw_pick_one() )
-			return( GTK_WIDGET( mainw_pick_one() ) );
+		Mainw *mainw;
+
+		if( (mainw = mainw_pick_one()) )
+			return( GTK_WIDGET( mainw ) );
 		else
 			return( main_window_top );
 	}
@@ -100,7 +102,7 @@ box_error( GtkWidget *par, const char *fmt, ... )
 		(iWindowBuildFn) box_build, buf, GTK_STOCK_DIALOG_ERROR, NULL );
 	idialog_set_callbacks( IDIALOG( idlg ), NULL, NULL, NULL, NULL );
 	idialog_add_ok( IDIALOG( idlg ), iwindow_true_cb, GTK_STOCK_OK );
-	idialog_set_parent( IDIALOG( idlg ), box_pick_parent( par ) );
+	iwindow_set_parent( IWINDOW( idlg ), box_pick_parent( par ) );
 	iwindow_build( IWINDOW( idlg ) );
 
 	gtk_widget_show( GTK_WIDGET( idlg ) );
@@ -177,13 +179,13 @@ box_info( GtkWidget *par, const char *top, const char *sub, ... )
 			buf, GTK_STOCK_DIALOG_INFO, NULL );
 	idialog_set_callbacks( IDIALOG( idlg ), NULL, NULL, NULL, NULL );
 	idialog_add_ok( IDIALOG( idlg ), iwindow_true_cb, GTK_STOCK_OK );
-	idialog_set_parent( IDIALOG( idlg ), box_pick_parent( par ) );
+	iwindow_set_parent( IWINDOW( idlg ), box_pick_parent( par ) );
 	iwindow_build( IWINDOW( idlg ) );
 
 	gtk_widget_show( GTK_WIDGET( idlg ) );
 }
 
-/* Pop up a `Are you sure?' window. 
+/* Pop up an 'Are you sure?' window. 
  */
 iDialog *
 box_yesno( GtkWidget *par, 
@@ -207,7 +209,7 @@ box_yesno( GtkWidget *par,
 	idialog_set_callbacks( IDIALOG( idlg ), cancelcb, NULL, NULL, client );
 	idialog_add_ok( IDIALOG( idlg ), okcb, yes_label );
 	idialog_set_notify( IDIALOG( idlg ), nfn, sys );
-	idialog_set_parent( IDIALOG( idlg ), box_pick_parent( par ) );
+	iwindow_set_parent( IWINDOW( idlg ), box_pick_parent( par ) );
 	iwindow_build( IWINDOW( idlg ) );
 
 	gtk_widget_show( GTK_WIDGET( idlg ) );
@@ -241,7 +243,7 @@ box_savenosave( GtkWidget *par,
 	idialog_add_ok( IDIALOG( idlg ), nosave, _( "Close _without Saving" ) );
 	idialog_add_ok( IDIALOG( idlg ), save, GTK_STOCK_SAVE );
 	idialog_set_notify( IDIALOG( idlg ), nfn, sys );
-	idialog_set_parent( IDIALOG( idlg ), box_pick_parent( par ) );
+	iwindow_set_parent( IWINDOW( idlg ), box_pick_parent( par ) );
 	iwindow_build( IWINDOW( idlg ) );
 
 	gtk_widget_show( GTK_WIDGET( idlg ) );
@@ -281,15 +283,14 @@ about_build( iDialog *idlg, GtkWidget *work )
 		PACKAGE );
 	buf_appendf( &buf, "%s\n\n", txt2 );
 
-	im_snprintf( txt2, MAX_DIALOG_TEXT, 
-		_( "%s: (c) 2005 The National Gallery" ), PACKAGE );
+	im_snprintf( txt2, MAX_DIALOG_TEXT, _( NIP_COPYRIGHT ), PACKAGE );
 	buf_appendf( &buf, "%s\n", txt2 );
 
 {
 	char buf1[FILENAME_MAX];
 	char buf2[FILENAME_MAX];
 
-	im_snprintf( buf1, FILENAME_MAX, "%s" IM_DIR_SEP_STR "start",
+	im_snprintf( buf1, FILENAME_MAX, "%s" G_DIR_SEPARATOR_S "start",
 		get_savedir() );
 	expand_variables( buf1, buf2 );
         nativeize_path( buf2 );
@@ -351,7 +352,7 @@ box_about( GtkWidget *par )
 	idialog_set_build( IDIALOG( idlg ), 
 		(iWindowBuildFn) about_build, NULL, NULL, NULL );
 	idialog_add_ok( IDIALOG( idlg ), iwindow_true_cb, GTK_STOCK_OK );
-	idialog_set_parent( IDIALOG( idlg ), box_pick_parent( par ) );
+	iwindow_set_parent( IWINDOW( idlg ), box_pick_parent( par ) );
 	iwindow_build( IWINDOW( idlg ) );
 
 	gtk_widget_show( GTK_WIDGET( idlg ) );
@@ -680,77 +681,101 @@ imageheader_destroy( GtkObject *object )
 		GTK_OBJECT_CLASS( imageheader_parent_class )->destroy( object );
 }
 
+/* Make a row for the header display. Align the names with "group".
+ */
+static GtkWidget *
+imageheader_row_new( GtkSizeGroup *group, const char *field, GValue *value )
+{
+	GtkWidget *hbox;
+	GtkWidget *label;
+	const char *extra;
+        char *str_value;
+
+	hbox = gtk_hbox_new( FALSE, 12 );
+
+	label = gtk_label_new( field );
+	gtk_misc_set_alignment( GTK_MISC( label ), 0.0, 0.5 );
+        gtk_box_pack_start( GTK_BOX( hbox ), label, FALSE, FALSE, 2 );
+	gtk_size_group_add_widget( group, label );
+
+	label = gtk_label_new( "=" );
+        gtk_box_pack_start( GTK_BOX( hbox ), label, FALSE, FALSE, 2 );
+
+	label = gtk_label_new( "" );
+        gtk_box_pack_start( GTK_BOX( hbox ), label, FALSE, FALSE, 2 );
+
+	/* Look for enums and decode them.
+	 */
+	extra = NULL;
+	if( strcmp( field, "Coding" ) == 0 )
+		extra = NN( im_Coding2char( g_value_get_int( value ) ) );
+	else if( strcmp( field, "BandFmt" ) == 0 )
+		extra = NN( im_BandFmt2char( g_value_get_int( value ) ) );
+	else if( strcmp( field, "Type" ) == 0 )
+		extra = NN( im_Type2char( g_value_get_int( value ) ) );
+
+	str_value = g_strdup_value_contents( value );
+	if( extra )
+		set_glabel( label, "%s - %s", str_value, extra );
+	else
+		set_glabel( label, "%s", str_value );
+	g_free( str_value );
+
+	return( hbox );
+}
+
+/* Mapped over the header fields to make rows.
+ */
+static void *
+imageheader_row_map_fn( IMAGE *im, 
+	const char *field, GValue *value, Imageheader *imageheader )
+{
+	GtkWidget *hbox;
+
+	hbox = imageheader_row_new( imageheader->group, field, value );
+	gtk_box_pack_start( GTK_BOX( imageheader->fields ), 
+		hbox, FALSE, FALSE, 0 );
+
+	return( NULL );
+}
+
 static void
 imageheader_refresh( Imageheader *imageheader )
 {
+	DESTROY_GTK( imageheader->fields );
+
 	if( imageheader->conv && imageheader->conv->ii ) {
 		IMAGE *im = imageinfo_get( FALSE, imageheader->conv->ii );
 
-		set_glabel( imageheader->xsize, "%d", im->Xsize );
-		set_glabel( imageheader->ysize, "%d", im->Ysize );
-		set_glabel( imageheader->bands, "%d", im->Bands );
-		set_glabel( imageheader->bbits, "%d", im->Bbits );
-		set_glabel( imageheader->fmt, "%s", 
-			NN( im_BandFmt2char( im->BandFmt ) ) );
-		set_glabel( imageheader->coding, "%s", 
-			NN( im_Coding2char( im->Coding ) ) );
-		set_glabel( imageheader->type, "%s", 
-			NN( im_Type2char( im->Type ) ) );
-		set_glabel( imageheader->xres, "%g", im->Xres );
-		set_glabel( imageheader->yres, "%g", im->Yres );
-		set_glabel( imageheader->xoff, "%d", im->Xoffset );
-		set_glabel( imageheader->yoff, "%d", im->Yoffset );
-		set_glabel( imageheader->filename, "%s", im->filename );
+		imageheader->group = 
+			gtk_size_group_new( GTK_SIZE_GROUP_HORIZONTAL );
+		imageheader->fields = gtk_vbox_new( FALSE, 2 );
+		gtk_scrolled_window_add_with_viewport( 
+			GTK_SCROLLED_WINDOW( imageheader->swin_fields ), 
+			imageheader->fields );
+		im_header_map( im, 
+			(im_header_map_fn) imageheader_row_map_fn,
+			imageheader );
+		gtk_widget_show_all( imageheader->fields );
 
 		gtk_text_buffer_set_text( 
 			gtk_text_view_get_buffer( 
 				GTK_TEXT_VIEW( imageheader->history ) ),
-			im->Hist, -1 );
+			im_history_get( im ), -1 );
 	}
 	else {
-		set_glabel( imageheader->xsize, "--" );
-		set_glabel( imageheader->ysize, "--" );
-		set_glabel( imageheader->bands, "--" );
-		set_glabel( imageheader->bbits, "--" );
-		set_glabel( imageheader->fmt, "--" );
-		set_glabel( imageheader->coding, "--" );
-		set_glabel( imageheader->type, "--" );
-		set_glabel( imageheader->xres, "--" );
-		set_glabel( imageheader->yres, "--" );
-		set_glabel( imageheader->xoff, "--" );
-		set_glabel( imageheader->yoff, "--" );
-		set_glabel( imageheader->filename, "--" );
 		gtk_editable_delete_text( GTK_EDITABLE( imageheader->history ),
 			0, -1 );
 	}
 }
 
-static GtkWidget *
-imageheader_label( GtkWidget *table, int row, const char *text )
-{
-	GtkWidget *label;
-
-	label = gtk_label_new( text );
-	gtk_misc_set_alignment( GTK_MISC( label ), 1.0, 0.5 );
-	gtk_table_attach( GTK_TABLE( table ), label,
-		0, 1, row, row + 1, 
-		GTK_FILL, GTK_FILL, 0, 0 );
-	label = gtk_label_new( text );
-	gtk_misc_set_alignment( GTK_MISC( label ), 0.0, 0.5 );
-	gtk_table_attach( GTK_TABLE( table ), label,
-		1, 2, row, row + 1, 
-		GTK_FILL, GTK_FILL, 0, 0 );
-
-	return( label );
-}
-
 static void
 imageheader_build( GtkWidget *widget )
 {
 	Imageheader *imageheader = IMAGEHEADER( widget );
 	iDialog *idlg = IDIALOG( widget );
 
-	GtkWidget *table;
+	GtkWidget *label;
 	GtkWidget *swin;
 	PangoFontDescription *font_desc;
 
@@ -763,26 +788,29 @@ imageheader_build( GtkWidget *widget )
 	if( IWINDOW_CLASS( imageheader_parent_class )->build )
 		(*IWINDOW_CLASS( imageheader_parent_class )->build)( widget );
 
-        table = gtk_table_new( 10, 2, FALSE );
-        gtk_box_pack_start( GTK_BOX( idlg->work ), table, FALSE, FALSE, 2 );
+	/* Scrolled area to hold fields.
+	 */
+	label = gtk_label_new( _( "Image header fields" ) );
+	gtk_misc_set_alignment( GTK_MISC( label ), 0.0, 0.5 );
+        gtk_box_pack_start( GTK_BOX( idlg->work ), label, FALSE, FALSE, 2 );
+	imageheader->swin_fields = gtk_scrolled_window_new( NULL, NULL );
+	gtk_scrolled_window_set_policy( 
+		GTK_SCROLLED_WINDOW( imageheader->swin_fields ),
+		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
+        gtk_box_pack_start( GTK_BOX( idlg->work ), 
+		imageheader->swin_fields, TRUE, TRUE, 2 );
 
-	imageheader->xsize = imageheader_label( table, 0, "Xsize = " );
-	imageheader->ysize = imageheader_label( table, 1, "Ysize = " );
-	imageheader->bands = imageheader_label( table, 2, "Bands = " );
-	imageheader->bbits = imageheader_label( table, 3, "Bbits = " );
-	imageheader->fmt = imageheader_label( table, 4, "BandFmt = " );
-	imageheader->coding = imageheader_label( table, 5, "Coding = " );
-	imageheader->type = imageheader_label( table, 6, "Type = " );
-	imageheader->xres = imageheader_label( table, 7, "Xres = " );
-	imageheader->yres = imageheader_label( table, 8, "Yres = " );
-	imageheader->xoff = imageheader_label( table, 9, "Xoffset = " );
-	imageheader->yoff = imageheader_label( table, 10, "Yoffset = " );
-	imageheader->filename = imageheader_label( table, 11, "filename = " );
+	/* Created in _refresh()
+	 */
+	imageheader->fields = NULL;
 
+	label = gtk_label_new( _( "Image history" ) );
+	gtk_misc_set_alignment( GTK_MISC( label ), 0.0, 0.5 );
+        gtk_box_pack_start( GTK_BOX( idlg->work ), label, FALSE, FALSE, 2 );
 	swin = gtk_scrolled_window_new( NULL, NULL );
 	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( swin ),
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
-        gtk_box_pack_start( GTK_BOX( idlg->work ), swin, TRUE, TRUE, 2 );
+        gtk_box_pack_end( GTK_BOX( idlg->work ), swin, TRUE, TRUE, 2 );
 	imageheader->history = gtk_text_view_new();
 	gtk_text_view_set_editable( GTK_TEXT_VIEW( imageheader->history ), 
 		FALSE );
@@ -795,6 +823,8 @@ imageheader_build( GtkWidget *widget )
 
 	imageheader_refresh( imageheader );
 
+        gtk_window_set_default_size( GTK_WINDOW( imageheader ), 550, 550 );
+
 	gtk_widget_show_all( idlg->work );
 }
 
@@ -883,7 +913,7 @@ imageheader_new( Conversion *conv )
 void
 box_url( GtkWidget *par, const char *url )
 {
-#ifdef HAVE_WINDOWS_H
+#ifdef OS_WIN32
 	char url2[FILENAME_MAX];
 	int v;
 	
@@ -895,7 +925,7 @@ box_url( GtkWidget *par, const char *url )
 			"windows error code = %d." ), url, v );
 		box_alert( par );
 	}
-#elif defined MAC_OSX
+#elif defined OS_DARWIN
 	static gboolean shown = FALSE;
 
 	(void) systemf( "open %s", url );
@@ -946,7 +976,7 @@ box_url( GtkWidget *par, const char *url )
 		 */
 		shown = TRUE;
 	}
-#endif /*HAVE_WINDOWS_H*/
+#endif /*OS_WIN32*/
 }
 
 /* Progress dialog.
@@ -1147,9 +1177,11 @@ static void
 splash_toggle_change_cb( GtkWidget *widget )
 {
 	if( GTK_TOGGLE_BUTTON( widget )->active ) 
-		unlinkf( "%s", NO_SPLASH );
+		unlinkf( "%s" G_DIR_SEPARATOR_S "%s", 
+			get_savedir(), NO_SPLASH );
 	else 
-		touchf( "%s", NO_SPLASH );
+		touchf( "%s" G_DIR_SEPARATOR_S "%s", 
+			get_savedir(), NO_SPLASH );
 }
 
 static void
@@ -1237,10 +1269,12 @@ splash_updatev( Splash *splash, const char *fmt, va_list ap )
 {
 	char buf1[100];
 	char buf2[150];
+	char buf3[150];
 
 	(void) im_vsnprintf( buf1, 100, fmt, ap );
+	escape_markup( buf1, buf3, 150 );
 	(void) im_snprintf( buf2, 150, 
-		"<span style=\"italic\" size=\"smaller\">%s</span>" , buf1 );
+		"<span style=\"italic\" size=\"smaller\">%s</span>" , buf3 );
 	gtk_label_set_markup( GTK_LABEL( splash->label ), buf2 );
 
 	while( g_main_context_iteration( NULL, FALSE ) )
@@ -1436,7 +1470,7 @@ fontbutton_clicked( GtkButton *button )
 			fontbutton );
 		idialog_add_ok( IDIALOG( fontbutton->fontchooser ), 
 			fontbutton_ok_cb, _( "Set Font" ) );
-		idialog_set_parent( IDIALOG( fontbutton->fontchooser ), 
+		iwindow_set_parent( IWINDOW( fontbutton->fontchooser ), 
 			GTK_WIDGET( button ) );
 		idialog_set_pinup( IDIALOG( fontbutton->fontchooser ), TRUE );
 		iwindow_build( IWINDOW( fontbutton->fontchooser ) );
@@ -1525,7 +1559,7 @@ fontbutton_set_font_name( Fontbutton *fontbutton, const char *font_name )
 
 	if( !fontbutton->font_name ||
 		strcmp( fontbutton->font_name, font_name ) != 0 ) {
-		SETSTR( fontbutton->font_name, font_name );
+		IM_SETSTR( fontbutton->font_name, font_name );
 
 		im_strncpy( font, font_name, 256 );
 		for( i = strlen( font ) - 1; i > 0 && isdigit( font[i] ); i-- )
