@@ -146,10 +146,10 @@ browse_init( Browse *browse )
 GtkType
 browse_get_type( void )
 {
-	static GtkType browse_type = 0;
+	static GtkType type = 0;
 
-	if (!browse_type) {
-		static const GtkTypeInfo browse_info = {
+	if( !type)  {
+		static const GtkTypeInfo info = {
 			"Browse",
 			sizeof( Browse ),
 			sizeof( BrowseClass ),
@@ -160,10 +160,10 @@ browse_get_type( void )
 			(GtkClassInitFunc) NULL,
 		};
 
-		browse_type = gtk_type_unique( TYPE_IDIALOG, &browse_info );
+		type = gtk_type_unique( TYPE_IDIALOG, &info );
 	}
 
-	return( browse_type );
+	return( type );
 }
 
 GtkWidget *
@@ -178,7 +178,7 @@ void
 browse_set_filesel( Browse *browse, Filesel *filesel )
 {
 	browse->filesel = filesel;
-	idialog_set_parent( IDIALOG( browse ), GTK_WIDGET( filesel ) );
+	iwindow_set_parent( IWINDOW( browse ), GTK_WIDGET( filesel ) );
 }
 
 /* Single click on a button.
@@ -230,13 +230,17 @@ browse_add_widget( Browse *browse, GtkWidget *but )
 static void
 browse_add_image( Browse *browse, Imageinfo *ii )
 {
+	Conversion *conv;
 	Imagedisplay *id;
 
 	GtkWidget *but;
 	BufInfo caption;
         char buf[MAX_LINELENGTH];
 
-	id = imagedisplay_new( conversion_new( ii ) );
+	conv = conversion_new( ii );
+	conv->tile_size = 16;
+	conv->fade_steps = 1;
+	id = imagedisplay_new( conv );
 	gtk_widget_set_size_request( GTK_WIDGET( id ), 64, 64 );
 	imagedisplay_set_shrink_to_fit( id, TRUE );
 
@@ -342,8 +346,7 @@ browse_idle( Browse *browse )
 	/* If we're using it, it will have been reffed by the imagedisplay in
 	 * the button.
 	 */
-	if( ii )
-		imageinfo_destroy_nonheap( ii );
+	MANAGED_UNREF( ii );
 
 	im_free( name );
 
@@ -367,7 +370,7 @@ browse_refresh( Browse *browse, const gchar *dirname )
 
 	browse_refresh_stop( browse );
 
-	SETSTR( browse->dirname, dirname );
+	IM_SETSTR( browse->dirname, dirname );
 	IM_FREEF( gtk_widget_destroy, browse->table );
         browse->row = 0;
 
