@@ -29,6 +29,11 @@
 
 #include "ip.h"
 
+#ifdef HAVE_GSL
+#include <gsl/gsl_sf_gamma.h>
+#include <gsl/gsl_errno.h>
+#endif /*HAVE_GSL*/
+
 /* Trace builtin calls.
 #define DEBUG
  */
@@ -55,36 +60,40 @@ static gboolean isany( Reduce *rc, PElement *base ) { return( TRUE ); }
 
 /* Other PEIS as functions.
  */
-static gboolean pe_isimage( Reduce *rc, PElement *base ) 
+static gboolean pe_is_image( Reduce *rc, PElement *base ) 
 	{ return( PEISIMAGE( base ) ); }
-static gboolean pe_isreal( Reduce *rc, PElement *base ) 
+static gboolean pe_is_real( Reduce *rc, PElement *base ) 
 	{ return( PEISREAL( base ) ); }
-static gboolean pe_iscomplex( Reduce *rc, PElement *base ) 
+static gboolean pe_is_complex( Reduce *rc, PElement *base ) 
 	{ return( PEISCOMPLEX( base ) ); }
-static gboolean pe_isbool( Reduce *rc, PElement *base ) 
+static gboolean pe_is_bool( Reduce *rc, PElement *base ) 
 	{ return( PEISBOOL( base ) ); }
-static gboolean pe_ischar( Reduce *rc, PElement *base ) 
+static gboolean pe_is_char( Reduce *rc, PElement *base ) 
 	{ return( PEISCHAR( base ) ); }
-static gboolean pe_islist( Reduce *rc, PElement *base ) 
+static gboolean pe_is_list( Reduce *rc, PElement *base ) 
 	{ return( PEISLIST( base ) ); }
-static gboolean pe_isflist( Reduce *rc, PElement *base ) 
+static gboolean pe_is_flist( Reduce *rc, PElement *base ) 
 	{ return( PEISFLIST( base ) ); }
-static gboolean pe_isclass( Reduce *rc, PElement *base )
+static gboolean pe_is_class( Reduce *rc, PElement *base )
         { return( PEISCLASS( base ) ); }
+static gboolean pe_is_gobject( Reduce *rc, PElement *base )
+        { return( PEISMANAGEDGOBJECT( base ) ); }
 
 /* The types we might want to spot for builtins.
  */
-static BuiltinTypeSpot vimage_spot = { "vips_image", pe_isimage };
-static BuiltinTypeSpot real_spot = { "real", pe_isreal };
-static BuiltinTypeSpot bool_spot = { "bool", pe_isbool };
+static BuiltinTypeSpot vimage_spot = { "vips_image", pe_is_image };
+static BuiltinTypeSpot real_spot = { "real", pe_is_real };
+static BuiltinTypeSpot bool_spot = { "bool", pe_is_bool };
 static BuiltinTypeSpot complex_spot = { "complex|image", iscomplexarg };
-static BuiltinTypeSpot flist_spot = { "non-empty list", pe_isflist };
-static BuiltinTypeSpot realvec_spot = { "[real]", reduce_isrealvec };
-static BuiltinTypeSpot matrix_spot = { "[[real]]", reduce_ismatrix };
-static BuiltinTypeSpot string_spot = { "[char]", reduce_isfinitestring };
+static BuiltinTypeSpot flist_spot = { "non-empty list", pe_is_flist };
+static BuiltinTypeSpot realvec_spot = { "[real]", reduce_is_realvec };
+static BuiltinTypeSpot matrix_spot = { "[[real]]", reduce_is_matrix };
+static BuiltinTypeSpot string_spot = { "[char]", reduce_is_finitestring };
+static BuiltinTypeSpot list_spot = { "[*]", reduce_is_list };
 static BuiltinTypeSpot math_spot = { "image|real|complex", ismatharg };
-static BuiltinTypeSpot instance_spot = { "class instance", pe_isclass };
+static BuiltinTypeSpot instance_spot = { "class instance", pe_is_class };
 static BuiltinTypeSpot any_spot = { "any", isany };
+static BuiltinTypeSpot gobject_spot = { "GObject", pe_is_gobject };
 
 /* Args for "_".
  */
@@ -95,7 +104,6 @@ static BuiltinTypeSpot *underscore_args[] = {
 
 /* Do a _ call. Args already spotted.
  */
-/*ARGSUSED*/
 static void
 apply_underscore_call( Reduce *rc, 
 	const char *name, HeapNode **arg, PElement *out )
@@ -108,34 +116,8 @@ apply_underscore_call( Reduce *rc,
 
 	/* Pump though gettext.
 	 */
-	heap_string_new( rc->heap, _( text ), out );
-}
-
-/* Args for "get_member".
- */
-static BuiltinTypeSpot *get_member_args[] = {
-        &string_spot,
-        &any_spot
-};
-
-/* Do a get_member call. Args already spotted.
- */
-/*ARGSUSED*/
-static void
-apply_get_member_call( Reduce *rc, 
-	const char *name, HeapNode **arg, PElement *out )
-{
-        PElement rhs;
-	char mname[MAX_STRSIZE];
-        PElement member;
-
-	PEPOINTRIGHT( arg[1], &rhs );
-	(void) reduce_get_string( rc, &rhs, mname, MAX_STRSIZE );
-        PEPOINTRIGHT( arg[0], &rhs );
-	if( !class_get_member( &rhs, mname, NULL, &member ) )
+	if( !heap_string_new( rc->heap, _( text ), out ) )
 		reduce_throw( rc );
-
-	PEPUTPE( out, &member );
 }
 
 /* Args for "has_member".
@@ -147,7 +129,6 @@ static BuiltinTypeSpot *has_member_args[] = {
 
 /* Do a has_member call. Args already spotted.
  */
-/*ARGSUSED*/
 static void
 apply_has_member_call( Reduce *rc, 
 	const char *name, HeapNode **arg, PElement *out )
@@ -172,7 +153,6 @@ static BuiltinTypeSpot *is_instanceof_args[] = {
 
 /* Do an is_instance call. Args already spotted.
  */
-/*ARGSUSED*/
 static void
 apply_is_instanceof_call( Reduce *rc, 
 	const char *name, HeapNode **arg, PElement *out )
@@ -183,7 +163,7 @@ apply_is_instanceof_call( Reduce *rc,
 	PEPOINTRIGHT( arg[1], &rhs );
 	(void) reduce_get_string( rc, &rhs, kname, MAX_STRSIZE );
         PEPOINTRIGHT( arg[0], &rhs );
-	PEPUTP( out, ELEMENT_BOOL, reduce_isinstanceof( rc, kname, &rhs ) );
+	PEPUTP( out, ELEMENT_BOOL, reduce_is_instanceof( rc, kname, &rhs ) );
 }
 
 /* Args for builtin on complex.
@@ -194,7 +174,6 @@ static BuiltinTypeSpot *complex_args[] = {
 
 /* Do a complex op. Args already spotted.
  */
-/*ARGSUSED*/
 static void
 apply_complex_call( Reduce *rc, 
 	const char *name, HeapNode **arg, PElement *out )
@@ -231,7 +210,6 @@ static BuiltinTypeSpot *flist_args[] = {
 
 /* Do a complex op. Args already spotted.
  */
-/*ARGSUSED*/
 static void
 apply_list_call( Reduce *rc, 
 	const char *name, HeapNode **arg, PElement *out )
@@ -254,6 +232,43 @@ apply_list_call( Reduce *rc,
 		error( "internal error #098734953" );
 }
 
+/* "gammq"
+ */
+static BuiltinTypeSpot *gammq_args[] = { 
+	&real_spot,
+	&real_spot
+};
+
+static void
+apply_gammq_call( Reduce *rc, 
+	const char *name, HeapNode **arg, PElement *out )
+{
+	PElement rhs;
+	double a, x, Q;
+
+	PEPOINTRIGHT( arg[1], &rhs );
+	a = PEGETREAL( &rhs );
+	PEPOINTRIGHT( arg[0], &rhs );
+	x = PEGETREAL( &rhs );
+
+	if( a <= 0 || x < 0 ) {
+		error_top( _( "Out of range." ) );
+		error_sub( _( "gammq arguments must be a > 0, x >= 0." ) );
+		reduce_throw( rc );
+	}
+
+#ifdef HAVE_GSL
+	Q = gsl_sf_gamma_inc_Q( a, x );
+#else /*!HAVE_GSL*/
+	error_top( _( "Not available." ) );
+	error_sub( _( "No GSL library available for gammq." ) );
+	reduce_throw( rc );
+#endif /*HAVE_GSL*/
+
+	if( !heap_real_new( rc->heap, Q, out ) )
+		reduce_throw( rc );
+}
+
 /* Args for "vips_image". 
  */
 static BuiltinTypeSpot *image_args[] = { 
@@ -262,12 +277,11 @@ static BuiltinTypeSpot *image_args[] = {
 
 /* Do a image call.
  */
-/*ARGSUSED*/
 static void
 apply_image_call( Reduce *rc, 
 	const char *name, HeapNode **arg, PElement *out )
 {
-	Heap *hi = rc->heap;
+	Heap *heap = rc->heap;
 	PElement rhs;
 	char buf[FILENAME_MAX];
 	char *fn;
@@ -276,21 +290,136 @@ apply_image_call( Reduce *rc,
 	/* Get string. 
 	 */
 	PEPOINTRIGHT( arg[0], &rhs );
-	if( reduce_get_string( rc, &rhs, buf, FILENAME_MAX ) < 0 )
-		error( "internal error #29753946" );
+	(void) reduce_get_string( rc, &rhs, buf, FILENAME_MAX );
 
 	/* Try to load image from given string.
 	 */
 	if( !(fn = path_find_file( PATH_SEARCH, buf )) )
 		reduce_throw( rc );
-	if( !(ii = imageinfo_new_input( main_imageinfogroup, NULL, hi, fn )) ) {
+	if( !(ii = imageinfo_new_input( 
+		main_imageinfogroup, NULL, heap, fn )) ) {
 		IM_FREE( fn );
 		reduce_throw( rc );
 	}
 	IM_FREE( fn );
 
-	PEPUTP( out, ELEMENT_IMAGE, ii );
-	imageinfo_destroy_nonheap( ii );
+	PEPUTP( out, ELEMENT_MANAGED, ii );
+	MANAGED_UNREF( ii );
+}
+
+/* Args for "vips_image_new". 
+ */
+static BuiltinTypeSpot *image_new_args[] = { 
+	&string_spot,
+	&list_spot 
+};
+
+/* Do a image_new call.
+ */
+static void
+apply_image_new_call( Reduce *rc, 
+	const char *name, HeapNode **arg, PElement *out )
+{
+	PElement rhs;
+	char buf[FILENAME_MAX];
+	Managed *managed;
+
+	/* Get string. 
+	 */
+	PEPOINTRIGHT( arg[1], &rhs );
+	(void) reduce_get_string( rc, &rhs, buf, FILENAME_MAX );
+
+	/* List of options.
+	 */
+	PEPOINTRIGHT( arg[0], &rhs );
+
+	managed = call8_image_new( rc, buf, &rhs );
+
+	PEPUTP( out, ELEMENT_MANAGED, managed );
+	managed_destroy_nonheap( managed );
+}
+
+/* Args for "vips_call". 
+ */
+static BuiltinTypeSpot *call_args[] = { 
+	&string_spot,
+	&list_spot,
+	&list_spot 
+};
+
+/* Do a vips_call call.
+ */
+static void
+apply_call_call( Reduce *rc, 
+	const char *name, HeapNode **arg, PElement *out )
+{
+	PElement rhs;
+	char buf[256];
+	PElement required;
+	PElement optional;
+
+	PEPOINTRIGHT( arg[2], &rhs );
+	reduce_get_string( rc, &rhs, buf, 256 );
+	PEPOINTRIGHT( arg[1], &required );
+	PEPOINTRIGHT( arg[0], &optional );
+
+	call8_vips( rc, buf, &required, &optional, out );
+}
+
+/* Args for "vips_header_get". 
+ */
+static BuiltinTypeSpot *header_get_args[] = { 
+	&string_spot,
+	&gobject_spot
+};
+
+/* Do a vips_header_get call.
+ */
+static void
+apply_header_get_call( Reduce *rc, 
+	const char *name, HeapNode **arg, PElement *out )
+{
+	PElement rhs;
+	char buf[256];
+	GObject *object;
+
+	PEPOINTRIGHT( arg[1], &rhs );
+	reduce_get_string( rc, &rhs, buf, 256 );
+	PEPOINTRIGHT( arg[0], &rhs );
+	assert( PEISMANAGEDGOBJECT( &rhs ) );
+	object = PEGETMANAGEDGOBJECT( &rhs );
+
+	if( !VIPS_IS_OBJECT( object ) ) {
+		error_top( _( "Bad argument." ) );
+		error_sub( _( "GObject is not a VipsObject." ) );
+		reduce_throw( rc );
+	}
+
+	call8_header_get( rc, buf, VIPS_OBJECT( object ), out );
+}
+
+/* Args for "read". 
+ */
+static BuiltinTypeSpot *read_args[] = { 
+	&string_spot 
+};
+
+/* Do a read call.
+ */
+static void
+apply_read_call( Reduce *rc, 
+	const char *name, HeapNode **arg, PElement *out )
+{
+	PElement rhs;
+	char buf[FILENAME_MAX];
+
+	/* Get string. 
+	 */
+	PEPOINTRIGHT( arg[0], &rhs );
+	(void) reduce_get_string( rc, &rhs, buf, FILENAME_MAX );
+
+	if( !heap_file_new( rc->heap, buf, out ) )
+		reduce_throw( rc );
 }
 
 /* Args for "math". 
@@ -336,7 +465,6 @@ static MathFn math_fn[] = {
 
 /* Do a math function (eg. sin, cos, tan).
  */
-/*ARGSUSED*/
 static void
 apply_math_call( Reduce *rc, 
 	const char *name, HeapNode **arg, PElement *out )
@@ -364,7 +492,8 @@ apply_math_call( Reduce *rc,
 		double a = PEGETREAL( &rhs );
 		double b = math_fn[i].rfn( a );
 
-		heap_real_new( rc->heap, b, out );
+		if( !heap_real_new( rc->heap, b, out ) )
+			reduce_throw( rc );
 	}
 	else if( PEISCOMPLEX( &rhs ) ) {
 		error_top( _( "Not implemented." ) );
@@ -391,18 +520,17 @@ typedef struct {
 /* Table of predicate functions ... name and implementation.
  */
 static PredicateFn predicate_fn[] = {
-	{ "is_image", &pe_isimage },
-	{ "is_bool", &pe_isbool },
-	{ "is_real", &pe_isreal },
-	{ "is_char", &pe_ischar },
-	{ "is_class", &pe_isclass },
-	{ "is_list", &pe_islist },
-	{ "is_complex", &pe_iscomplex }
+	{ "is_image", &pe_is_image },
+	{ "is_bool", &pe_is_bool },
+	{ "is_real", &pe_is_real },
+	{ "is_char", &pe_is_char },
+	{ "is_class", &pe_is_class },
+	{ "is_list", &pe_is_list },
+	{ "is_complex", &pe_is_complex }
 };
 
 /* Do a predicate function (eg. is_bool)
  */
-/*ARGSUSED*/
 static void
 apply_pred_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 {
@@ -433,7 +561,6 @@ static BuiltinTypeSpot *error_args[] = {
 
 /* Do "error".
  */
-/*ARGSUSED*/
 static void
 apply_error_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 {
@@ -458,7 +585,6 @@ static BuiltinTypeSpot *search_args[] = {
 
 /* Do "search".
  */
-/*ARGSUSED*/
 static void
 apply_search_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 {
@@ -474,7 +600,10 @@ apply_search_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 	if( !(fn = path_find_file( PATH_SEARCH, buf )) )
 		reduce_throw( rc );
 
-	heap_string_new( rc->heap, fn, out );
+	if( !heap_string_new( rc->heap, fn, out ) ) {
+		IM_FREE( fn );
+		reduce_throw( rc );
+	}
 	IM_FREE( fn );
 }
 
@@ -486,7 +615,6 @@ static BuiltinTypeSpot *print_args[] = {
 
 /* Do "print".
  */
-/*ARGSUSED*/
 static void
 apply_print_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 {
@@ -498,7 +626,89 @@ apply_print_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 	PEPOINTRIGHT( arg[0], &rhs );
 	itext_value_ev( rc, &buf, &rhs );
 
-	heap_string_new( rc->heap, buf_all( &buf ), out );
+	if( !heap_string_new( rc->heap, buf_all( &buf ), out ) )
+		reduce_throw( rc );
+}
+
+/* Args for "dir". 
+ */
+static BuiltinTypeSpot *dir_args[] = { 
+	&any_spot 
+};
+
+static void *
+dir_object_member( Symbol *sym, PElement *value, 
+	Reduce *rc, PElement *list )
+{
+	PElement t;
+
+	if( !heap_list_add( rc->heap, list, &t ) ||
+		!heap_string_new( rc->heap, IOBJECT( sym )->name, &t ) )
+		reduce_throw( rc );
+	(void) heap_list_next( list );
+
+	return( NULL );
+}
+
+static void *
+dir_object( Reduce *rc, PElement *list, PElement *instance, PElement *out )
+{
+	PElement p;
+
+	/* p walks down the list as we build it, list stays pointing at the
+	 * head ready to be written to out.
+	 */
+	p = *list;
+	heap_list_init( &p );
+	class_map( instance, (class_map_fn) dir_object_member, rc, &p );
+	PEPUTPE( out, list );
+
+	return( NULL );
+}
+
+static void *
+dir_scope( Symbol *sym, Reduce *rc, PElement *list )
+{
+	PElement t;
+
+	if( !heap_list_add( rc->heap, list, &t ) ||
+		!heap_string_new( rc->heap, IOBJECT( sym )->name, &t ) )
+		reduce_throw( rc );
+	(void) heap_list_next( list );
+
+	return( NULL );
+}
+
+/* Do "dir". 
+ */
+static void
+apply_dir_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
+{
+	PElement rhs;
+
+        PEPOINTRIGHT( arg[0], &rhs );
+
+	if( PEISCLASS( &rhs ) ) 
+		/* This is more complex than it looks. We have to walk a class
+		 * instance generating a list of member names, while not
+		 * destroying the instance as we go, in the case that out will
+		 * overwrite (rhs) arg[0].
+		 */
+		reduce_safe_pointer( rc, (reduce_safe_pointer_fn) dir_object,
+			&rhs, out, NULL, NULL );
+	else if( PEISSYMREF( &rhs ) ) {
+		Symbol *sym = PEGETSYMREF( &rhs );
+
+		if( is_scope( sym ) && sym->expr && sym->expr->compile ) {
+			PElement list;
+
+			list = *out;
+			heap_list_init( &list );
+
+			icontainer_map( ICONTAINER( sym->expr->compile ),
+				(icontainer_map_fn) dir_scope, rc, &list );
+		}
+	}
 }
 
 /* Args for "expand". 
@@ -509,7 +719,6 @@ static BuiltinTypeSpot *expand_args[] = {
 
 /* Do "expand".
  */
-/*ARGSUSED*/
 static void
 apply_expand_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 {
@@ -518,10 +727,57 @@ apply_expand_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 	char txt2[FILENAME_MAX];
 
 	PEPOINTRIGHT( arg[0], &rhs );
-	heap_get_string( &rhs, txt, FILENAME_MAX );
+	(void) reduce_get_string( rc, &rhs, txt, FILENAME_MAX );
 	expand_variables( txt, txt2 );
 
-	heap_string_new( rc->heap, txt2, out );
+	if( !heap_string_new( rc->heap, txt2, out ) )
+		reduce_throw( rc );
+}
+
+/* Args for "name2gtype". 
+ */
+static BuiltinTypeSpot *name2gtype_args[] = { 
+	&string_spot 
+};
+
+/* Do "name2gtype".
+ */
+static void
+apply_name2gtype_call( Reduce *rc, const char *name, 
+	HeapNode **arg, PElement *out )
+{
+	PElement rhs;
+	char txt[FILENAME_MAX];
+	int gtype;
+
+	PEPOINTRIGHT( arg[0], &rhs );
+	(void) reduce_get_string( rc, &rhs, txt, FILENAME_MAX );
+	gtype = g_type_from_name( txt );
+
+	if( !heap_real_new( rc->heap, gtype, out ) )
+		reduce_throw( rc );
+}
+
+/* Args for "gtype2name". 
+ */
+static BuiltinTypeSpot *gtype2name_args[] = { 
+	&real_spot 
+};
+
+/* Do "gtype2name".
+ */
+static void
+apply_gtype2name_call( Reduce *rc, const char *name, 
+	HeapNode **arg, PElement *out )
+{
+	PElement rhs;
+	int gtype;
+
+	PEPOINTRIGHT( arg[0], &rhs );
+	gtype = PEGETREAL( &rhs );
+
+	if( !heap_string_new( rc->heap, g_type_name( gtype ), out ) )
+		reduce_throw( rc );
 }
 
 /* All ip's builtin functions. 
@@ -529,12 +785,24 @@ apply_expand_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 static BuiltinInfo builtin_table[] = {
 	/* Other.
 	 */
+	{ "dir", FALSE, 1, &dir_args[0], &apply_dir_call },
 	{ "search", FALSE, 1, &search_args[0], &apply_search_call },
 	{ "error", FALSE, 1, &error_args[0], &apply_error_call },
 	{ "print", FALSE, 1, &print_args[0], &apply_print_call },
 	{ "expand", FALSE, 1, &expand_args[0], &apply_expand_call },
+	{ "name2gtype", FALSE, 1, &name2gtype_args[0], &apply_name2gtype_call },
+	{ "gtype2name", FALSE, 1, &gtype2name_args[0], &apply_gtype2name_call },
 	{ "_", FALSE, 1, &underscore_args[0], &apply_underscore_call },
-	
+
+	/* vips8 wrapper.
+	 */
+	{ "vips8_image_new", FALSE,  IM_NUMBER( image_new_args ), 
+		&image_new_args[0], apply_image_new_call },
+	{ "vips8_call", FALSE,  IM_NUMBER( call_args ), 
+		&call_args[0], apply_call_call },
+	{ "vips8_header_get", FALSE,  IM_NUMBER( header_get_args ), 
+		&header_get_args[0], apply_header_get_call },
+
 	/* Predicates.
 	 */
 	{ "is_image", FALSE, IM_NUMBER( pred_args ), 
@@ -555,8 +823,6 @@ static BuiltinInfo builtin_table[] = {
 		&is_instanceof_args[0], apply_is_instanceof_call },
 	{ "has_member", FALSE, IM_NUMBER( has_member_args ), 
 		&has_member_args[0], apply_has_member_call },
-	{ "get_member", FALSE, IM_NUMBER( get_member_args ), 
-		&get_member_args[0], apply_get_member_call },
 
 	/* List and complex projections.
 	 */
@@ -596,20 +862,40 @@ static BuiltinInfo builtin_table[] = {
 	{ "floor", TRUE, IM_NUMBER( math_args ), 
 		&math_args[0], apply_math_call },
 
+	/* Optional GSL funcs.
+	 */
+	{ "gammq", TRUE, IM_NUMBER( gammq_args ), 
+		&gammq_args[0], apply_gammq_call },
+
 	/* Constructors.
 	 */
 	{ "vips_image", FALSE,  
 		IM_NUMBER( image_args ), &image_args[0], apply_image_call },
+	{ "read", FALSE,  
+		IM_NUMBER( read_args ), &read_args[0], apply_read_call },
 };
 
-/* Make the _builtin toolkit and populate.
- */
+#ifdef HAVE_GSL
+static void
+builtin_gsl_error( const char *reason, const char *file, 
+	int line, int gsl_errno )
+{
+	error_top( _( "GSL library error." ) );
+	error_sub( "%s - (%s:%d) - %s", 
+		reason, file, line, gsl_strerror( gsl_errno ) );
+
+	reduce_throw( reduce_context );
+}
+#endif /*HAVE_GSL*/
+
 void
 builtin_init( void )
 {
 	Toolkit *kit;
         int i;
 
+	/* Make the _builtin toolkit and populate.
+	 */
 	kit = toolkit_new( main_toolkitgroup, "_builtin" );
 
         for( i = 0; i < IM_NUMBER( builtin_table ); i++ ) {
@@ -627,6 +913,12 @@ builtin_init( void )
 	filemodel_set_auto_load( FILEMODEL( kit ) );
 	filemodel_set_modified( FILEMODEL( kit ), FALSE );
 	kit->pseudo = TRUE;
+
+	/* Start up GSL, if we have it.
+	 */
+#ifdef HAVE_GSL
+	gsl_set_error_handler( builtin_gsl_error );
+#endif /*HAVE_GSL*/
 }
 
 /* Make a usage error.
@@ -651,7 +943,7 @@ builtin_usage( BufInfo *buf, BuiltinInfo *builtin )
 
 #ifdef DEBUG
 static void
-builtin_trace_args( Heap *hi, const char *name, int n, HeapNode **arg )
+builtin_trace_args( Heap *heap, const char *name, int n, HeapNode **arg )
 {
 	int i;
 	BufInfo buf;
