@@ -27,6 +27,55 @@
 
  */
 
+/* Member types we automate.
+ */
+typedef enum {
+	CLASSMODEL_MEMBER_INT,
+	CLASSMODEL_MEMBER_ENUM,		/* Like int, but extent has max value */
+	CLASSMODEL_MEMBER_BOOLEAN,
+	CLASSMODEL_MEMBER_DOUBLE,
+	CLASSMODEL_MEMBER_STRING,
+	CLASSMODEL_MEMBER_STRING_LIST,
+	CLASSMODEL_MEMBER_REALVEC_FIXED,/* Eg. Colour's triplet */
+	CLASSMODEL_MEMBER_MATRIX,
+	CLASSMODEL_MEMBER_OPTIONS,
+	CLASSMODEL_MEMBER_IMAGE
+} ClassmodelMemberType;
+
+/* A matrix value.
+ */
+typedef struct _MatrixValue {
+	double *coeff;			/* Base coeffs */
+	int width;			/* Size of matrix */
+	int height;
+} MatrixValue;
+
+/* An image value.
+ */
+typedef struct {
+	Imageinfo *ii;
+
+	/* Can get "changed" for reload if the file changes behind our backs.
+	 * Recalc the classmodel if this happens.
+	 */
+	guint file_changed_sid;
+	Classmodel *classmodel;
+} ImageValue;
+
+/* A member needing automation.
+ */
+typedef struct {
+	ClassmodelMemberType type;
+	void *details;			/* eg. the set of allowed options */
+	int extent;			/* Vector length, enum max, etc. */
+
+	const char *member_name;	/* Name as known in nip class defs */
+	const char *save_name;		/* As known in save files */
+	const char *user_name;		/* i18n'd name for dialogs */
+
+	guint offset;			/* Struct offset */
+} ClassmodelMember;
+
 #define TYPE_CLASSMODEL (classmodel_get_type())
 #define CLASSMODEL( obj ) \
 	(G_TYPE_CHECK_INSTANCE_CAST( (obj), TYPE_CLASSMODEL, Classmodel ))
@@ -55,6 +104,12 @@ struct _Classmodel {
 	 * and matrix). Used to set the filename for the "save" dialog.
 	 */
 	char *filename;
+
+	/* For things that have a pop-up window (eg. iimage, plot), the
+	 * position and size of the window.
+	 */
+	int window_x, window_y;
+	int window_width, window_height;
 };
 
 typedef struct _ClassmodelClass {
@@ -77,8 +132,17 @@ typedef struct _ClassmodelClass {
 
 	FileselFileType **filetype;
 	const char *filetype_pref;
+
+	ClassmodelMember *members;
+	int n_members;
+	void (*reset)( Classmodel * );
 } ClassmodelClass;
 
+void image_value_init( ImageValue *image, Classmodel *classmodel );
+void image_value_destroy( ImageValue *image );
+void image_value_set( ImageValue *image, Imageinfo *ii );
+void image_value_caption( ImageValue *value, BufInfo *buf );
+
 void *classmodel_get_instance( Classmodel *classmodel );
 void classmodel_graphic_save( Classmodel *classmodel, GtkWidget *parent );
 void classmodel_graphic_replace( Classmodel *classmodel, GtkWidget *parent );
@@ -90,3 +154,5 @@ GType classmodel_get_type( void );
 
 void classmodel_update( Classmodel *classmodel );
 void classmodel_set_edited( Classmodel *classmodel, gboolean edited );
+
+Classmodel *classmodel_new_classmodel( GType type, Rhs *rhs );
