@@ -29,6 +29,10 @@
 
 #include "ip.h"
 
+/* A lot of this file is just for debugging.
+#define DEBUG
+ */
+
 /* Dump a binary operator.
  */
 char *
@@ -115,9 +119,9 @@ decode_NodeType( NodeType tag )
 	case TAG_COMPLEX:	return( "TAG_COMPLEX" );
 	case TAG_CLASS:		return( "TAG_CLASS" );
 	case TAG_GEN:		return( "TAG_GEN" );
+	case TAG_FILE:		return( "TAG_FILE" );
 	case TAG_SHARED:	return( "TAG_SHARED" );
 	case TAG_REFERENCE:	return( "TAG_REFERENCE" );
-	case TAG_DOT:		return( "TAG_DOT" );
 
 	default:
 		assert( FALSE );
@@ -191,6 +195,12 @@ decode_SymbolType_user( SymbolType t )
 	}
 }
 
+/* From here on, just used for debugging.
+ */
+#ifdef DEBUG
+
+#warning "DEBUG on in dump.c"
+
 /* Do a tiny dump of a symbol. Just a few characters.
  */
 void *
@@ -227,8 +237,8 @@ dump_expr( Expr *expr )
 {
 	Symbol *sym = expr->sym;
 
-	printf( "expr (0x%x)->sym->name = \"%s\"\n", 
-		GPOINTER_TO_UINT( expr ), IOBJECT( sym )->name );
+	printf( "expr (%p)->sym->name = \"%s\"\n", 
+		expr, IOBJECT( sym )->name );
 	printf( "%s->row = %s\n", IOBJECT( sym )->name, 
 		expr->row?"(set)":"(not set)" );
 
@@ -268,8 +278,8 @@ dump_compile( Compile *compile )
 	BufInfo buf;
 	char str[100];
 
-	printf( "compile (0x%x)->sym->name = \"%s\"\n", 
-		GPOINTER_TO_UINT( compile ), IOBJECT( sym )->name );
+	printf( "compile (%p)->sym->name = \"%s\"\n", 
+		compile, IOBJECT( sym )->name );
 
 	printf( "%s->class = %s\n", 
 		IOBJECT( sym )->name, bool_to_char( compile->is_klass ) );
@@ -484,8 +494,6 @@ psymv( char *name )
 static void
 print_element( int nsp, EType type, void *arg )
 {
-	Imageinfo *ii;
-
 	switch( type ) {
 	case ELEMENT_NOVAL:
 		printf( "no value (%d)\n", GPOINTER_TO_INT( arg ) );
@@ -544,17 +552,12 @@ print_element( int nsp, EType type, void *arg )
 			((HeapStaticString*)arg)->text ); 
 		break;
 
-	case ELEMENT_ELIST:
-		printf( "empty-list []" ); 
+	case ELEMENT_MANAGED:
+		printf( "Managed* %p", arg ); 
 		break;
 
-	case ELEMENT_IMAGE:
-		ii = (Imageinfo*)arg;
-
-		if( ii && ii->im )
-			printf( "image \"%s\"",	ii->im->filename ); 
-		else
-			printf( "no-image" ); 
+	case ELEMENT_ELIST:
+		printf( "empty-list []" ); 
 		break;
 
 	default:
@@ -595,7 +598,6 @@ graph_heap( int nsp, HeapNode *hn )
 	switch( hn->type ) {
 	case TAG_APPL:
 	case TAG_CONS:
-	case TAG_DOT:
 		printf( "\n" );
 		printf( "%s", spc( nsp ) );
 		printf( "left:  " );
@@ -640,17 +642,17 @@ graph_heap( int nsp, HeapNode *hn )
 			GETRIGHT( GETRIGHT( hn ) )->body.num );
 		break;
 
+	case TAG_FILE:
+		printf( "list generator file=%s\n", 
+			MANAGEDFILE( GETLEFT( hn ) )->file->fname );
+		break;
+
 	case TAG_FREE:
 	default:
 		assert( FALSE );
 	}
 }
 
-void 
-graph_test( Heap *heap )
-{
-}
-
 /* Pretty-print a const.
  */
 static void
@@ -661,6 +663,9 @@ dump_parseconst( ParseConst *pc )
 		printf( "%G", pc->val.num );
 		break;
 
+	case PARSE_CONST_COMPLEX:
+		printf( "%Gj", pc->val.num );
+
 	case PARSE_CONST_STR:
 		printf( "\"%s\"", pc->val.str );
 		break;
@@ -831,3 +836,5 @@ dump_symbol_heap( Symbol *sym )
 		pgraph( &sym->expr->root );
 	printf( "\n" );
 }
+
+#endif /*DEBUG*/
