@@ -120,7 +120,7 @@ expressionview_refresh( vObject *vobject )
 #ifdef DEBUG
 	printf( "expressionview_refresh: " );
 	row_name_print( row );
-	printf( " (0x%x)\n", (unsigned int) vobject );
+	printf( " (%p)\n", vobject );
 #endif /*DEBUG*/
 
 	formula_set_edit( expressionview->formula, 
