@@ -58,10 +58,10 @@ filemodel_register( Filemodel *filemodel )
 			filemodel );
 
 #ifdef DEBUG
-		printf( "filemodel_register: %s \"%s\" (0x%x)\n",
+		printf( "filemodel_register: %s \"%s\" (%p)\n",
 			G_OBJECT_TYPE_NAME( filemodel ),
 			IOBJECT( filemodel )->name,
-			(unsigned int) filemodel );
+			filemodel );
 #endif /*DEBUG*/
 	}
 }
@@ -75,10 +75,10 @@ filemodel_unregister( Filemodel *filemodel )
 			filemodel );
 
 #ifdef DEBUG
-		printf( "filemodel_unregister: %s \"%s\" (0x%x)\n",
+		printf( "filemodel_unregister: %s \"%s\" (%p)\n",
 			G_OBJECT_TYPE_NAME( filemodel ),
 			IOBJECT( filemodel )->name,
-			(unsigned int) filemodel );
+			filemodel );
 #endif /*DEBUG*/
 	}
 }
@@ -152,11 +152,13 @@ filemodel_info( iObject *iobject, BufInfo *buf )
 		bool_to_char( filemodel->auto_load ) );
 }
 
+/* filename can be NULL for unset.
+ */
 void
 filemodel_set_filename( Filemodel *filemodel, const char *filename )
 {
-	if( filename && filemodel->filename != filename ) {
-		SETSTR( filemodel->filename, filename );
+	if( filemodel->filename != filename ) {
+		IM_SETSTR( filemodel->filename, filename );
 		iobject_changed( IOBJECT( filemodel ) );
 	}
 }
@@ -249,7 +251,7 @@ filemodel_save_all_xml( Filemodel *filemodel, const char *filename )
 	xmlDoc *xdoc;
 	char namespace[256];
 
-	if( !(xdoc = xmlNewDoc( "1.0" )) ) {
+	if( !(xdoc = xmlNewDoc( (xmlChar *) "1.0" )) ) {
 		error_top( _( "XML library error." ) );
 		error_sub( _( "model_save_filename: xmlNewDoc() failed" ) );
 		return( FALSE );
@@ -262,7 +264,8 @@ filemodel_save_all_xml( Filemodel *filemodel, const char *filename )
 	im_snprintf( namespace, 256, "%s/%d.%d.%d",
 		NAMESPACE, 
 		filemodel->major, filemodel->minor, filemodel->micro );
-	if( !(xdoc->children = xmlNewDocNode( xdoc, NULL, "root", NULL )) ||
+	if( !(xdoc->children = xmlNewDocNode( xdoc, 
+		NULL, (xmlChar *) "root", NULL )) ||
 		!set_sprop( xdoc->children, "xmlns", namespace ) ) {
 		error_top( _( "XML library error." ) );
 		error_sub( _( "model_save_filename: xmlNewDocNode() failed" ) );
@@ -278,9 +281,9 @@ filemodel_save_all_xml( Filemodel *filemodel, const char *filename )
 
 	prettify_tree( xdoc );
 
-	if( GPOINTER_TO_INT( call_string_filename( 
-		(call_string_fn) xmlSaveFile, 
-		filename, xdoc, NULL, NULL ) ) == -1 ) {
+	if( calli_string_filename( 
+		(calli_string_fn) xmlSaveFile, 
+			filename, xdoc, NULL, NULL ) == -1 ) {
 		error_top( _( "Save failed." ) );
 		error_sub( _( "Save of %s \"%s\" to file \"%s\" failed.\n%s" ),
 			G_OBJECT_TYPE_NAME( filemodel ), 
@@ -442,14 +445,15 @@ filemodel_load_all_xml( Filemodel *filemodel,
 	 */
 	if( !(xnode = xmlDocGetRootElement( state->xdoc )) ||
 		!xnode->nsDef ||
-		!isprefix( NAMESPACE, xnode->nsDef->href ) ) {
+		!is_prefix( NAMESPACE, (char *) xnode->nsDef->href ) ) {
 		error_top( _( "Load failed." ) );
 		error_sub( _( "Can't load XML file \"%s\", "
 			"it's not a %s save file." ), 
 			state->filename, PACKAGE );
 		return( FALSE );
 	}
-	if( sscanf( xnode->nsDef->href + strlen( NAMESPACE ) + 1, "%d.%d.%d",
+	if( sscanf( (char *) xnode->nsDef->href + strlen( NAMESPACE ) + 1, 
+		"%d.%d.%d",
 		&state->major, &state->minor, &state->micro ) != 3 ) {
 		error_top( _( "Load failed." ) );
 		error_sub( _( "Can't load XML file \"%s\", "
@@ -657,7 +661,7 @@ filemodel_inter_saveas_cb( iWindow *iwnd, void *client,
 	filesel_set_filetype( filesel, 
 		class->filetype, 
 		watch_int_get( main_watchgroup, class->filetype_pref, 0 ) );
-	idialog_set_parent( IDIALOG( filesel ), GTK_WIDGET( iwnd ) );
+	iwindow_set_parent( IWINDOW( filesel ), GTK_WIDGET( iwnd ) );
 	filesel_set_done( filesel, filemodel_inter_save_cb, filemodel );
 	idialog_set_notify( IDIALOG( filesel ), nfn, sys );
 	iwindow_build( IWINDOW( filesel ) );
@@ -680,16 +684,8 @@ filemodel_inter_save( iWindow *parent, Filemodel *filemodel )
 	if( filemodel->filename ) {
 		if( !filemodel_save_all( filemodel, filemodel->filename ) ) 
 			box_alert( GTK_WIDGET( parent ) );
-		else {
+		else 
 			filemodel_set_modified( filemodel, FALSE );
-			box_info( GTK_WIDGET( parent ), 
-				_( "Save successful." ),
-				_( "%s \"%s\" successfully saved "
-				"to file \"%s\"." ),
-				G_OBJECT_TYPE_NAME( filemodel ), 
-				NN( IOBJECT( filemodel )->name ),
-				filemodel->filename );
-		}
 	}
 	else 
 		filemodel_inter_saveas( parent, filemodel );
@@ -835,7 +831,7 @@ filemodel_inter_loadas_cb( iWindow *iwnd, void *client,
 	filesel_set_filetype( filesel, 
 		class->filetype, 
 		watch_int_get( main_watchgroup, class->filetype_pref, 0 ) );
-	idialog_set_parent( IDIALOG( filesel ), GTK_WIDGET( iwnd ) );
+	iwindow_set_parent( IWINDOW( filesel ), GTK_WIDGET( iwnd ) );
 	filesel_set_done( filesel, filemodel_inter_load_cb, filemodel );
 	idialog_set_notify( IDIALOG( filesel ), nfn, sys );
 	iwindow_build( IWINDOW( filesel ) );
@@ -908,14 +904,14 @@ filemodel_set_auto_load( Filemodel *filemodel )
 	 */
 	if( filemodel->filename && 
 		strstr( filemodel->filename, 
-			"share" IM_DIR_SEP_STR PACKAGE ) ) {
-		char *p = strrchr( filemodel->filename, IM_DIR_SEP );
+			"share" G_DIR_SEPARATOR_S PACKAGE ) ) {
+		char *p = strrchr( filemodel->filename, G_DIR_SEPARATOR );
 		char buf[FILENAME_MAX];
 
 		assert( p );
 
-		im_snprintf( buf, FILENAME_MAX, "$SAVEDIR" IM_DIR_SEP_STR
-			"start" IM_DIR_SEP_STR "%s", p + 1 );
+		im_snprintf( buf, FILENAME_MAX, "$SAVEDIR" G_DIR_SEPARATOR_S
+			"start" G_DIR_SEPARATOR_S "%s", p + 1 );
 		filemodel_set_filename( filemodel, buf );
 	}
 }
