@@ -76,6 +76,8 @@ static const char *tiff_suffs[] = { ".tif", ".tiff", NULL };
 static const char *jpeg_suffs[] = { ".jpg", ".jpeg", ".jpe", NULL };
 static const char *png_suffs[] = { ".png", NULL };
 static const char *ppm_suffs[] = { ".ppm", ".pgm", ".pbm", NULL };
+static const char *exr_suffs[] = { ".exr", NULL };
+static const char *analyze_suffs[] = { ".img", ".hdr", NULL };
 static const char *icc_suffs[] = { ".icc", ".icm", NULL };
 static const char *vips_suffs[] = { ".v", NULL };
 static const char *workspace_suffs[] = { ".ws", NULL };
@@ -84,9 +86,10 @@ static const char *mor_suffs[] = { ".mor", NULL };
 static const char *con_suffs[] = { ".con", NULL };
 static const char *mat_suffs[] = { ".mat", NULL };
 static const char *def_suffs[] = { ".def", NULL };
+static const char *csv_suffs[] = { ".csv", NULL };
 static const char *all_suffs[] = { "", NULL };
 
-static FileselFileType
+FileselFileType
         filesel_tfile_type = 
 		{ N_( "TIFF image files (*.tif, *.tiff)" ), tiff_suffs },
         filesel_jfile_type = 
@@ -97,6 +100,10 @@ static FileselFileType
 		{ N_( "VIPS image files (*.v)" ), vips_suffs },
         filesel_pfile_type = 
 		{ N_( "PPM image files (*.ppm, *.pgm, *.pbm)" ), ppm_suffs },
+        filesel_efile_type = 
+		{ N_( "OpenEXR image files (*.exr)" ), exr_suffs },
+        filesel_afile_type = 
+		{ N_( "Analyze image files (*.img, *.hdr)" ), analyze_suffs },
         filesel_wfile_type = 
 		{ N_( "Workspace files (*.ws)" ), workspace_suffs },
         filesel_rfile_type = 
@@ -111,6 +118,8 @@ static FileselFileType
 		{ N_( "Definition files (*.def)" ), def_suffs },
         filesel_ifile_type = 
 		{ N_( "ICC profiles (*.icc, *.icm)" ), icc_suffs },
+        filesel_sfile_type = 
+		{ N_( "CSV files (*.csv)" ), csv_suffs },
         filesel_allfile_type = 
 		{ N_( "All files (*)" ), all_suffs };
 
@@ -124,6 +133,8 @@ FileselFileType
         *filesel_type_image[] = { 
 		&filesel_vfile_type, &filesel_tfile_type, &filesel_jfile_type, 
 		&filesel_pfile_type, &filesel_nfile_type, 
+		&filesel_afile_type, &filesel_efile_type,
+		&filesel_sfile_type, 
 		&filesel_allfile_type, NULL 
 	},
         *filesel_type_matrix[] = { 
@@ -133,6 +144,8 @@ FileselFileType
         *filesel_type_mainw[] = { 
 		&filesel_vfile_type, &filesel_tfile_type, &filesel_jfile_type, 
 		&filesel_pfile_type, &filesel_nfile_type,
+		&filesel_afile_type, &filesel_efile_type,
+		&filesel_sfile_type, 
 		&filesel_xfile_type, &filesel_cfile_type, &filesel_rfile_type, 
 		&filesel_mfile_type, 
 		&filesel_wfile_type, 
@@ -141,6 +154,8 @@ FileselFileType
         *filesel_type_any[] = { 
 		&filesel_vfile_type, &filesel_tfile_type, &filesel_jfile_type, 
 		&filesel_pfile_type, &filesel_nfile_type,
+		&filesel_afile_type, &filesel_efile_type,
+		&filesel_sfile_type, 
 		&filesel_wfile_type, 
 		&filesel_dfile_type, 
 		&filesel_xfile_type, &filesel_cfile_type, &filesel_rfile_type, 
@@ -151,8 +166,8 @@ FileselFileType
 
 /* Is a file of type ... just look at the suffix.
  */
-static gboolean
-is_file_type( FileselFileType *type, const char *filename )
+gboolean
+is_file_type( const FileselFileType *type, const char *filename )
 {
 	const char **p;
 	const char *suf;
@@ -234,6 +249,11 @@ filesel_tiff_mode( char *out )
 		im_snprintf( buf, FILENAME_MAX, ":%d", IP_TIFF_JPEG_Q );
 		strcat( ctype, buf );
 	}
+	if( IP_TIFF_COMPRESSION == TIFF_COMPRESSION_DEFLATE ||
+		IP_TIFF_COMPRESSION == TIFF_COMPRESSION_LZW ) {
+		im_snprintf( buf, FILENAME_MAX, ":%d", IP_TIFF_PREDICTOR + 1 );
+		strcat( ctype, buf );
+	}
 
 	strcpy( ltype, decode_tiff_layout( IP_TIFF_LAYOUT ) );
 	if( IP_TIFF_LAYOUT == TIFF_LAYOUT_TILE ) {
@@ -254,24 +274,39 @@ filesel_tiff_mode( char *out )
 static void
 filesel_jpeg_mode( char *out )
 {
-	/*
+	char profile[FILENAME_MAX];
 
-		FIXME ... removed ICC profile embedding, far too dangerous,
-		we're bound to attach the wrong profile all the time
-		 
-		we need a dialog to pop up on each JPEG save with dangerous
-		options like this one, ditto for TIFF icc profile embedding
+	switch( IP_JPEG_ICC_PROFILE ) {
+	case 0:
+		/* Use embedded profile ... do nothing.
+		 */
+		strcpy( profile, "" );
 
-	char buf[FILENAME_MAX];
-	char buf2[FILENAME_MAX];
+		break;
 
-	im_strncpy( buf, IP_JPEG_ICC_PROFILE, FILENAME_MAX );
-        expand_variables( buf, buf2 );
-        nativeize_path( buf2 );
-	im_snprintf( out, 256, "%d,%s", IP_JPEG_Q, buf2 );
-	 */
+	case 1:
+	{
+		/* Embed from file.
+		 */
+		char buf[FILENAME_MAX];
+		char buf2[FILENAME_MAX];
 
-	im_snprintf( out, 256, "%d", IP_JPEG_Q );
+		im_strncpy( buf, IP_JPEG_ICC_PROFILE_FILE, FILENAME_MAX );
+		expand_variables( buf, buf2 );
+		nativeize_path( buf2 );
+		im_snprintf( profile, FILENAME_MAX, ",%s", buf2 );
+
+		break;
+	}
+	default:
+		/* Again, do nothing.
+		 */
+		strcpy( profile, "" );
+
+		break;
+	}
+
+	im_snprintf( out, 256, "%d%s", IP_JPEG_Q, profile );
 }
 
 /* Make a PNG save format string.
@@ -298,18 +333,34 @@ filesel_ppm_mode( char *out )
 	}
 }
 
+/* Make a CSV save format string.
+ */
+static void
+filesel_csv_mode( char *out )
+{
+	/* We have to escape ":" and "," characters in the separator string.
+	 */
+	char separator[256];
+
+	escape_mode( IP_CSV_SEPARATOR, separator, 256 );
+
+	im_snprintf( out, 256, "sep:%s", separator );
+}
+
 typedef void (*make_mode_fn)( char *buf );
 
 typedef struct {
 	FileselFileType *type;
+	const char *caption_filter;
 	make_mode_fn mode_fn;
 } FileselMode;
 
 static FileselMode filesel_mode_table[] = {
-	{ &filesel_jfile_type, filesel_jpeg_mode },
-	{ &filesel_nfile_type, filesel_png_mode },
-	{ &filesel_tfile_type, filesel_tiff_mode },
-	{ &filesel_pfile_type, filesel_ppm_mode }
+	{ &filesel_jfile_type, "JPEG", filesel_jpeg_mode },
+	{ &filesel_nfile_type, "PNG", filesel_png_mode },
+	{ &filesel_tfile_type, "TIFF", filesel_tiff_mode },
+	{ &filesel_sfile_type, "CSV", filesel_csv_mode },
+	{ &filesel_pfile_type, "PPM", filesel_ppm_mode }
 };
 
 /* Add our image save settings to the end of a filename.
@@ -327,10 +378,23 @@ filesel_add_mode( char *filename )
 			filesel_mode_table[i].mode_fn( mode );
 			im_snprintf( filename + l, MAX_STRSIZE - l, 
 				":%s", mode );
+
 			return;
 		}
 }
 
+static const char *
+filesel_get_filter( const char *filename )
+{
+	int i;
+
+	for( i = 0; i < IM_NUMBER( filesel_mode_table ); i++ )
+		if( is_file_type( filesel_mode_table[i].type, filename ) ) 
+			return( filesel_mode_table[i].caption_filter );
+
+	return( NULL );
+}
+
 static void
 filesel_destroy( GtkObject *object )
 {
@@ -451,7 +515,7 @@ filesel_dir_enter( Filesel *filesel )
 			browse_refresh( filesel->browse, dir );
 		filesel_space_update( filesel, dir );
 		if( !filesel->start_name )
-			SETSTR( filesel_last_dir, dir );
+			IM_SETSTR( filesel_last_dir, dir );
 
 		filesel->current_dir = dir;
 		dir = NULL;
@@ -499,7 +563,7 @@ filesel_get_filetype( Filesel *filesel )
 		name = gtk_file_filter_get_name( filter );
 
 		for( i = 0; filesel->type[i]; i++ ) 
-			if( strcmp( filesel->type[i]->name, name ) == 0 )
+			if( strcmp( _( filesel->type[i]->name ), name ) == 0 )
 				break;
 
 		assert( filesel->type[i] );
@@ -517,7 +581,7 @@ filesel_find_file_type( FileselFileType **type, const char *filename )
 
 	for( i = 0; type[i]; i++ ) 
 		for( j = 0; type[i]->suffixes[j]; j++ ) 
-			if( iscasepostfix( type[i]->suffixes[j], filename ) )
+			if( is_casepostfix( type[i]->suffixes[j], filename ) )
 				return( i );
 
 	return( -1 );
@@ -539,7 +603,7 @@ filesel_set_filetype_from_filename( Filesel *filesel, const char *name )
 
 	/* If we've not got a sensible filename, don't bother.
 	 */
-	if( (p = strrchr( name, IM_DIR_SEP )) &&
+	if( (p = strrchr( name, G_DIR_SEPARATOR )) &&
 		strspn( p + 1, " \n\t" ) == strlen( p + 1 ) ) 
 		return;
 
@@ -568,6 +632,10 @@ filesel_set_filename( Filesel *filesel, const char *name )
 	absoluteize_path( buf );
 	filesel_set_filetype_from_filename( filesel, buf );
 
+#ifdef DEBUG
+	printf( "filesel_set_filename: %s\n", buf ); 
+#endif /*DEBUG*/
+
 	/* set_filename() will only select existing files, we need to be able
 	 * to set any filename (eg. for increment filename), so we have to
 	 * set_current_name() as well.
@@ -579,6 +647,13 @@ filesel_set_filename( Filesel *filesel, const char *name )
 			GTK_FILE_CHOOSER( filesel->chooser ), 
 			im_skip_dir( buf ) );
 
+	/* The file isn't actually selected until we hit the main loop again.
+	 * If we don't loop here, the next gtk_file_chooser_get_filename()
+	 * will fail.
+	 */
+	while( g_main_context_iteration( NULL, FALSE ) )
+		;
+
 	filesel->start_name = TRUE;
 
 	return( TRUE );
@@ -589,9 +664,15 @@ filesel_set_filename( Filesel *filesel, const char *name )
 char *
 filesel_get_filename( Filesel *filesel )
 {
-	char *name = gtk_file_chooser_get_filename(
+	char *name;
+
+	name = gtk_file_chooser_get_filename(
 		GTK_FILE_CHOOSER( filesel->chooser ) );
 
+#ifdef DEBUG
+	printf( "filesel_get_filename: %s\n", name ); 
+#endif /*DEBUG*/
+
 	if( !name ) {
 		error_top( _( "Bad filename." ) );
 		error_sub( _( "No file selected." ) );
@@ -736,7 +817,7 @@ filesel_build( GtkWidget *widget )
 		
 		filesel->filter[i] = gtk_file_filter_new();
 
-		gtk_file_filter_set_name( filesel->filter[i], type->name );
+		gtk_file_filter_set_name( filesel->filter[i], _( type->name ) );
 
 		for( j = 0; type->suffixes[j]; j++ ) {
 			buf_init_static( &patt, buf, FILENAME_MAX );
@@ -890,75 +971,6 @@ filesel_refresh_all( void )
         (void) slist_map( filesel_all, (SListMapFn) filesel_refresh, NULL );
 }
 
-/* Back from the user function ... unset the hourglass, and update.
- */
-static void
-filesel_trigger2( void *sys, iWindowResult result )
-{
-	iWindowSusp *susp = (iWindowSusp *) sys;
-	Filesel *filesel = FILESEL( susp->client );
-
-	/* Back from the user function ... restore the pointer.
-	 */
-	set_pointer();
-
-	/* If this is a save, assume that there is now a new file, 
-	 * and ask all fsb's to update.
-	 */
-	if( filesel->save && result != IWINDOW_ERROR )
-		filesel_refresh_all();
-
-	if( result != IWINDOW_TRUE ) {
-		/* Failure ... bomb out.
-		 */
-		iwindow_susp_return( susp, result );
-		return;
-	}
-
-	/* Increment the filename, if required.
-	 */
-	if( filesel->incr ) {
-		filesel_increment_filename( filesel );
-		filesel_refresh( filesel );
-	}
-
-	/* Success!
-	 */
-	iwindow_susp_return( susp, result );
-}
-
-/* Start of user done ... shut down our suspension, and set the hglass.
- */
-static void
-filesel_trigger( iWindow *iwnd, Filesel *filesel, 
-	iWindowNotifyFn nfn, void *sys )
-{
-	/* Suspend the callback for a bit.
-	 */
-	iWindowSusp *susp = iwindow_susp_new( NULL, iwnd, filesel, nfn, sys );
-
-	/* If there's a filetype pref, update it.
-	 */
-	if( filesel->type_pref )
-		prefs_set( filesel->type_pref, 
-			"%d", filesel_get_filetype( filesel ) );
-
-	set_hourglass();
-	filesel->done_cb( IWINDOW( filesel ), 
-		filesel->client, filesel_trigger2, susp );
-}
-
-/* OK in "file exists" yesno ... trigger the user callback.
- */
-static void
-filesel_done_yesno( iWindow *iwnd, 
-	void *client, iWindowNotifyFn nfn, void *sys )
-{
-	Filesel *filesel = FILESEL( client );
-
-	filesel_trigger( iwnd, filesel, nfn, sys );
-}
-
 static void
 filesel_init( Filesel *filesel )
 {
@@ -986,6 +998,7 @@ filesel_init( Filesel *filesel )
 		iwindow_true_cb, NULL, NULL, NULL );
 	idialog_set_pinup( IDIALOG( filesel ), TRUE );
 	idialog_set_nosep( IDIALOG( filesel ), TRUE );
+	idialog_set_button_focus( IDIALOG( filesel ), FALSE );
 	idialog_set_help_tag( IDIALOG( filesel ), "sec:loadsave" );
 
 	filesel_all = g_slist_prepend( filesel_all, filesel );
@@ -1029,6 +1042,132 @@ filesel_set_done( Filesel *filesel, iWindowFn done_cb, void *client )
 	filesel->client = client;
 }
 
+/* Back from the user function ... unset the hourglass, and update.
+ */
+static void
+filesel_trigger2( void *sys, iWindowResult result )
+{
+	iWindowSusp *susp = (iWindowSusp *) sys;
+	Filesel *filesel = FILESEL( susp->client );
+
+	/* Back from the user function ... restore the pointer.
+	 */
+	set_pointer();
+
+	/* If this is a save, assume that there is now a new file, 
+	 * and ask all fsb's to update.
+	 */
+	if( filesel->save && result != IWINDOW_ERROR )
+		filesel_refresh_all();
+
+	if( result != IWINDOW_TRUE ) {
+		/* Failure ... bomb out.
+		 */
+		iwindow_susp_return( susp, result );
+		return;
+	}
+
+	/* Increment the filename, if required.
+	 */
+	if( filesel->incr ) {
+		filesel_increment_filename( filesel );
+		filesel_refresh( filesel );
+	}
+
+	/* Success!
+	 */
+	iwindow_susp_return( susp, result );
+}
+
+/* Start of user done ... shut down our suspension, and set the hglass.
+ */
+static void
+filesel_trigger( Filesel *filesel, iWindow *iwnd, 
+	iWindowNotifyFn nfn, void *sys )
+{
+	/* Suspend the callback for a bit.
+	 */
+	iWindowSusp *susp = iwindow_susp_new( NULL, iwnd, filesel, nfn, sys );
+
+	/* If there's a filetype pref, update it.
+	 */
+	if( filesel->type_pref )
+		prefs_set( filesel->type_pref, 
+			"%d", filesel_get_filetype( filesel ) );
+
+	set_hourglass();
+	filesel->done_cb( IWINDOW( filesel ), 
+		filesel->client, filesel_trigger2, susp );
+}
+
+static void
+filesel_prefs_ok_cb( iWindow *iwnd, void *client,
+	iWindowNotifyFn nfn, void *sys )
+{
+	Filesel *filesel = FILESEL( client );
+
+	/* Force a recalc, in case we've changed the autorecalc 
+	 * settings. Also does a scan on any widgets.
+	 */
+	symbol_recalculate_all_force( TRUE );
+
+	filesel_trigger( filesel, iwnd, nfn, sys );
+}
+
+static void
+filesel_prefs( Filesel *filesel, iWindow *iwnd, 
+	const char *caption_filter, 
+	iWindowNotifyFn nfn, void *sys )
+{
+	Prefs *prefs;
+
+	if( !(prefs = prefs_new( caption_filter )) ) {
+		nfn( sys, IWINDOW_ERROR );
+		return;
+	}
+
+	/* Expands to (eg.) "TIFF Save Preferences".
+	 */
+	iwindow_set_title( IWINDOW( prefs ), 
+		_( "%s Save Preferences" ), caption_filter );
+	iwindow_set_parent( IWINDOW( prefs ), GTK_WIDGET( iwnd ) );
+	idialog_set_callbacks( IDIALOG( prefs ), 
+		iwindow_true_cb, NULL, NULL, filesel );
+	idialog_add_ok( IDIALOG( prefs ), filesel_prefs_ok_cb, GTK_STOCK_SAVE );
+	idialog_set_notify( IDIALOG( prefs ), nfn, sys );
+	iwindow_build( IWINDOW( prefs ) );
+
+	gtk_widget_show( GTK_WIDGET( prefs ) );
+}
+
+/* We have a filename and it's OK to overwrite. Is it a type for which we have
+ * to offer preferences?
+ */
+static void
+filesel_yesno_cb( iWindow *iwnd, void *client,
+	iWindowNotifyFn nfn, void *sys )
+{
+	Filesel *filesel = FILESEL( client );
+	char *filename;
+	const char *caption_filter;
+
+	if( filesel->save ) {
+		if( !(filename = filesel_get_filename( filesel )) ) 
+			nfn( sys, IWINDOW_ERROR );
+		else {
+			if( (caption_filter = filesel_get_filter( filename )) ) 
+				filesel_prefs( filesel, iwnd, caption_filter,
+					nfn, sys );
+			else
+				filesel_trigger( filesel, iwnd, nfn, sys );
+
+			g_free( filename );
+		}
+	}
+	else
+		filesel_trigger( filesel, iwnd, nfn, sys );
+}
+
 static void
 filesel_done_cb( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys )
 {
@@ -1039,12 +1178,14 @@ filesel_done_cb( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys )
 	printf( "filesel_done\n" );
 #endif /*DEBUG*/
 
-	if( (filename = filesel_get_filename( filesel )) ) {
+	if( !(filename = filesel_get_filename( filesel )) ) 
+		nfn( sys, IWINDOW_ERROR );
+	else {
 		/* File exists and we are saving? Do a yesno before we carry on.
 		 */
 		if( filesel->save && existsf( "%s", filename ) ) {
 			box_yesno( GTK_WIDGET( filesel ),
-				filesel_done_yesno, iwindow_true_cb, filesel,
+				filesel_yesno_cb, iwindow_true_cb, filesel,
 				nfn, sys,
 				_( "Overwrite" ),
 				_( "Overwrite file?" ),
@@ -1054,14 +1195,10 @@ filesel_done_cb( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys )
 		else
 			/* Just call the user function directly.
 			 */
-			filesel_trigger( iwnd, filesel, nfn, sys );
+			filesel_yesno_cb( iwnd, filesel, nfn, sys );
 
 		g_free( filename );
 	}
-	else
-		/* No file selected, or not a valid file. 
-		 */
-		nfn( sys, IWINDOW_ERROR );
 }
 
 void 
