@@ -37,8 +37,11 @@
 /* Our signals. 
  */
 enum {
+	EDIT,		
 	CHANGED,		
 	ACTIVATE,	
+	ENTER,	
+	LEAVE,	
 	LAST_SIGNAL
 };
 
@@ -89,7 +92,7 @@ formula_key_press_event_cb( GtkWidget *widget, GdkEventKey *ev,
 		/*
 
 			FIXME ... really we want to go back to the edit mode
-			set by our environment (eg. if we're in a how_formula
+			set by our environment (eg. if we're in a show_formula
 			workspace, should stay in show formula).
 
 		 */
@@ -154,6 +157,11 @@ formula_add_edit( Formula *formula )
 	gtk_container_add( GTK_CONTAINER( formula->entry_frame ), 
 		formula->entry );
 	gtk_widget_show( formula->entry );
+
+	/* Tell everyone we are in edit mode ... used to add to resettable,
+	 * for example.
+	 */
+	g_signal_emit( G_OBJECT( formula ), formula_signals[EDIT], 0 );
 }
 
 static void 
@@ -165,6 +173,7 @@ formula_refresh( Formula *formula )
 		formula_add_edit( formula );
                 gtk_widget_show( formula->entry_frame );
                 gtk_widget_hide( formula->right_label );
+		formula->changed = FALSE;
 	}
 	else {
                 gtk_widget_show( formula->right_label );
@@ -172,7 +181,10 @@ formula_refresh( Formula *formula )
                 IM_FREEF( gtk_widget_destroy, formula->entry_frame );
 	}
 
-	if( formula->entry && formula->expr ) {
+	/* Don't update the formula display if the user has edited the text ...
+	 * we shouldn't destroy their work.
+	 */
+	if( formula->entry && formula->expr && !formula->changed ) {
 		/* Make sure we don't trigger "changed" when we zap in new
 		 * text.
 		 */
@@ -190,7 +202,10 @@ formula_refresh( Formula *formula )
 	else
                 gtk_widget_hide( formula->left_label );
 	if( formula->value ) 
-		set_glabel( formula->right_label, "%s", formula->value );
+		/* Just display the first line of the formula ... it can be
+		 * mutiline for class members, for example.
+		 */
+		set_glabel1( formula->right_label, "%s", formula->value );
 }
 
 /* Change edit mode.
@@ -204,6 +219,17 @@ formula_set_edit( Formula *formula, gboolean edit )
 	}
 }
 
+/* Change sensitive mode.
+ */
+void 
+formula_set_sensitive( Formula *formula, gboolean sensitive )
+{
+	formula->sensitive = sensitive;
+
+	if( !formula->sensitive )
+		formula_set_edit( formula, FALSE );
+}
+
 /* Re-read the text. TRUE if we saw a change.
  */
 gboolean
@@ -226,7 +252,7 @@ formula_scan( Formula *formula )
 		if( expr && 
 			strspn( expr, WHITESPACE ) != strlen( expr ) &&
 			strcmp( expr, formula->expr ) != 0 ) {
-			SETSTR( formula->expr, expr );
+			IM_SETSTR( formula->expr, expr );
 			changed = TRUE;
 		}
 
@@ -243,9 +269,15 @@ formula_enter_notify_event( GtkWidget *widget, GdkEventCrossing *event )
 
 	event_widget = gtk_get_event_widget( (GdkEvent *) event );
 
-	if( event_widget == widget && event->detail != GDK_NOTIFY_INFERIOR ) 
+	if( event_widget == widget && event->detail != GDK_NOTIFY_INFERIOR ) {
 		gtk_widget_set_state( widget, GTK_STATE_PRELIGHT );
 
+		/* Tell people about our highlight change ... used to (eg.) set 
+		 * flash help.
+		 */
+		g_signal_emit( G_OBJECT( widget ), formula_signals[ENTER], 0 );
+	}
+
 	return( FALSE );
 }
 
@@ -256,9 +288,15 @@ formula_leave_notify_event( GtkWidget *widget, GdkEventCrossing *event )
 
 	event_widget = gtk_get_event_widget( (GdkEvent *) event );
 
-	if( event_widget == widget && event->detail != GDK_NOTIFY_INFERIOR ) 
+	if( event_widget == widget && event->detail != GDK_NOTIFY_INFERIOR ) {
 		gtk_widget_set_state( widget, GTK_STATE_NORMAL );
 
+		/* Tell people about our highlight change ... used to (eg.) set 
+		 * flash help.
+		 */
+		g_signal_emit( G_OBJECT( widget ), formula_signals[LEAVE], 0 );
+	}
+
 	return( FALSE );
 }
 
@@ -272,7 +310,7 @@ formula_button_press_event( GtkWidget *widget, GdkEventButton *event )
 	if( event->type == GDK_BUTTON_PRESS ) {
 		Formula *formula = FORMULA( widget );
 
-		if( event->button == 1 ) {
+		if( event->button == 1 && formula->sensitive ) {
 			if( !formula->edit ) {
 				formula_set_edit( formula, TRUE );
 				gtk_widget_grab_focus( formula->entry );
@@ -285,6 +323,12 @@ formula_button_press_event( GtkWidget *widget, GdkEventButton *event )
 	return( handled );
 }       
 
+static void
+formula_real_changed( Formula *formula )
+{
+	formula->changed = TRUE;
+}
+
 static void
 formula_class_init( FormulaClass *class )
 {
@@ -301,6 +345,13 @@ formula_class_init( FormulaClass *class )
 
 	/* Create signals.
 	 */
+	formula_signals[EDIT] = g_signal_new( "edit",
+		G_OBJECT_CLASS_TYPE( gobject_class ),
+		G_SIGNAL_RUN_FIRST,
+		G_STRUCT_OFFSET( FormulaClass, changed ),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0 );
 	formula_signals[CHANGED] = g_signal_new( "changed",
 		G_OBJECT_CLASS_TYPE( gobject_class ),
 		G_SIGNAL_RUN_FIRST,
@@ -315,18 +366,51 @@ formula_class_init( FormulaClass *class )
 		NULL, NULL,
 		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE, 0 );
+	formula_signals[ENTER] = g_signal_new( "enter",
+		G_OBJECT_CLASS_TYPE( gobject_class ),
+		G_SIGNAL_RUN_FIRST,
+		G_STRUCT_OFFSET( FormulaClass, enter ),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0 );
+	formula_signals[LEAVE] = g_signal_new( "leave",
+		G_OBJECT_CLASS_TYPE( gobject_class ),
+		G_SIGNAL_RUN_FIRST,
+		G_STRUCT_OFFSET( FormulaClass, leave ),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0 );
 
 	/* Init methods.
 	 */
+	class->changed = formula_real_changed;
 }
 
 static void
 formula_init( Formula *formula )
 {
+	/* How annoying! To avoid vertical resizes on edit/view toggles we
+	 * need to add differing amounts of padding to the label. 
+	 
+	  	FIXME ... get this from the style somehow
+	
+	 */
+#ifdef OS_WIN32
+	/* with either wimp theme or gtk default.
+	 */
+	const int vpadding = 7;
+#else /*!OS_WIN32*/
+	/* clearlooks
+	 */
+	const int vpadding = 8;
+#endif /*OS_WIN32*/
+
 	formula->caption = NULL;
 	formula->value = NULL;
 	formula->expr = NULL;
 	formula->edit = FALSE;
+	formula->sensitive = TRUE;
+	formula->changed = FALSE;
 	formula->entry_frame = NULL;
 
 	formula->hbox = gtk_hbox_new( FALSE, 12 );
@@ -335,14 +419,14 @@ formula_init( Formula *formula )
 
         formula->left_label = gtk_label_new( "" );
         gtk_misc_set_alignment( GTK_MISC( formula->left_label ), 0, 0.5 );
-        gtk_misc_set_padding( GTK_MISC( formula->left_label ), 2, 7 );
+        gtk_misc_set_padding( GTK_MISC( formula->left_label ), 2, vpadding );
 	gtk_box_pack_start( GTK_BOX( formula->hbox ), 
 		formula->left_label, FALSE, FALSE, 2 );
         gtk_widget_show( formula->left_label );
 
         formula->right_label = gtk_label_new( "" );
         gtk_misc_set_alignment( GTK_MISC( formula->right_label ), 0, 0.5 );
-        gtk_misc_set_padding( GTK_MISC( formula->right_label ), 2, 7 );
+        gtk_misc_set_padding( GTK_MISC( formula->right_label ), 2, vpadding );
 	gtk_box_pack_start( GTK_BOX( formula->hbox ), 
 		formula->right_label, TRUE, TRUE, 0 );
         gtk_widget_show( formula->right_label );
@@ -389,7 +473,7 @@ formula_set_caption( Formula *formula, const char *caption )
 	}
 	else if( caption && (!formula->caption || 
 		strcmp( caption, formula->caption ) != 0) ) {
-		SETSTR( formula->caption, caption );
+		IM_SETSTR( formula->caption, caption );
 		formula_refresh( formula );
 	}
 }
@@ -399,17 +483,22 @@ formula_set_value_expr( Formula *formula, const char *value, const char *expr )
 {
 	gboolean changed;
 
+#ifdef DEBUG
+	printf( "formula_set_value_expr: value=\"%s\", expr=\"%s\"\n",
+		value, expr );
+#endif /*DEBUG*/
+
 	changed = FALSE;
 
 	if( value && (!formula->value || 
 		strcmp( value, formula->value ) != 0) ) {
-		SETSTR( formula->value, value );
+		IM_SETSTR( formula->value, value );
 		changed = TRUE;
 	}
 
 	if( expr && (!formula->expr || 
 		strcmp( expr, formula->expr ) != 0) ) {
-		SETSTR( formula->expr, expr );
+		IM_SETSTR( formula->expr, expr );
 		changed = TRUE;
 	}
 
