@@ -97,9 +97,9 @@ heap_map( HeapNode *hn, heap_map_fn fn, void *a, void *b )
 	case TAG_REFERENCE:
 	case TAG_COMPLEX:
 	case TAG_GEN:
+	case TAG_FILE:
 	case TAG_CLASS:
 	case TAG_DOUBLE:
-	case TAG_DOT:
 		return( fn( hn, a, b ) );
 
 	case TAG_SHARED:
@@ -126,7 +126,6 @@ heap_check_free( Heap *heap )
 {
 	HeapNode *hn;
 	HeapBlock *hb;
-	GSList *p;
 
 	/* Clear all the DEBUG flags.
 	 */
@@ -160,16 +159,18 @@ heap_check_free( Heap *heap )
 				(hn->flgs & FLAG_DEBUG) );
 		}
 	}
-
-	/* Test itable list.
-	 */
-	for( p = heap->itable; p; p = p->next ) {
-		Imageinfo *ii = (Imageinfo *) p->data;
-
-		assert( ii );
-	}
 }
 
+#ifdef DEBUG_HEAP_GC
+static void
+heap_check_managed( void *key, void *value, Heap *heap )
+{
+	/* Validate pointer.
+	 */
+	(void) MANAGED( value );
+}
+#endif /*DEBUG_HEAP_GC*/
+
 /* Test for sanity. 
  */
 int
@@ -180,6 +181,7 @@ heap_sanity( Heap *heap )
 #ifdef DEBUG_HEAP_GC
 	heap_gc( heap );
 	heap_check_free( heap );
+	g_hash_table_foreach( heap->mtable, (GHFunc) heap_check_managed, heap );
 #endif /*DEBUG_HEAP_GC*/
 
 	return( 0 );
@@ -210,18 +212,30 @@ heapblock_free( HeapBlock *hb )
 	IM_FREE( hb );
 }
 
+static void 
+heap_dispose_print( void *key, void *value )
+{ 
+	Managed *managed = MANAGED( value );
+
+	iobject_print( IOBJECT( managed ) );
+}
+
 static void
 heap_dispose( GObject *gobject )
 {
 	Heap *heap = HEAP( gobject );
 
-	/* Force a garbage-collect to free any image temps.
+	/* Repeatedly close managed objects. Each close can trigger other
+	 * closes, so we need to loop until done.
 	 */
-	(void) heap_gc( heap );
+	managed_clear( heap );
+	while( managed_free_unused( heap ) )
+		;
 
-	/* Check all images are dead.
+	/* Check all managed objects are dead.
 	 */
-	slist_map( heap->itable, (SListMapFn) imageinfo_dump, NULL );
+	g_hash_table_foreach( heap->mtable, 
+		(GHFunc) heap_dispose_print, NULL );
 
 	IM_FREEF( g_source_remove, heap->gc_tid );
 
@@ -260,9 +274,12 @@ heap_finalize( GObject *gobject )
 
 	if( heap->hb )
 		heapblock_free( heap->hb );
+
 	IM_FREEF( g_hash_table_destroy, heap->emark );
 
-	IM_FREEF( g_slist_free, heap->rmark );
+	IM_FREEF( g_hash_table_destroy, heap->rmark );
+
+	IM_FREEF( g_hash_table_destroy, heap->mtable );
 
 	heap_all = g_slist_remove( heap_all, heap );
 
@@ -278,8 +295,7 @@ heap_info( iObject *iobject, BufInfo *buf )
 	if( heap->compile )
 		if( heap->compile->sym ) {
 			symbol_qualified_name( heap->compile->sym, buf );
-			buf_appendf( buf, "(0x%x) (sym)\n", 
-				GPOINTER_TO_UINT( heap->compile->sym ) ); 
+			buf_appendf( buf, "(%p) (sym)\n", heap->compile->sym ); 
 		}
 		else
 			buf_appendf( buf, "(compile, but no sym)\n" );
@@ -291,11 +307,11 @@ heap_info( iObject *iobject, BufInfo *buf )
 	buf_appendf( buf, "emark = %d pointers\n", 
 		g_hash_table_size( heap->emark ) );
 	buf_appendf( buf, "rmark = %d pointers\n", 
-		g_slist_length( heap->rmark ) );
+		g_hash_table_size( heap->rmark ) );
 	buf_appendf( buf, "ncells (cells allocated) = %d\n", heap->ncells );
 	buf_appendf( buf, "nfree (cells free at last GC) = %d\n", heap->nfree );
-	buf_appendf( buf, "itable (Imageinfo blocks) = %d pointers\n", 
-		g_slist_length( heap->itable ) );
+	buf_appendf( buf, "mtable (Managed blocks) = %d pointers\n", 
+		g_hash_table_size( heap->mtable ) );
 
 	IOBJECT_CLASS( parent_class )->info( iobject, buf );
 }
@@ -319,7 +335,7 @@ heapblock_empty( HeapBlock *hb )
 	PPUTLEFT( &hb->node[hb->sz - 1], ELEMENT_NODE, NULL );
 }
 
-/* Add another HeapBlock.
+/* Add another HeapBlock, if we can.
  */
 static gboolean
 heapblock_create( Heap *heap, int sz )
@@ -395,8 +411,8 @@ heap_init( Heap *heap )
 	heap->filled = FALSE;
 
 	heap->emark = g_hash_table_new( NULL, g_direct_equal );
-	heap->rmark = NULL;
-	heap->itable = NULL;
+	heap->rmark = g_hash_table_new( NULL, g_direct_equal );
+	heap->mtable = g_hash_table_new( NULL, g_direct_equal );
 	heap->statics = g_hash_table_new( g_str_hash, g_str_equal );
 
 	heap->gc_tid = 0;
@@ -503,24 +519,6 @@ heap_serial_new( Heap *heap )
 	return( heap->serial );
 }
 
-static void *
-heap_image_clear( Imageinfo *ii )
-{
-	ii->marked = FALSE;
-
-	return( NULL );
-}
-
-static void
-heap_mark_image( Imageinfo *ii, Heap *heap )
-{
-	/* Make sure it's on this heap's image table.
-	 */
-	assert( ii->heap == heap );
-
-	imageinfo_mark( ii );
-}
-
 /* Mark a tree. Avoid recursion because of the danger of C stack overflow on 
  * large heaps.
  */
@@ -553,23 +551,15 @@ heap_mark_tree( Heap *heap, HeapNode *hn )
 			case TAG_CLASS:
 			case TAG_APPL:
 			case TAG_CONS:
-				if( hn->rtype == ELEMENT_IMAGE ) {
-					Imageinfo *ii;
+				if( GETRT( hn ) == ELEMENT_MANAGED ) 
+					managed_mark( (Managed *) 
+						GETRIGHT( hn ) );
+				if( GETLT( hn ) == ELEMENT_MANAGED ) 
+					managed_mark( (Managed *) 
+						GETLEFT( hn ) );
 
-					if( (ii = (Imageinfo *) 
-						hn->body.ptrs.right) )
-						heap_mark_image( ii, heap );
-				}
-				if( hn->ltype == ELEMENT_IMAGE ) {
-					Imageinfo *ii;
-
-					if( (ii = (Imageinfo *) 
-						hn->body.ptrs.left) )
-						heap_mark_image( ii, heap );
-				}
-
-				if( hn->rtype == ELEMENT_NODE ) {
-					if( hn->ltype == ELEMENT_NODE ) {
+				if( GETRT( hn ) == ELEMENT_NODE ) {
+					if( GETLT( hn ) == ELEMENT_NODE ) {
 						pending = g_slist_prepend( 
 							pending, 
 							GETRIGHT( hn ) );
@@ -578,18 +568,22 @@ heap_mark_tree( Heap *heap, HeapNode *hn )
 					else
 						hn = GETRIGHT( hn );
 				}
-				else if( hn->ltype == ELEMENT_NODE ) 
+				else if( GETLT( hn ) == ELEMENT_NODE ) 
 					hn = GETLEFT( hn );
 
 				break;
 
+			case TAG_FILE:
+				assert( GETLT( hn ) == ELEMENT_MANAGED );
+				managed_mark( (Managed *) GETLEFT( hn ) );
+				break;
+
 			case TAG_DOUBLE:
-			case TAG_DOT:
 				break;
 
 			case TAG_SHARED:
 			case TAG_REFERENCE:
-				if( hn->ltype == ELEMENT_NODE ) 
+				if( GETLT( hn ) == ELEMENT_NODE ) 
 					hn = GETLEFT( hn );
 				break;
 
@@ -606,8 +600,8 @@ heap_mark_tree( Heap *heap, HeapNode *hn )
 static void *
 mark_pelement( PElement *base, Heap *heap )
 {
-	if( PEISIMAGE( base ) && PEGETII( base ) ) 
-		heap_mark_image( PEGETII( base ), heap );
+	if( PEISMANAGED( base ) )
+		managed_mark( MANAGED( PEGETVAL( base ) ) );
 	else if( PEISNODE( base ) ) 
 		heap_mark_tree( heap, PEGETVAL( base ) );
 
@@ -629,8 +623,9 @@ mark_element( void *key, void *value, Heap *heap )
 /* Mark a reduce context ... the heapnodes on the spine stack etc.
  */
 static void *
-mark_reduce( Reduce *rc, Heap *heap )
+mark_reduce( void *key, void *value, Heap *heap )
 {
+	Reduce *rc = (Reduce *) value;
 	int i;
 
 #ifdef DEBUG
@@ -643,27 +638,6 @@ mark_reduce( Reduce *rc, Heap *heap )
 	return( NULL );
 }
 
-/* Look over the image table, freeing unreferenced images.
- */
-static void *
-free_images( Imageinfo *ii )
-{
-	if( !ii->marked && !ii->count ) {
-#ifdef DEBUG
-		printf( "heap_gc: closing unreferenced image \"%s\"\n",
-			ii->im->filename );
-#endif /*DEBUG*/
-
-		imageinfo_destroy_heap( ii );
-
-		/* Indicate that we closed to cause a remap. 
-		 */
-		return( ii );
-	}
-
-	return( NULL );
-}
-
 /* Check for unreffed static strings.
  */
 static gboolean 
@@ -703,20 +677,19 @@ heap_gc( Heap *heap )
 	printf( "heap_gc: starting GC for heap %s\n", IOBJECT( heap )->name );
 #endif /*DEBUG*/
 
-	/* Clear marks on images. Nodes should all be clear already.
+	/* Clear marks on managed objects. Nodes should all be clear already.
 	 */
-	if( slist_map( heap->itable, (SListMapFn) heap_image_clear, NULL ) )
-		return( FALSE );
+	managed_clear( heap );
 
 	/* Free all unreffed static strings.
 	 */
 	g_hash_table_foreach_remove( heap->statics, 
 		(GHRFunc) heap_static_check, NULL );
 
-	/* All flgs should be clear, so just mark.
+	/* All flags should be clear, so just mark.
 	 */
 	g_hash_table_foreach( heap->emark, (GHFunc) mark_element, heap );
-	slist_map( heap->rmark, (SListMapFn) mark_reduce, heap );
+	g_hash_table_foreach( heap->rmark, (GHFunc) mark_reduce, heap );
 
 	/* And sweep up unmarked into new free list.
 	 */
@@ -751,11 +724,11 @@ heap_gc( Heap *heap )
 	heap->ncells = ncells;
 	heap->nfree = nfree;
 
-	/* Repeatedly close unused images. We can't map, since each close can
-	 * trigger other closes.
+	/* Close unused managed objects. It can (potentially) take a couple of
+	 * passes through mtable to free everything ... but we'll do more on
+	 * the next GC.
 	 */
-	while( slist_map( heap->itable, (SListMapFn) free_images, heap ) )
-		;
+	managed_free_unused( heap );
 
 #ifdef DEBUG
 	printf( "heap_gc: %d cells in %d blocks, %d in use\n",
@@ -815,15 +788,7 @@ heap_unregister_element( Heap *heap, Element *root )
 void
 heap_register_reduce( Heap *heap, Reduce *rc )
 {
-	/* Make sure we're on the GC list.
-	 */
-	if( !g_slist_find( heap->rmark, rc ) ) {
-		heap->rmark = g_slist_prepend( heap->rmark, rc );
-#ifdef DEBUG
-		printf( "heap_register_reduce: %d pointers\n",
-			g_slist_length( heap->rmark ) );
-#endif 
-	}
+	g_hash_table_insert( heap->rmark, rc, rc );
 }
 
 /* Unregister a reduce context.
@@ -831,16 +796,10 @@ heap_register_reduce( Heap *heap, Reduce *rc )
 void
 heap_unregister_reduce( Heap *heap, Reduce *rc )
 {
-	if( g_slist_find( heap->rmark, rc ) ) {
-		heap->rmark = g_slist_remove( heap->rmark, rc );
-#ifdef DEBUG
-		printf( "heap_unregister_reduce: %d pointers\n",
-			g_slist_length( heap->rmark ) );
-#endif 
-	}
+	g_hash_table_remove( heap->rmark, rc );
 }
 
-/* Allocate a new HeapNode.
+/* Allocate a new HeapNode ... long version. See NEWNODE() macro.
  */
 HeapNode *
 heap_getmem( Heap *heap ) 
@@ -866,13 +825,14 @@ heap_getmem( Heap *heap )
 	if( !heap_gc( heap ) )
 		return( NULL );
 
-	/* Is heap over 80% full? Add another heap block.
+	/* Is heap over x% full? Add another heap block if we can.
 	 */
 	pcused = 100 * (heap->ncells - heap->nfree) / heap->ncells;
-	if( pcused > 80 ) {
+	if( pcused > 70 ) {
 		if( !heapblock_create( heap, heap->rsz ) )
 			return( NULL );
 	}
+
 	if( !heap->free ) {
 		error_top( _( "Heap full." ) );
 		if( heap->compile ) {
@@ -893,6 +853,7 @@ heap_getmem( Heap *heap )
 	}
 
 	EXTRACTNODE( heap, hn );
+
 	return( hn );
 }
 
@@ -921,16 +882,6 @@ heap_real_new( Heap *heap, double in, PElement *out )
 	return( TRUE );
 }
 
-/* Write an image to an element.
- */
-gboolean
-heap_image_new( Heap *heap, Imageinfo *ii, PElement *out )
-{
-	PEPUTP( out, ELEMENT_IMAGE, ii );
-
-	return( TRUE );
-}
-
 /* Write an element to an element.
  */
 gboolean
@@ -988,34 +939,6 @@ heap_complex_new( Heap *heap, double rp, double ip, PElement *out )
 	return( TRUE );
 }
 
-gboolean
-heap_dot_tag_new( Heap *heap, const char *tag, Compile *compile, PElement *out )
-{
-	HeapNode *hn1;
-
-	if( NEWNODE( heap, hn1 ) )
-		return( FALSE );
-	hn1->type = TAG_DOT;
-	PPUT( hn1, ELEMENT_TAG, tag, ELEMENT_COMPILEREF, compile ); 
-	PEPUTP( out, ELEMENT_NODE, hn1 );
-
-	return( TRUE );
-}
-
-gboolean
-heap_dot_sym_new( Heap *heap, Symbol *sym, Compile *compile, PElement *out )
-{
-	HeapNode *hn1;
-
-	if( NEWNODE( heap, hn1 ) )
-		return( FALSE );
-	hn1->type = TAG_DOT;
-	PPUT( hn1, ELEMENT_SYMREF, sym, ELEMENT_COMPILEREF, compile ); 
-	PEPUTP( out, ELEMENT_NODE, hn1 );
-
-	return( TRUE );
-}
-
 /* Set list to [].
  */
 void
@@ -1048,6 +971,7 @@ heap_list_add( Heap *heap, PElement *list, PElement *data )
 }
 
 /* Move list on to the next RHS. list points at [], or pointer to next node.
+ * Used with heap_list_init()/heap_list_add() to build lists.
  */
 gboolean
 heap_list_next( PElement *list )
@@ -1108,6 +1032,36 @@ heap_appl_add( Heap *heap, PElement *base, PElement *parm )
 	return( TRUE );
 }
 
+/* Make a lazy file read node.
+ */
+gboolean
+heap_file_new( Heap *heap, const char *filename, PElement *out )
+{
+	Managedfile *managedfile;
+	HeapNode *hn;
+
+	if( !(managedfile = managedfile_new( heap, filename )) )
+		return( FALSE );
+
+	/* Make sure the managedfile survives a GC.
+	 */
+	MANAGED_REF( managedfile );
+
+	if( NEWNODE( heap, hn ) ) {
+		MANAGED_UNREF( managedfile );
+		return( FALSE );
+	}
+	hn->type = TAG_FILE;
+	PPUT( hn, 
+		ELEMENT_MANAGED, managedfile,
+		ELEMENT_ELIST, NULL );
+	PEPUTP( out, ELEMENT_NODE, hn );
+
+	MANAGED_UNREF( managedfile );
+
+	return( TRUE );
+}
+
 /* Make a string.
  */
 gboolean
@@ -1273,6 +1227,83 @@ heap_map_list( PElement *base, heap_map_list_fn fn, void *a, void *b )
 	return( NULL );
 }
 
+/* Iterate over a list. Move list on to the next tl, point data at the
+ * head of the current node, FALSE for []. 
+ */
+gboolean
+heap_get_list_next( PElement *list, PElement *data )
+{
+	Reduce *rc = reduce_context;
+
+	if( !reduce_pelement( rc, reduce_spine, list ) )
+		return( FALSE );
+
+	if( PEISFLIST( list ) ) {
+		HeapNode *hn = PEGETVAL( list );
+
+		PEPOINTRIGHT( hn, list );
+		PEPOINTLEFT( hn, data );
+
+		return( TRUE );
+	}
+	else
+		return( FALSE );
+}
+
+typedef struct _HeapMapDict {
+	heap_map_dict_fn fn;
+	void *a;
+	void *b;
+} HeapMapDict;
+
+static void *
+heap_map_dict_entry( PElement *head, HeapMapDict *map_dict )
+{
+	Reduce *rc = reduce_context;
+	char key[256];
+	PElement p1, p2;
+	void *result;
+
+	if( !reduce_pelement( rc, reduce_spine, head ) )
+		return( FALSE );
+	if( !PEISFLIST( head ) ) 
+		heap_error_typecheck( head, "heap_map_dict", "list" );
+	PEGETHD( &p1, head );
+	if( !heap_get_string( &p1, key, 256 ) )
+		return( FALSE );
+
+	PEGETTL( &p2, head );
+	if( !reduce_pelement( rc, reduce_spine, &p2 ) )
+		return( FALSE );
+	if( !PEISFLIST( &p2 ) ) 
+		heap_error_typecheck( &p2, "heap_map_dict", "list" );
+	PEGETHD( &p1, &p2 );
+	result = map_dict->fn( key, &p1, map_dict->a, map_dict->b );
+
+	PEGETTL( &p1, &p2 );
+	if( !reduce_pelement( rc, reduce_spine, &p1 ) )
+		return( FALSE );
+	if( !PEISELIST( &p1 ) ) 
+		heap_error_typecheck( &p1, "heap_map_dict", "[]" );
+
+	return( result );
+}
+
+/* Map over a list of ["key", value] pairs.
+ */
+void *
+heap_map_dict( PElement *base, heap_map_dict_fn fn, void *a, void *b )
+{
+	HeapMapDict map_dict;
+
+	map_dict.fn = fn;
+	map_dict.a = a;
+	map_dict.b = b;
+
+	return( heap_map_list( base, 
+		(heap_map_list_fn) heap_map_dict_entry, &map_dict, NULL ) );
+}
+
 /* Evaluate a PElement into a string buffer. 
  */
 gboolean
@@ -1407,24 +1438,24 @@ heap_get_matrix( PElement *base, double *buf, int n, int *xsize, int *ysize )
 }
 
 gboolean 
-heap_iselist( PElement *base, gboolean *out ) 
+heap_is_elist( PElement *base, gboolean *out ) 
 {
 	Reduce *rc = reduce_context;
 
 	REDUCE_CATCH_START( FALSE );
-	*out = reduce_iselist( rc, base );
+	*out = reduce_is_elist( rc, base );
 	REDUCE_CATCH_STOP; 
 
 	return( TRUE );
 }
 
 gboolean 
-heap_islist( PElement *base, gboolean *out ) 
+heap_is_list( PElement *base, gboolean *out ) 
 {
 	Reduce *rc = reduce_context;
 
 	REDUCE_CATCH_START( FALSE );
-	*out = reduce_islist( rc, base );
+	*out = reduce_is_list( rc, base );
 	REDUCE_CATCH_STOP; 
 
 	return( TRUE );
@@ -1434,72 +1465,72 @@ heap_islist( PElement *base, gboolean *out )
  * string and will take a while to eval.
  */
 gboolean 
-heap_isstring( PElement *base, gboolean *out ) 
+heap_is_string( PElement *base, gboolean *out ) 
 {
 	Reduce *rc = reduce_context;
 
 	REDUCE_CATCH_START( FALSE );
-	*out = reduce_isstring( rc, base );
+	*out = reduce_is_string( rc, base );
 	REDUCE_CATCH_STOP; 
 
 	return( TRUE );
 }
 
 gboolean 
-heap_isrealvec( PElement *base, gboolean *out ) 
+heap_is_realvec( PElement *base, gboolean *out ) 
 {
 	Reduce *rc = reduce_context;
 
 	REDUCE_CATCH_START( FALSE );
-	*out = reduce_isrealvec( rc, base );
+	*out = reduce_is_realvec( rc, base );
 	REDUCE_CATCH_STOP; 
 
 	return( TRUE );
 }
 
 gboolean 
-heap_ismatrix( PElement *base, gboolean *out ) 
+heap_is_matrix( PElement *base, gboolean *out ) 
 {
 	Reduce *rc = reduce_context;
 
 	REDUCE_CATCH_START( FALSE );
-	*out = reduce_ismatrix( rc, base );
+	*out = reduce_is_matrix( rc, base );
 	REDUCE_CATCH_STOP; 
 
 	return( TRUE );
 }
 
 gboolean
-heap_isclass( PElement *base, gboolean *out )
+heap_is_class( PElement *base, gboolean *out )
 {
 	Reduce *rc = reduce_context;
 
 	REDUCE_CATCH_START( FALSE );
-	*out = reduce_isclass( rc, base );
+	*out = reduce_is_class( rc, base );
 	REDUCE_CATCH_STOP; 
 
 	return( TRUE );
 }
 
 gboolean
-heap_isinstanceof_exact( const char *name, PElement *klass, gboolean *out )
+heap_is_instanceof_exact( const char *name, PElement *klass, gboolean *out )
 {
 	Reduce *rc = reduce_context;
 
 	REDUCE_CATCH_START( FALSE );
-	*out = reduce_isinstanceof_exact( rc, name, klass );
+	*out = reduce_is_instanceof_exact( rc, name, klass );
 	REDUCE_CATCH_STOP; 
 
 	return( TRUE );
 }
 
 gboolean
-heap_isinstanceof( const char *name, PElement *klass, gboolean *out )
+heap_is_instanceof( const char *name, PElement *klass, gboolean *out )
 {
 	Reduce *rc = reduce_context;
 
 	REDUCE_CATCH_START( FALSE );
-	*out = reduce_isinstanceof( rc, name, klass );
+	*out = reduce_is_instanceof( rc, name, klass );
 	REDUCE_CATCH_STOP; 
 
 	return( TRUE );
@@ -1677,7 +1708,6 @@ heap_copy( Heap *heap, Compile *compile, PElement *out )
 	case ELEMENT_COMB:
 	case ELEMENT_TAG:
 	case ELEMENT_ELIST:
-	case ELEMENT_IMAGE:
 	case ELEMENT_STATIC:
 		/* Copy value.
 		 */
@@ -1689,6 +1719,7 @@ heap_copy( Heap *heap, Compile *compile, PElement *out )
 		 */
 		break;
 
+	case ELEMENT_MANAGED:
 	default:
 		assert( FALSE );
 	}
@@ -1704,7 +1735,7 @@ heap_static_string_new( Heap *heap, const char *text )
 	if( !(string = g_hash_table_lookup( heap->statics, text )) ) {
 		PElement pe;
 
-		if( !(string = IM_NEW( NULL, HeapStaticString )) )
+		if( !(string = INEW( NULL, HeapStaticString )) )
 			return( NULL );
 
 		string->heap = heap;
@@ -1735,7 +1766,7 @@ heap_static_string_new( Heap *heap, const char *text )
 }
 
 void *
-heap_static_string_destroy( HeapStaticString *string )
+heap_static_string_unref( HeapStaticString *string )
 {
 	assert( g_hash_table_lookup( string->heap->statics, string->text ) ==
 		string );
@@ -1749,3 +1780,130 @@ heap_static_string_destroy( HeapStaticString *string )
 
 	return( NULL );
 }
+
+/* Try to make a gvalue from a heap object. 
+ */
+gboolean
+heap_ip_to_gvalue( PElement *in, GValue *out )
+{
+	Reduce *rc = reduce_context;
+
+	if( !reduce_pelement( rc, reduce_spine_strict, in ) )
+		return( FALSE );
+
+	if( PEISREAL( in ) ) {
+		g_value_init( out, G_TYPE_DOUBLE );
+		g_value_set_double( out, PEGETREAL( in ) );
+	}
+	else if( PEISBOOL( in ) ) {
+		g_value_init( out, G_TYPE_BOOLEAN );
+		g_value_set_boolean( out, PEGETBOOL( in ) );
+	}
+	else if( PEISCOMPLEX( in ) ) {
+		printf( "ip_to_gvalue: no complex gtype!\n" );
+		return( FALSE );
+	}
+	else if( PEISLIST( in ) ) {
+		gboolean result;
+
+		if( !heap_is_string( in, &result ) )
+			return( FALSE );
+		if( result ) {
+			char name[256];
+
+			if( !heap_get_string( in, name, 256 ) )
+				return( FALSE );
+			g_value_init( out, G_TYPE_STRING );
+			g_value_set_string( out, name );
+		}
+		else {
+			error_top( _( "Unimplemented list type." ) );
+			return( FALSE );
+		}
+	}
+	else if( PEISMANAGED( in ) && IS_MANAGEDGOBJECT( PEGETVAL( in ) ) ) {
+		g_value_init( out, G_TYPE_OBJECT );
+		g_value_set_object( out, 
+			MANAGEDGOBJECT( PEGETMANAGED( in ) )->object );
+	}
+	else
+		return( FALSE );
+
+	return( TRUE );
+}
+
+/* Try to make a heap object from a gvalue.
+ */
+gboolean
+heap_gvalue_to_ip( GValue *in, PElement *out )
+{
+	Reduce *rc = reduce_context;
+	Heap *heap = rc->heap;
+
+	if( G_VALUE_HOLDS_BOOLEAN( in ) ) {
+		PEPUTP( out, ELEMENT_BOOL, (int) g_value_get_boolean( in ) );
+	}
+	else if( G_VALUE_HOLDS_CHAR( in ) ) {
+		PEPUTP( out, ELEMENT_CHAR, (int) g_value_get_char( in ) );
+	}
+	else if( G_VALUE_HOLDS_UCHAR( in ) ) {
+		PEPUTP( out, ELEMENT_CHAR, (int) g_value_get_uchar( in ) );
+	}
+	else if( G_VALUE_HOLDS_INT( in ) ) {
+		if( !heap_real_new( heap, g_value_get_int( in ), out ) )
+			return( FALSE );
+	}
+	else if( G_VALUE_HOLDS_UINT( in ) ) {
+		if( !heap_real_new( heap, g_value_get_uint( in ), out ) )
+			return( FALSE );
+	}
+	else if( G_VALUE_HOLDS_LONG( in ) ) {
+		if( !heap_real_new( heap, g_value_get_long( in ), out ) )
+			return( FALSE );
+	}
+	else if( G_VALUE_HOLDS_ULONG( in ) ) {
+		if( !heap_real_new( heap, g_value_get_ulong( in ), out ) )
+			return( FALSE );
+	}
+	else if( G_VALUE_HOLDS_INT64( in ) ) {
+		if( !heap_real_new( heap, g_value_get_int64( in ), out ) )
+			return( FALSE );
+	}
+	else if( G_VALUE_HOLDS_UINT64( in ) ) {
+		if( !heap_real_new( heap, g_value_get_uint64( in ), out ) )
+			return( FALSE );
+	}
+	else if( G_VALUE_HOLDS_FLOAT( in ) ) {
+		if( !heap_real_new( heap, g_value_get_float( in ), out ) )
+			return( FALSE );
+	}
+	else if( G_VALUE_HOLDS_DOUBLE( in ) ) {
+		if( !heap_real_new( heap, g_value_get_double( in ), out ) )
+			return( FALSE );
+	}
+	else if( G_VALUE_HOLDS_ENUM( in ) ) {
+		if( !heap_real_new( heap, g_value_get_enum( in ), out ) )
+			return( FALSE );
+	}
+	else if( G_VALUE_HOLDS_STRING( in ) ) {
+		if( !heap_string_new( heap, g_value_get_string( in ), out ) )
+			return( FALSE );
+	}
+	else if( G_VALUE_HOLDS_OBJECT( in ) ) {
+		GObject *object;
+		Managed *managed;
+
+		object = g_value_get_object( in );
+		managed = MANAGED( managedgobject_new( heap, object ) );
+		PEPUTP( out, ELEMENT_MANAGED, managed );
+	}
+	else {
+		error_top( _( "Unimplemented type." ) );
+		error_sub( _( "Unable to convert %s to a nip type." ), 
+			G_VALUE_TYPE_NAME( in ) ); 
+
+		return( FALSE );
+	}
+
+	return( TRUE );
+}
