@@ -48,14 +48,18 @@ static iObjectClass *parent_class = NULL;
 
 static guint icontainer_signals[SIG_LAST] = { 0 };
 
-/* Number of children.
- */
 int
 icontainer_get_n_children( iContainer *icontainer )
 {
 	return( g_slist_length( icontainer->children ) );
 }
 
+GSList *
+icontainer_get_children( iContainer *icontainer )
+{
+	return( g_slist_copy( icontainer->children ) );
+}
+
 void *
 icontainer_map( iContainer *icontainer, icontainer_map_fn fn, void *a, void *b )
 {
@@ -312,17 +316,16 @@ icontainer_pos_renumber( iContainer *icontainer )
 #endif /*DEBUG_VERBOSE*/
 }
 
-static gint
+gint
 icontainer_name_compare( iContainer *a, iContainer *b )
 {
         return( strcasecmp( IOBJECT( a )->name, IOBJECT( b )->name ) );
 }
 
 void
-icontainer_name_sort( iContainer *icontainer )
+icontainer_custom_sort( iContainer *icontainer, GCompareFunc fn )
 {
-        icontainer->children = g_slist_sort( icontainer->children, 
-		(GCompareFunc) icontainer_name_compare );
+        icontainer->children = g_slist_sort( icontainer->children, fn );
 	icontainer_pos_renumber( icontainer );
 	iobject_changed( IOBJECT( icontainer ) );
 }
@@ -448,6 +451,8 @@ icontainer_finalize( GObject *gobject )
 
 	IM_FREEF( g_hash_table_destroy, icontainer->child_hash );
 
+	FREESID( icontainer->destroy_sid, icontainer );
+
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
 
@@ -483,10 +488,9 @@ icontainer_real_child_add( iContainer *parent, iContainer *child, int pos )
 	printf( "\tpos = %d\n", pos );
 #endif /*DEBUG*/
 
-        if( pos >= 0 ) {
+        if( pos >= 0 ) 
                 parent->children = g_slist_insert( parent->children,
                         child, pos );
-	}
         else
                 parent->children = g_slist_append( parent->children, child );
         child->parent = parent;
