@@ -128,10 +128,10 @@ imagemodel_dispose( GObject *gobject )
 	FREESID( imagemodel->conv_changed_sid, imagemodel->conv );
 	FREESID( imagemodel->conv_imageinfo_changed_sid, imagemodel->conv );
 	UNREF( imagemodel->conv );
-	IM_FREEF( imageinfo_destroy_nonheap, imagemodel->ink );
+	MANAGED_UNREF( imagemodel->ink );
 	IM_FREE( imagemodel->font_name );
 	IM_FREE( imagemodel->text );
-	IM_FREEF( imageinfo_destroy_nonheap, imagemodel->text_mask );
+	MANAGED_UNREF( imagemodel->text_mask );
 
 	G_OBJECT_CLASS( parent_class )->dispose( gobject );
 }
@@ -227,20 +227,19 @@ imagemodel_refresh_ink( Imagemodel *imagemodel, Imageinfo *ii )
 		ink_im->Type == main_im->Type ) 
 		return;
 
-	IM_FREEF( imageinfo_destroy_nonheap, imagemodel->ink );
+	MANAGED_UNREF( imagemodel->ink );
 
 	if( (imagemodel->ink = imageinfo_new_temp( 
 		main_imageinfogroup, 
 		reduce_context->heap, NULL, "t" )) ) {
-		imageinfo_dup_nonheap( imagemodel->ink );
+		MANAGED_REF( imagemodel->ink );
 		im_initdesc( imagemodel->ink->im, 
 			1, 1, main_im->Bands, 
 			main_im->Bbits, main_im->BandFmt, 
 			main_im->Coding, main_im->Type, 
 			1.0, 1.0, 0, 0 );
-		if( im_setupout( imagemodel->ink->im ) )
-			IM_FREEF( imageinfo_destroy_nonheap, 
-				imagemodel->ink );
+		if( im_setupout( imagemodel->ink->im ) ) 
+			MANAGED_UNREF( imagemodel->ink );
 	}
 
 	if( imagemodel->ink && imagemodel->ink->im && 
@@ -293,7 +292,7 @@ imagemodel_init( Imagemodel *imagemodel )
 	imagemodel->text = NULL;
 	imagemodel->text_mask = NULL;
 
-	imagemodel->show_convert = DISPLAY_CONVERT;
+	imagemodel->show_convert = DISPLAY_CONVERSION;
 	imagemodel->scale = 1.0;
 	imagemodel->offset = 0.0;
 	imagemodel->falsecolour = FALSE;
@@ -328,7 +327,7 @@ imagemodel_get_type( void )
 static void
 imagemodel_iimage_changed_cb( iImage *iimage, Imagemodel *imagemodel )
 {
-	conversion_set_image( imagemodel->conv, iimage->instance.ii );
+	conversion_set_image( imagemodel->conv, iimage->value.ii );
 }
 
 static void
@@ -354,7 +353,7 @@ imagemodel_link( Imagemodel *imagemodel, iImage *iimage )
 
 	/* Install image.
 	 */
-	conversion_set_image( imagemodel->conv, iimage->instance.ii );
+	conversion_set_image( imagemodel->conv, iimage->value.ii );
 
 	/* Set name ... handy for debugging.
 	 */
@@ -503,13 +502,13 @@ imagemodel_refresh_text( Imagemodel *imagemodel )
 		return( FALSE );
 	}
 
-	IM_FREEF( imageinfo_destroy_nonheap, imagemodel->text_mask );
+	MANAGED_UNREF( imagemodel->text_mask );
 
 	if( !(imagemodel->text_mask = imageinfo_new_temp( main_imageinfogroup, 
 			reduce_context->heap, NULL, "t" )) ) 
 		return( FALSE );
 
-	imageinfo_dup_nonheap( imagemodel->text_mask );
+	MANAGED_REF( imagemodel->text_mask );
 
 	if( !imageinfo_paint_text( imagemodel->text_mask, 
 		imagemodel->font_name, imagemodel->text, 
