@@ -37,7 +37,7 @@
 
 #include "ip.h"
 
-static iWindowClass *parent_class = NULL;
+static FloatwindowClass *parent_class = NULL;
 
 /* All the magnification menus we have.
  */
@@ -104,10 +104,10 @@ imageview_init( Imageview *iv )
 GtkType
 imageview_get_type( void )
 {
-	static GtkType imageview_type = 0;
+	static GtkType type = 0;
 
-	if( !imageview_type ) {
-		static const GtkTypeInfo sinfo = {
+	if( !type ) {
+		static const GtkTypeInfo info = {
 			"Imageview",
 			sizeof( Imageview ),
 			sizeof( ImageviewClass ),
@@ -118,10 +118,10 @@ imageview_get_type( void )
 			(GtkClassInitFunc) NULL,
 		};
 
-		imageview_type = gtk_type_unique( TYPE_IWINDOW, &sinfo );
+		type = gtk_type_unique( TYPE_FLOATWINDOW, &info );
 	}
 
-	return( imageview_type );
+	return( type );
 }
 
 static void
@@ -136,7 +136,7 @@ imageview_refresh_title( Imageview *iv )
 	 */
 	if( ws ) {
 		Conversion *conv = imagemodel->conv;
-		Imageinfo *ii = iimage->instance.ii;
+		Imageinfo *ii = iimage->value.ii;
 
 		BufInfo buf;
 		char txt[512];
@@ -144,7 +144,7 @@ imageview_refresh_title( Imageview *iv )
 		buf_init_static( &buf, txt, 512 );
 		row_qualified_name_relative( ws->sym, row, &buf );
 
-		if( ii && IOBJECT( ii )->name && ii->from_file )
+		if( ii && imageinfo_is_from_file( ii ) )
 			buf_appendf( &buf, " - %s", IOBJECT( ii )->name );
 
 		buf_appendf( &buf, " - %.0f%%", 
@@ -323,7 +323,7 @@ imageview_new_arrow4_action_cb( GtkAction *action, Imageview *iv )
 	}
 
 	col = sym->expr->row->top_col;
-	model_scrollto( MODEL( col ) );
+	model_scrollto( MODEL( col ), MODEL_SCROLL_BOTTOM );
 }
 
 static void
@@ -377,7 +377,7 @@ imageview_header_action_cb( GtkAction *action, Imageview *iv )
 	idialog_set_callbacks( IDIALOG( imageheader ), 
 		NULL, NULL, NULL, iv );
 	idialog_add_ok( IDIALOG( imageheader ), iwindow_true_cb, _( "OK" ) );
-	idialog_set_parent( IDIALOG( imageheader ), GTK_WIDGET( iv ) );
+	iwindow_set_parent( IWINDOW( imageheader ), GTK_WIDGET( iv ) );
 	iwindow_build( IWINDOW( imageheader ) );
 
 	gtk_widget_show( imageheader );
@@ -723,8 +723,6 @@ imageview_build( Imageview *iv, GtkWidget *vbox, iImage *iimage )
 	GtkWidget *frame;
 	GList *focus_chain;
 
-	int w, h; 
-
 	/* All the model parts for our set of views.
 	 */
 	iv->imagemodel = imagemodel_new( iimage );
@@ -752,7 +750,7 @@ imageview_build( Imageview *iv, GtkWidget *vbox, iImage *iimage )
 	gtk_action_group_add_radio_actions( iv->action_group,
 		imageview_mode_radio_actions, 
 			G_N_ELEMENTS( imageview_mode_radio_actions ), 
-		WORKSPACE_MODE_REGULAR,
+		IMAGEMODEL_SELECT,
 		G_CALLBACK( imageview_mode_action_cb ),
 		GTK_WINDOW( iv ) );
 	gtk_action_group_add_radio_actions( iv->action_group,
@@ -815,45 +813,25 @@ imageview_build( Imageview *iv, GtkWidget *vbox, iImage *iimage )
 
 	/* Position and size to restore?
 	 */
-	if( iimage->window_width != -1 ) {
-		GdkScreen *screen = gtk_widget_get_screen( GTK_WIDGET( iv ) );
-
-		/* We need to clip x/y against the desktop size ... we may be
-		 * loading a workspace made on a machine with a big screen on
-		 * a machine with a small screen.
-
-		 	FIXME ... we could only clip if the window will be
-			completely off the screen? ie. ignore
-			iimage->window_width etc.
-
+	if( CLASSMODEL( iimage )->window_width != -1 ) {
+		/* Floatwindow will set pos/size.
 		 */
-		int window_x = IM_CLIP( 0, iimage->window_x, 
-			gdk_screen_get_width( screen ) - iimage->window_width );
-		int window_y = IM_CLIP( 0, iimage->window_y, 
-			gdk_screen_get_height( screen ) -
-				iimage->window_height );
-
-		gtk_widget_set_uposition( GTK_WIDGET( iv ), 
-			window_x, window_y );
-
-		gtk_window_set_default_size( GTK_WINDOW( iv ),
-			IM_MIN( iimage->window_width, 	
-				gdk_screen_get_width( screen ) ),
-			IM_MIN( iimage->window_height,
-				gdk_screen_get_height( screen ) ) );
-
 		iv->imagemodel->show_status = iimage->show_status;
 		iv->imagemodel->show_paintbox = iimage->show_paintbox;
 		iv->imagemodel->show_convert = iimage->show_convert;
 		iv->imagemodel->show_rulers = iimage->show_rulers;
 		iv->imagemodel->scale = iimage->scale;
 		iv->imagemodel->offset = iimage->offset;
+		iv->imagemodel->falsecolour = iimage->falsecolour;
+		iv->imagemodel->type = iimage->type;
 
-		imagepresent_set_mag_pos( iv->ip, 
-			iimage->image_mag,
-			iimage->image_left, iimage->image_top );
+		/* Our caller must call imagepresent_set_mag_pos() after
+		 * _show(). Not accurate if we set it here.
+		 */
 	}
 	else {
+		int w, h; 
+
 		/* Set initial size. This is really hard to do right :-( These
 		 * magic numbers will break with different themes.
 
@@ -861,11 +839,17 @@ imageview_build( Imageview *iv, GtkWidget *vbox, iImage *iimage )
 			calculate border size, then set default size and map?
 			yuk!
 
+			the magic numbers here are hard to derive, there are 
+			many, many widgets piled up together to make this 
+			window
+
+			last set correctly for clearlooks
+
 		 */
 		w = IM_MIN( IMAGE_WINDOW_WIDTH, 
-			iv->imagemodel->conv->image.width + 16 );
+			iv->imagemodel->conv->image.width + 14 );
 		h = IM_MIN( IMAGE_WINDOW_HEIGHT, 
-			iv->imagemodel->conv->image.height + 41 );
+			iv->imagemodel->conv->image.height + 39 );
 		gtk_window_set_default_size( GTK_WINDOW( iv ), w, h );
 		conversion_set_mag( iv->imagemodel->conv, 1 );
 	}
@@ -917,21 +901,13 @@ imageview_popdown( iWindow *iwnd, void *client,
 	 * the widgets have to all still be extant.
 	 */
 
-	/* Note position/size for later reuse.
-	 */
-	iimage->window_width = GTK_WIDGET( iv )->allocation.width;
-	iimage->window_height = GTK_WIDGET( iv )->allocation.height;
-	iimage->image_mag = conv->mag;
-	gdk_window_get_root_origin( 
-		gtk_widget_get_toplevel( GTK_WIDGET( iv ) )->window, 
-		&iimage->window_x, &iimage->window_y );
-
 	/* Save the centre of the window in image cods.
 	 */
 	conversion_disp_to_im( conv, 
 		imagemodel->visible.left + imagemodel->visible.width / 2, 
 		imagemodel->visible.top + imagemodel->visible.height / 2,
 		&iimage->image_left, &iimage->image_top );
+	iimage->image_mag = conv->mag;
 
 	iimage->show_status = imagemodel->show_status;
 	iimage->show_paintbox = imagemodel->show_paintbox;
@@ -942,10 +918,14 @@ imageview_popdown( iWindow *iwnd, void *client,
 	 */
 	if( iimage->show_convert != imagemodel->show_convert ||
 		iimage->scale != imagemodel->scale ||
-		iimage->offset != imagemodel->offset ) {
+		iimage->offset != imagemodel->offset ||
+		iimage->falsecolour != imagemodel->falsecolour ||
+		iimage->type != imagemodel->type ) {
 		iimage->show_convert = imagemodel->show_convert;
 		iimage->scale = imagemodel->scale;
 		iimage->offset = imagemodel->offset;
+		iimage->falsecolour = imagemodel->falsecolour;
+		iimage->type = imagemodel->type;
 		iobject_changed( IOBJECT( iimage ) );
 	}
 
@@ -953,11 +933,13 @@ imageview_popdown( iWindow *iwnd, void *client,
 }
 
 static void
-imageview_link( Imageview *iv, iImage *iimage )
+imageview_link( Imageview *iv, iImage *iimage, GtkWidget *parent )
 {
 	iwindow_set_build( IWINDOW( iv ), 
 		(iWindowBuildFn) imageview_build, iimage, NULL, NULL );
 	iwindow_set_popdown( IWINDOW( iv ), imageview_popdown, NULL );
+	iwindow_set_parent( IWINDOW( iv ), parent );
+	floatwindow_link( FLOATWINDOW( iv ), CLASSMODEL( iimage ) );
 	iwindow_build( IWINDOW( iv ) );
 	slist_map( iimage->classmodels,
 		(SListMapFn) imageview_add_region, iv );
@@ -968,11 +950,11 @@ imageview_link( Imageview *iv, iImage *iimage )
 }
 
 Imageview *
-imageview_new( iImage *iimage )
+imageview_new( iImage *iimage, GtkWidget *parent )
 {
 	Imageview *iv = gtk_type_new( TYPE_IMAGEVIEW );
 
-	imageview_link( iv, iimage );
+	imageview_link( iv, iimage, parent );
 
 	/* This is odd ... we wouldn't normally _show() the widget in _new(),
 	 * but restoring the scroll position doesn't work unless the window is
@@ -980,7 +962,7 @@ imageview_new( iImage *iimage )
 	 */
 	gtk_widget_show( GTK_WIDGET( iv ) );
 
-	if( iimage->window_width != -1 ) 
+	if( CLASSMODEL( iimage )->window_width != -1 ) 
 		imagepresent_set_mag_pos( iv->ip, 
 			iimage->image_mag,
 			iimage->image_left, iimage->image_top );
@@ -992,9 +974,9 @@ imageview_new( iImage *iimage )
  * can be -ve
  */
 Imageview *
-imageview_new_area( iImage *iimage, Rect *area )
+imageview_new_area( iImage *iimage, Rect *area, GtkWidget *parent )
 {
-	Imageview *iv = imageview_new( iimage );
+	Imageview *iv = imageview_new( iimage, parent );
 	Imagemodel *imagemodel = iv->imagemodel;
 	Conversion *conv = imagemodel->conv;
 	int shrink_x, shrink_y, shrink;
