@@ -108,9 +108,9 @@ extern int statfs();
 #ifdef HAVE_SYS_MOUNT_H
 #include <sys/mount.h>
 #endif /*HAVE_SYS_MOUNT_H*/
-#ifdef HAVE_WINDOWS_H
+#ifdef OS_WIN32
 #include <windows.h>
-#endif /*HAVE_WINDOWS_H*/
+#endif /*OS_WIN32*/
 #ifdef HAVE_FFTW
 #include <fftw.h>
 #endif /*HAVE_FFTW*/
@@ -128,9 +128,32 @@ extern int statfs();
 #include <vips/history.h>
 #include <vips/time.h>
 
+/* Experimental vips8 links.
+ */
+#ifdef HAVE_VIPS8
+#include <vips8/vips.h>
+#else /*!HAVE_VIPS8*/
+/* Some typedefs so we can build code containing vips8 names without falling
+ * over.
+ */
+typedef void VipsImage;
+typedef void VipsObject;
+#define VIPS_OBJECT(O) (O)
+#define VIPS_IS_OBJECT(O) (O)
+#define VIPS_NN(O) (O)
+#endif /*HAVE_VIPS8*/
+
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 
+/* If we're not using GNU C, elide __attribute__ 
+ */
+#ifndef __GNUC__
+#  ifndef __attribute__
+#    define  __attribute__(x)  /*NOTHING*/
+#  endif
+#endif
+
 /* Our general widgets.
  */
 #include "formula.h"
@@ -152,18 +175,20 @@ extern int statfs();
 #define MAX_TRACE (1024)	/* Biggest thing we print in trace */
 #define MAX_SSTACK (40)		/* Scope stack for parser */
 #define VIPS_HOMEPAGE "http://www.vips.ecs.soton.ac.uk"
-#define NIP_DOCPATH "$VIPSHOME" IM_DIR_SEP_STR "share" IM_DIR_SEP_STR \
-	"doc" IM_DIR_SEP_STR PACKAGE IM_DIR_SEP_STR "html"
-#define VIPS_DOCPATH "$VIPSHOME" IM_DIR_SEP_STR "share" IM_DIR_SEP_STR \
-	"doc" IM_DIR_SEP_STR "vips" IM_DIR_SEP_STR "html"
+#define NIP_DOCPATH "$VIPSHOME" G_DIR_SEPARATOR_S "share" G_DIR_SEPARATOR_S \
+	"doc" G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "html"
+#define VIPS_DOCPATH "$VIPSHOME" G_DIR_SEPARATOR_S "share" G_DIR_SEPARATOR_S \
+	"doc" G_DIR_SEPARATOR_S "vips" G_DIR_SEPARATOR_S "html"
 #define IP_NAME PACKAGE "-" VERSION
 #define NAMESPACE VIPS_HOMEPAGE "/" "nip" 
 				/* XML namespace ... note, not nip2! */
 #define MAX_LINELENGTH (120)	/* Max chars we display of value */
 #define MAX_RECENT (10)		/* Number of recent items in file menu */
+#define NIP_COPYRIGHT "%s: (c) 2007 Imperial College, London"
 
 /* Our stock_ids.
  */
+#define STOCK_NEXT_ERROR "nip-next-error"
 #define STOCK_DROPPER "nip-dropper"
 #define STOCK_DUPLICATE "nip-duplicate"
 #define STOCK_PAINTBRUSH "nip-paintbrush"
@@ -205,11 +230,11 @@ extern int statfs();
 
 /* win32 adds '_'.
  */
-#ifdef HAVE_WINDOWS_H
+#ifdef OS_WIN32
 #define popen(b,m) _popen(b,m)
 #define pclose(f) _pclose(f)
 #define mktemp(f) _mktemp(f)
-#endif
+#endif /*OS_WIN32*/
 
 /* Fwd ref these.
  */
@@ -231,6 +256,10 @@ typedef struct _Heapmodel Heapmodel;
 typedef struct _iArrow iArrow;
 typedef struct _iImage iImage;
 typedef struct _Imagedisplay Imagedisplay;
+typedef struct _Managed Managed;
+typedef struct _Managedfile Managedfile;
+typedef struct _Managedgvalue Managedgvalue;
+typedef struct _Managedgobject Managedgobject;
 typedef struct _Imageinfo Imageinfo;
 typedef struct _Imagepresent Imagepresent;
 typedef struct _Imagemodel Imagemodel;
@@ -254,6 +283,11 @@ typedef struct _Rhsview Rhsview;
 typedef struct _Row Row;
 typedef struct _Rowview Rowview;
 typedef struct _Statusview Statusview;
+typedef struct _Plotstatus Plotstatus;
+typedef struct _Plot Plot;
+typedef struct _Plotwindow Plotwindow;
+typedef struct _Plotpresent Plotpresent;
+typedef struct _Plotmodel Plotmodel;
 typedef struct _Subcolumn Subcolumn;
 typedef struct _Subcolumnview Subcolumnview;
 typedef struct _Symbol Symbol;
@@ -270,6 +304,8 @@ typedef struct _Workspace Workspace;
 typedef struct _Workspacegroup Workspacegroup;
 typedef struct _Workspacegroupview Workspacegroupview;
 typedef struct _Workspaceview Workspaceview;
+typedef struct _Prefworkspaceview Prefworkspaceview;
+typedef struct _Prefcolumnview Prefcolumnview;
 typedef struct _iText iText;
 typedef struct _Expression Expression;
 typedef struct _Mainw Mainw;
@@ -288,8 +324,8 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 
 /* Util stuff.
  */
-#include "gtkutil.h"
 #include "util.h"
+#include "gtkutil.h"
 #include "wild.h"
 #include "path.h"
 #include "iobject.h"
@@ -298,11 +334,16 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #include "idialog.h"
 #include "boxes.h"
 #include "filesel.h"
+#include "managed.h"
+#include "managedfile.h"
+#include "managedgvalue.h"
+#include "managedgobject.h"
 #include "imageinfo.h"
 #include "imagedisplay.h"
 #include "colourdisplay.h"
 #include "imagemodel.h"
 #include "imagepresent.h"
+#include "floatwindow.h"
 #include "imageview.h"
 #include "tslider.h"
 
@@ -345,6 +386,7 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #include "toolkitview.h"
 #include "toolview.h"
 #include "watch.h"
+#include "value.h"
 
 /* Per module includes, any order
  */
@@ -353,6 +395,7 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #include "program.h"
 #include "conversionview.h"
 #include "statusview.h"
+#include "plotstatus.h"
 #include "mainw.h"
 #include "browse.h"
 #include "builtin.h"
@@ -362,9 +405,12 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #include "main.h"
 #include "predicate.h"
 #include "slider.h"
+#include "clock.h"
 #include "pathname.h"
 #include "fontname.h"
 #include "group.h"
+#include "real.h"
+#include "vector.h"
 #include "colour.h"
 #include "number.h"
 #include "istring.h"
@@ -375,257 +421,37 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #include "numberview.h"
 #include "matrix.h"
 #include "matrixview.h"
+#include "plot.h"
+#include "plotview.h"
+#include "plotmodel.h"
+#include "plotpresent.h"
+#include "plotwindow.h"
 #include "option.h"
 #include "optionview.h"
 #include "iimage.h"
 #include "iregion.h"
 #include "iregiongroup.h"
 #include "iarrow.h"
-#include "iarrowview.h"
+#include "valueview.h"
 #include "sliderview.h"
 #include "pathnameview.h"
 #include "fontnameview.h"
-#include "groupview.h"
 #include "colourview.h"
 #include "iimageview.h"
 #include "iregionview.h"
 #include "iregiongroupview.h"
 #include "prefs.h"
+#include "prefworkspaceview.h"
+#include "prefcolumnview.h"
 #include "regionview.h"
 #include "itext.h"
 #include "itextview.h"
 #include "toggle.h"
 #include "toggleview.h"
 #include "vips_call.h"
+#include "call8.h"
+#include "parser.h"
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
 #endif /*WITH_DMALLOC*/
-
-/* Default show states.
- */
-#define DISPLAY_RULERS \
-	(watch_bool_get( main_watchgroup, "DISPLAY_RULERS", FALSE ))
-#define DISPLAY_STATUS \
-	(watch_bool_get( main_watchgroup, "DISPLAY_STATUS", FALSE ))
-#define DISPLAY_CONVERT \
-	(watch_bool_get( main_watchgroup, "DISPLAY_CONVERSION", FALSE ))
-
-/* Update children during paint.
- */
-#define PAINTBOX_RECOMP \
-	(watch_bool_get( main_watchgroup, "PAINTBOX_RECOMP", TRUE ))
-
-/* Help browser.
- */
-#define BOX_BROWSER (watch_string_get( main_watchgroup, "BROWSER", "mozilla" ))
-#define BOX_BROWSER_REMOTE (watch_string_get( main_watchgroup, \
-	"BROWSER_REMOTE", "-remote 'openURL(%s)'" ))
-
-/* Thumbnail size.
- */
-#define DISPLAY_THUMBNAIL \
-	(watch_int_get( main_watchgroup, "DISPLAY_THUMBNAIL", 64 ))
-
-/* File stuff.
- */
-#define PIN_FILESEL \
-	(watch_bool_get( main_watchgroup, "CALC_PIN_FILESEL", FALSE ))
-#define IP_JPEG_Q \
-	(watch_int_get( main_watchgroup, "JPEG_Q", 75 ))
-#define IP_PPM_MODE \
-	(watch_int_get( main_watchgroup, "PPM_MODE", 0 ))
-#define IP_PNG_COMPRESSION \
-	(watch_int_get( main_watchgroup, "PNG_COMPRESSION", 6 ))
-#define IP_PNG_INTERLACE \
-	(watch_int_get( main_watchgroup, "PNG_INTERLACE", 0 ))
-#define IP_TIFF_COMPRESSION \
-	(watch_int_get( main_watchgroup, "TIFF_COMPRESSION", 0 ))
-#define IP_TIFF_JPEG_Q \
-	(watch_int_get( main_watchgroup, "TIFF_JPEG_Q", 75 ))
-#define IP_TIFF_LAYOUT \
-	(watch_int_get( main_watchgroup, "TIFF_LAYOUT", 0 ))
-#define IP_TIFF_TILE_WIDTH \
-	(watch_int_get( main_watchgroup, "TIFF_TILE_WIDTH", 128 ))
-#define IP_TIFF_TILE_HEIGHT \
-	(watch_int_get( main_watchgroup, "TIFF_TILE_HEIGHT", 128 ))
-#define IP_TIFF_MULTI_RES \
-	(watch_int_get( main_watchgroup, "TIFF_MULTI_RES", 0 ))
-#define IP_TIFF_FORMAT \
-	(watch_int_get( main_watchgroup, "TIFF_FORMAT", 0 ))
-
-/* Autoreload. 
- */
-#define CALC_RELOAD (watch_bool_get( main_watchgroup, "CALC_RELOAD", FALSE ))
-
-/* Max chars we print. Limit to MAX_LINELENGTH to stop out-of-control stuff.
- */
-#define LINELENGTH \
-	(IM_MIN( MAX_LINELENGTH, \
-		watch_int_get( main_watchgroup, "CALC_LINELENGTH", 80 ) ))
-
-/* CPUs we work over.
- */
-#define VIPS_CPUS \
-	(watch_int_get( main_watchgroup, "VIPS_CPUS", 1 ))
-
-/* Bar prefs.
- */
-#define MAINW_TOOLBAR \
-	(watch_bool_get( main_watchgroup, "MAINW_TOOLBAR", TRUE ))
-#define MAINW_TOOLBAR_STYLE \
-	(watch_int_get( main_watchgroup, "MAINW_TOOLBAR_STYLE", 0 ))
-#define MAINW_STATUSBAR \
-	(watch_bool_get( main_watchgroup, "MAINW_STATUSBAR", TRUE ))
-#define MAINW_TOOLKITBROWSER \
-	(watch_bool_get( main_watchgroup, "MAINW_TOOLKITBROWSER", FALSE ))
-#define MAINW_PANE_POSITION \
-	(watch_int_get( main_watchgroup, "MAINW_PANE_POSITION", 400 ))
-
-/* Heap size.
- */
-#define MAX_HEAPSIZE (watch_int_get( main_watchgroup, "CALC_MAX_HEAP", 200000 ))
-
-/* Region dragging.
- */
-#ifdef NO_UPDATE
-#define CALC_RECOMP_REGION \
-	(watch_bool_get( main_watchgroup, "CALC_RECOMP_REGION", FALSE ))
-#else
-#define CALC_RECOMP_REGION \
-	(watch_bool_get( main_watchgroup, "CALC_RECOMP_REGION", TRUE ))
-#endif
-
-/* Slider dragging.
- */
-#define CALC_RECOMP_SLIDER \
-	(watch_bool_get( main_watchgroup, "CALC_RECOMP_SLIDER", FALSE ))
-
-/* Popup new objects.
- */
-#define POPUP_NEW_ROWS \
-	(watch_bool_get( main_watchgroup, "POPUP_NEW_ROWS", FALSE ))
-
-/* Draw LEDs rather than recolouring tally buttons.
- */
-#define CALC_DISPLAY_LED \
-	(watch_bool_get( main_watchgroup, "CALC_DISPLAY_LED", FALSE ))
-
-/* Autorecomp.
- */
-#define CALC_RECOMP (watch_bool_get( main_watchgroup, "CALC_RECOMP", TRUE ))
-
-/* Number of vips calls to memoise.
- */
-#define VIPS_HISTORY_MAX \
-	(watch_int_get( main_watchgroup, "VIPS_HISTORY_MAX", 200 ))
-
-/* Auto save wses.
- */
-#define AUTO_WS_SAVE \
-	(watch_bool_get( main_watchgroup, "CALC_AUTO_WS_SAVE", TRUE ))
-
-/* Image window geometry.
- */
-#define IMAGE_WINDOW_WIDTH \
-	(watch_int_get( main_watchgroup, "IMAGE_WINDOW_WIDTH", 600 ))
-#define IMAGE_WINDOW_HEIGHT \
-	(watch_int_get( main_watchgroup, "IMAGE_WINDOW_HEIGHT", 650 ))
-
-/* Default font.
- */
-#define PAINTBOX_FONT \
-	(watch_string_get( main_watchgroup, "PAINTBOX_FONT", "Sans 12" ))
-
-/* Max undo steps ... -1 == unlimited.
- */
-#define PAINTBOX_MAX_UNDO \
-	(watch_int_get( main_watchgroup, "PAINTBOX_MAX_UNDO", -1 ))
-
-/* Default image file type.
- */
-#define IMAGE_FILE_TYPE \
-	(watch_int_get( main_watchgroup, "IMAGE_FILE_TYPE", 0 ))
-
-/* Prefs we watch.
- */
-#define PATH_SEARCH (watch_path_get( main_watchgroup, "CALC_PATH_SEARCH", \
-	path_search_default ))
-#define PATH_START (watch_path_get( main_watchgroup, "CALC_PATH_START", \
-	path_start_default ))
-#define PATH_TMP (watch_string_get( main_watchgroup, "CALC_PATH_TMP", \
-	path_tmp_default ))
-
-/* Disassemble functions.
- */
-#define TRACE_FUNCTIONS \
-	(watch_bool_get( main_watchgroup, "CALC_TRACE_FUNCTIONS", FALSE ))
-
-/* Program window.
- */
-#define PROGRAM_PANE_POSITION \
-	(watch_double_get( main_watchgroup, "PROGRAM_PANE_POSITION", 200 ))
-
-/* Global variables from parse.y.
-
-	FIXME move to its own header ... parser.h?
-
- */
-
-/* Our input stream can be attached to either a string or a FILE. 
- * Keep track of the state of play here.
- */
-typedef struct {
-	iOpenFile *of;		/* Non-NULL if we read from a file */
-	char *str;		/* Non-NULL if we read from a string */
-	char *strpos;		/* Position in string */
-
-	char buf[MAX_STRSIZE];	/* Accumulate text of each definition here */
-	int bwp;		/* Write point in the above */
-	int bsp[MAX_SSTACK];	/* Start point stack */
-	int bspsp;		/* Stack pointer */
-
-	int lineno;		/* Current line number */
-	int charno;		/* Character in line */
-	int pcharno;		/* Characters in previous line */
-	int charpos;		/* Characters read by lex so far */
-
-	int oldchar;            /* unget buffer, -1 for no unget */
-} InputState;
-
-extern InputState input_state;
-
-/* Function declarations for parse.y.
- */
-void yyerror( const char *sub, ... );	
-extern int yyleng;			/* lex stuff */
-
-/* Lex gathers tokens here for workspace.c
- */
-extern BufInfo lex_text;
-
-/* Attach input for lex.
- */
-void attach_input_file( iOpenFile *of );
-void attach_input_string( const char *str );
-int ip_input( void );
-void ip_unput( int ch );
-gboolean is_EOF( void );
-
-/* Parse stuff.
- */
-
-/* Order and number important ... see table in parse_rhs()
- */
-typedef enum {
-	PARSE_RHS = 0,		/* eg. "a + b" */
-	PARSE_PARAMS,		/* eg. "a b = a + b" */
-	PARSE_SUPER 		/* eg. "fred c d" */
-} ParseRhsSyntax;
-
-gboolean parse_toplevel( Toolkit *kit, int pos );
-gboolean parse_onedef( Toolkit *kit, int pos );
-gboolean parse_rhs( Expr *expr, ParseRhsSyntax syntax );
-
-extern Symbol *last_top_sym;
-
