@@ -59,6 +59,15 @@ itext_finalize( GObject *gobject )
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
 
+static void
+itext_info( iObject *iobject, BufInfo *buf )
+{
+	iText *itext = ITEXT( iobject );
+
+	buf_appends( buf, _( "Formula" ) );
+	buf_appendf( buf, ": %s\n", NN( itext->formula ) );
+}
+
 /* Fwd ref this.
  */
 static gboolean itext_add_element( BufInfo *buf, 
@@ -105,6 +114,8 @@ itext_add_string( PElement *base, BufInfo *buf )
 	if( !reduce_pelement( rc, reduce_spine, base ) ) 
 		return( base );
 	if( PEISCHAR( base ) )
+		/* Don't escape chars in string mode.
+		 */
 		buf_appendf( buf, "%c", PEGETCHAR( base ) );
 	else {
 		/* Help! Fall back to ordinary item print.
@@ -122,6 +133,43 @@ itext_add_string( PElement *base, BufInfo *buf )
 	return( NULL );
 }
 
+/* Print a char ... we need to escape \n etc.
+ */
+static void
+itext_add_char( int ch, BufInfo *buf )
+{
+	char in[2];
+	char out[3];
+
+	in[0] = ch;
+	in[1] = '\0';
+	my_strecpy( out, in, FALSE );
+
+	buf_appends( buf, out );
+}
+
+/* Print a complex. 
+ */
+static void
+itext_add_complex( double rp, double ip, BufInfo *buf )
+{
+	if( PRINT_CARTESIAN ) 
+		buf_appendf( buf, "(%.12g, %.12g)", rp, ip );
+	else {
+		if( rp == 0 ) {
+			if( ip == 0 ) 
+				buf_appendf( buf, "0" );
+			else 
+				buf_appendf( buf, "%.12gj", ip );
+		}
+		else if( ip == 0 ) 
+			buf_appendf( buf, "%.12g", rp );
+		else
+			buf_appendf( buf, "%.12g + %.12gj", rp, ip );
+
+	}
+}
+
 /* Try to decompile.
  */
 static gboolean
@@ -138,15 +186,18 @@ itext_decompile_element( BufInfo *buf, PElement *base, gboolean top )
 		buf_appendf( buf, "%g", PEGETREAL( base ) );
 	else if( PEISBOOL( base ) ) 
 		buf_appends( buf, bool_to_char( PEGETBOOL( base ) ) );
-	else if( PEISCHAR( base ) ) 
-		buf_appendf( buf, "'%c'", (int) PEGETCHAR( base ) );
+	else if( PEISCHAR( base ) ) {
+		buf_appends( buf, "'" );
+		itext_add_char( (int) PEGETCHAR( base ), buf );
+		buf_appends( buf, "'" );
+	}
 	else if( PEISCOMPLEX( base ) ) 
-		buf_appendf( buf, "(%g,%g)", 
-			PEGETREALPART( base ), PEGETIMAGPART( base ) );
+		itext_add_complex( PEGETREALPART( base ), PEGETIMAGPART( base ),
+			buf );
 	else if( PEISELIST( base ) ) {
 		buf_appends( buf, "[ ]" );
 	}
-	else if( !heap_isstring( base, &result ) ) 
+	else if( !heap_is_string( base, &result ) ) 
 		/* Eval error.
 		 */
 		return( FALSE );
@@ -181,6 +232,20 @@ itext_decompile_element( BufInfo *buf, PElement *base, gboolean top )
 		if( !top )
 			buf_appends( buf, ")" );
 	}
+#ifdef HAVE_VIPS8
+	else if( PEISVIPSOBJECT( base ) ) {
+		VipsObject *object = PEGETVIPSOBJECT( base );
+
+		buf_appendf( buf, "<%s: %s>", object->name, object->caption );
+	}
+#endif /*HAVE_VIPS8*/
+	else if( PEISMANAGED( base ) ) {
+		Managed *managed = PEGETMANAGED( base );
+
+		buf_appends( buf, "<" );
+		iobject_info( IOBJECT( managed ), buf );
+		buf_appends( buf, ">" );
+	}
 	else if( PEISCLASS( base ) ) {
 		Compile *compile = PEGETCLASSCOMPILE( base );
 		PElement params;
@@ -264,15 +329,19 @@ itext_add_element( BufInfo *buf, PElement *base,
 		buf_appendf( buf, "%.7g", PEGETREAL( base ) );
 	else if( PEISBOOL( base ) ) 
 		buf_appends( buf, bool_to_char( PEGETBOOL( base ) ) );
-	else if( PEISCHAR( base ) ) 
-		buf_appendf( buf, "'%c'", (int) PEGETCHAR( base ) );
-	else if( PEISCOMPLEX( base ) ) 
-		buf_appendf( buf, "(%.12g,%.12g)", 
-			PEGETREALPART( base ), PEGETIMAGPART( base ) );
+	else if( PEISCHAR( base ) ) {
+		buf_appends( buf, "'" );
+		itext_add_char( (int) PEGETCHAR( base ), buf );
+		buf_appends( buf, "'" );
+	}
+	else if( PEISCOMPLEX( base ) ) {
+		itext_add_complex( PEGETREALPART( base ), PEGETIMAGPART( base ),
+			buf );
+	}
 	else if( PEISELIST( base ) ) {
 		buf_appends( buf, "[ ]" );
 	}
-	else if( !heap_isstring( base, &result ) ) 
+	else if( !heap_is_string( base, &result ) ) 
 		/* Eval error.
 		 */
 		return( FALSE );
@@ -300,31 +369,26 @@ itext_add_element( BufInfo *buf, PElement *base,
 			return( FALSE );
 		buf_appends( buf, "]" );
 	}
+#ifdef HAVE_VIPS8
+	else if( PEISVIPSOBJECT( base ) ) {
+		VipsObject *object = PEGETVIPSOBJECT( base );
+
+		buf_appendf( buf, "<%s %s, %s>", 
+			G_OBJECT_TYPE_NAME( object ),
+			object->name, object->caption );
+	}
+#endif /*HAVE_VIPS8*/
 	else if( PEISIMAGE( base ) ) {
-		Imageinfo *ii = PEGETII( base );
-		IMAGE *im = imageinfo_get( FALSE, ii );
+		buf_appendf( buf, "<" );
+		buf_appendi( buf, imageinfo_get( FALSE, PEGETII( base ) ) );
+		buf_appendf( buf, ">" );
+	}
+	else if( PEISMANAGED( base ) ) {
+		Managed *managed = PEGETMANAGED( base );
 
-		if( !im ) {
-			buf_appends( buf, _( "no image" ) );
-			return( TRUE );
-		}
-
-		/* Coded? Special warning.
-		 */
-		if( im->Coding != IM_CODING_NONE ) 
-			buf_appendf( buf, "%s, ", 
-				NN( im_Coding2char( im->Coding ) ) );
-
-		/* Format string expands to (eg.) "<2000x3000 128-bit
-	 	 * complex, 3 bands>"
-		 */
-		buf_appendf( buf, ngettext( 
-				"<%dx%d %s, %d band>",
-				"<%dx%d %s, %d bands>",
-				im->Bands ),
-			im->Xsize, im->Ysize,
-			decode_bandfmt( im->BandFmt ),
-			im->Bands );
+		buf_appends( buf, "<" );
+		iobject_info( IOBJECT( managed ), buf );
+		buf_appends( buf, ">" );
 	}
 	else if( PEISCLASS( base ) ) {
 		Compile *compile = PEGETCLASSCOMPILE( base );
@@ -523,12 +587,12 @@ itext_update_model( Heapmodel *heapmodel )
 		else 
 			new_formula = buf_all( &itext->decompile );
 
-		SETSTR( itext->formula_default, new_formula );
+		IM_SETSTR( itext->formula_default, new_formula );
 
 		/* Don't use itext_set_formula(), as we don't want to set
 		 * _modified or recomp.
 		 */
-		SETSTR( itext->formula, itext->formula_default );
+		IM_SETSTR( itext->formula, itext->formula_default );
 	}
 
 	return( HEAPMODEL_CLASS( parent_class )->update_model( heapmodel ) );
@@ -674,6 +738,12 @@ itext_load( Model *model,
 		state, parent, xnode ) );
 }
 
+static View *
+itext_view_new( Model *model, View *parent )
+{
+	return( itextview_new() );
+}
+
 static xmlNode *
 itext_save( Model *model, xmlNode *xnode )
 {
@@ -696,6 +766,7 @@ static void
 itext_class_init( iTextClass *class )
 {
 	GObjectClass *gobject_class = (GObjectClass *) class;
+	iObjectClass *iobject_class = (iObjectClass *) class;
 	iContainerClass *icontainer_class = (iContainerClass *) class;
 	ModelClass *model_class = (ModelClass *) class;
 	HeapmodelClass *heapmodel_class = (HeapmodelClass *) class;
@@ -709,9 +780,11 @@ itext_class_init( iTextClass *class )
 	 */
 	gobject_class->finalize = itext_finalize;
 
+	iobject_class->info = itext_info;
+
 	icontainer_class->parent_add = itext_parent_add;
 
-	model_class->view_new = itextview_new;
+	model_class->view_new = itext_view_new;
 	model_class->save = itext_save;
 	model_class->load = itext_load;
 
@@ -803,7 +876,7 @@ gboolean
 itext_set_formula( iText *itext, const char *formula )
 {
 	if( !itext->formula || strcmp( itext->formula, formula ) != 0 ) {
-		SETSTR( itext->formula, formula );
+		IM_SETSTR( itext->formula, formula );
 
 		heapmodel_set_modified( HEAPMODEL( itext ), TRUE );
 
