@@ -54,6 +54,102 @@ itextview_destroy( GtkObject *object )
 	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
 
+static void 
+itextview_refresh( vObject *vobject )
+{
+	iTextview *itextview = ITEXTVIEW( vobject );
+	iText *itext = ITEXT( VOBJECT( itextview )->iobject );
+	Row *row = HEAPMODEL( itext )->row;
+
+	const char *display;
+
+#ifdef DEBUG
+	printf( "itextview_refresh: " );
+	row_name_print( row );
+	printf( " (%p)\n", vobject );
+#endif /*DEBUG*/
+
+	/* Only reset edit mode if the text isn't being edited. We
+	 * don't want the user to lose work.
+	 */
+	if( !itextview->formula->edit ) 
+		switch( row->ws->mode ) {
+		case WORKSPACE_MODE_REGULAR:
+			formula_set_edit( itextview->formula, FALSE );
+			formula_set_sensitive( itextview->formula, TRUE );
+			break;
+
+		case WORKSPACE_MODE_FORMULA:
+			formula_set_edit( itextview->formula, TRUE );
+			formula_set_sensitive( itextview->formula, TRUE );
+			break;
+
+		case WORKSPACE_MODE_NOEDIT:
+			formula_set_edit( itextview->formula, FALSE );
+			formula_set_sensitive( itextview->formula, FALSE );
+			break;
+
+		default:
+			g_assert_not_reached();
+		}
+
+	/* We display the formula if this is a class ... we assume the members
+	 * and/or the graphic will represent the value.
+	 */
+	if( row->is_class )
+		display = itext->formula;
+	else
+		display = buf_all( &itext->value );
+
+	if( itextview->formula && itext->value.base )
+		formula_set_value_expr( itextview->formula,
+			display, itext->formula );
+
+	VOBJECT_CLASS( parent_class )->refresh( vobject );
+}
+
+static void
+itextview_link( View *view, Model *model, View *parent )
+{
+	iTextview *itextview = ITEXTVIEW( view );
+	iText *itext = ITEXT( model );
+	Row *row = HEAPMODEL( itext )->row;
+
+#ifdef DEBUG
+	printf( "itextview_link: " );
+	row_name_print( row );
+	printf( "\n" );
+#endif /*DEBUG*/
+
+	VIEW_CLASS( parent_class )->link( view, model, parent );
+
+	/* Edit mode defaults to edit mode for workspace.
+	 */
+        formula_set_edit( itextview->formula, 
+		row->ws->mode == WORKSPACE_MODE_FORMULA );
+}
+
+/* Reset edit mode ... go back to whatever is set for this ws.
+ */
+static void 
+itextview_reset( View *view )
+{
+	iTextview *itextview = ITEXTVIEW( view );
+	iText *itext = ITEXT( VOBJECT( itextview )->iobject );
+	Row *row = HEAPMODEL( itext )->row;
+
+#ifdef DEBUG
+	printf( "itextview_reset: " );
+	row_name_print( row );
+	printf( "\n" );
+#endif /*DEBUG*/
+
+	formula_set_edit( ITEXTVIEW( view )->formula, 
+		row->ws->mode == WORKSPACE_MODE_FORMULA );
+
+	VIEW_CLASS( parent_class )->reset( view );
+}
+
 /* Re-read the text in a tally entry. 
  */
 static void *
@@ -77,103 +173,6 @@ itextview_scan( View *view )
 	return( VIEW_CLASS( parent_class )->scan( view ) );
 }
 
-void
-itextview_activate_cb( GtkWidget *wid, iTextview *itextview )
-{
-	iText *itext = ITEXT( VOBJECT( itextview )->iobject );
-	Row *row = HEAPMODEL( itext )->row;
-
-	/* Reset edits on this row and all children.
-	 */
-	(void) icontainer_map_all( ICONTAINER( row ),
-		(icontainer_map_fn) heapmodel_clear_edited, NULL );
-
-	/* Make sure we scan this text, even if it's not been edited.
-	 */
-	view_scannable_register( VIEW( itextview ) );
-
-	filemodel_set_modified( FILEMODEL( row->ws ), TRUE );
-
-	symbol_recalculate_all();
-}
-
-static void 
-itextview_refresh( vObject *vobject )
-{
-	iTextview *itextview = ITEXTVIEW( vobject );
-	iText *itext = ITEXT( VOBJECT( itextview )->iobject );
-	Row *row = HEAPMODEL( itext )->row;
-
-#ifdef DEBUG
-	printf( "itextview_refresh: " );
-	row_name_print( row );
-	printf( " (0x%x)\n", (unsigned int) vobject );
-#endif /*DEBUG*/
-
-	formula_set_edit( itextview->formula, 
-		row->ws->mode == WORKSPACE_MODE_FORMULA );
-	if( itextview->formula && itext->value.base )
-		formula_set_value_expr( itextview->formula,
-			buf_all( &itext->value ), itext->formula );
-
-	VOBJECT_CLASS( parent_class )->refresh( vobject );
-}
-
-/* Change edit mode.
- */
-static void 
-itextview_set_edit( iTextview *itextview, gboolean edit )
-{
-	iText *itext = ITEXT( VOBJECT( itextview )->iobject );
-
-	formula_set_edit( itextview->formula, edit );
-
-	if( edit ) {
-		view_resettable_register( VIEW( itextview ) );
-
-		/* Make sure we have a formula in the text.
-		 */
-		if( !itext->formula )
-			heapmodel_update_model( HEAPMODEL( itext ) );
-	}
-
-	vobject_refresh_queue( VOBJECT( itextview ) );
-}
-
-static void
-itextview_link( View *view, Model *model, View *parent )
-{
-	iTextview *itextview = ITEXTVIEW( view );
-	iText *itext = ITEXT( model );
-	Row *row = HEAPMODEL( itext )->row;
-
-#ifdef DEBUG
-	printf( "itextview_link: " );
-	row_name_print( row );
-	printf( "\n" );
-#endif /*DEBUG*/
-
-	VIEW_CLASS( parent_class )->link( view, model, parent );
-
-	/* Edit mode defaults to edit mode for workspace.
-	 */
-        itextview_set_edit( itextview, 
-		row->ws->mode == WORKSPACE_MODE_FORMULA );
-}
-
-/* Reset edit mode ... go back to whatever is set for this ws.
- */
-static void 
-itextview_reset( View *view )
-{
-	iTextview *itextview = ITEXTVIEW( view );
-	iText *itext = ITEXT( VOBJECT( itextview )->iobject );
-	Row *row = HEAPMODEL( itext )->row;
-
-	itextview_set_edit( ITEXTVIEW( view ), 
-		row->ws->mode == WORKSPACE_MODE_FORMULA );
-}
-
 static void
 itextview_class_init( iTextviewClass *class )
 {
@@ -197,16 +196,68 @@ itextview_class_init( iTextviewClass *class )
 	view_class->scan = itextview_scan;
 }
 
+void
+itextview_edit_cb( Formula *formula, iTextview *itextview )
+{
+	view_resettable_register( VIEW( itextview ) );
+}
+
+void
+itextview_activate_cb( Formula *formula, iTextview *itextview )
+{
+	iText *itext = ITEXT( VOBJECT( itextview )->iobject );
+	Row *row = HEAPMODEL( itext )->row;
+
+	/* Reset edits on this row and all children. Our (potentially) next
+	 * text will invlidate all of them.
+	 */
+	(void) icontainer_map_all( ICONTAINER( row ),
+		(icontainer_map_fn) heapmodel_clear_edited, NULL );
+
+	/* Make sure we scan this text, even if it's not been edited.
+	 */
+	view_scannable_register( VIEW( itextview ) );
+
+	filemodel_set_modified( FILEMODEL( row->ws ), TRUE );
+
+	symbol_recalculate_all();
+}
+
+static void
+itextview_enter_cb( Formula *formula, iTextview *itextview )
+{
+	iText *itext = ITEXT( VOBJECT( itextview )->iobject );
+	Row *row = HEAPMODEL( itext )->row;
+
+	row_set_status( row );
+	row_show_dependants( row );
+}
+
+static void
+itextview_leave_cb( Formula *formula, iTextview *itextview )
+{
+	iText *itext = ITEXT( VOBJECT( itextview )->iobject );
+	Row *row = HEAPMODEL( itext )->row;
+
+	row_hide_dependants( row );
+}
+
 static void
 itextview_init( iTextview *itextview )
 {
 	itextview->formula = formula_new();
-        gtk_signal_connect_object( GTK_OBJECT( itextview->formula ), 
-		"changed", 
-		GTK_SIGNAL_FUNC( view_changed_cb ), 
-		GTK_OBJECT( itextview ) );
+
+        gtk_signal_connect( GTK_OBJECT( itextview->formula ), "edit", 
+		GTK_SIGNAL_FUNC( itextview_edit_cb ), itextview );
+        gtk_signal_connect_object( GTK_OBJECT( itextview->formula ), "changed", 
+		GTK_SIGNAL_FUNC( view_changed_cb ), itextview );
         gtk_signal_connect( GTK_OBJECT( itextview->formula ), "activate",
                 GTK_SIGNAL_FUNC( itextview_activate_cb ), itextview );
+        gtk_signal_connect( GTK_OBJECT( itextview->formula ), "enter", 
+		GTK_SIGNAL_FUNC( itextview_enter_cb ), itextview );
+        gtk_signal_connect( GTK_OBJECT( itextview->formula ), "leave", 
+		GTK_SIGNAL_FUNC( itextview_leave_cb ), itextview );
+
         gtk_box_pack_start( GTK_BOX( itextview ), 
 		GTK_WIDGET( itextview->formula ), TRUE, FALSE, 0 );
         gtk_widget_show( GTK_WIDGET( itextview->formula ) );
