@@ -51,6 +51,10 @@
 
  */
 
+/*
+#define DEBUG
+ */
+
 #include "ip.h"
 
 /* Cursor bitmaps.
@@ -70,10 +74,6 @@
 #include "BITMAPS/watch_8.xbm"
 #include "BITMAPS/watch_msk.xbm"
 
-/*
-#define DEBUG
- */
-
 static GtkWindowClass *parent_class = NULL;
 
 /* List of all iwindows.
@@ -290,6 +290,11 @@ iwindow_cursor_update( iWindow *iwnd )
 			}
 		}
 
+		/* Pref to disable crosshair.
+		 */
+		if( best_shape == IWINDOW_SHAPE_EDIT && !DISPLAY_CROSSHAIR )
+			best_shape = IWINDOW_SHAPE_NONE;
+
 		gdk_window_set_cursor( 
 			iwindow_get_work_window( iwnd ),
 			iwindow_cursor[best_shape] );
@@ -311,19 +316,37 @@ iwindow_cursor_set( iWindow *iwnd, iWindowShape shape )
 }
 
 static int iwindow_hglass_count = 0;
+static GTimer *iwindow_hglass_timer = NULL;
+static GTimer *iwindow_hglass_update_timer = NULL;
+
+/* Delay before we start showing the spinning hglass.
+ */
+static const double iwindow_hglass_delay = 0.2;
+
+/* Delay between cursor updates.
+ */
+static const double iwindow_hglass_update = 0.5;
 
 void
 animate_hourglass( void )
 {
 	static iWindowShape shape = IWINDOW_SHAPE_HGLASS1;
 
-	if( iwindow_hglass_count ) {
-		iwindow_map_all( 
-			(iWindowMapFn) iwindow_cursor_set, (void *) shape );
+	if( iwindow_hglass_count &&
+		g_timer_elapsed( iwindow_hglass_timer, NULL ) > 
+			iwindow_hglass_delay ) {
 
-		shape += 1;
-		if( shape > IWINDOW_SHAPE_HGLASS8 )
-			shape = IWINDOW_SHAPE_HGLASS1;
+		if( g_timer_elapsed( iwindow_hglass_update_timer, NULL ) > 
+			iwindow_hglass_update ) {
+			iwindow_map_all( 
+				(iWindowMapFn) iwindow_cursor_set, 
+				(void *) shape );
+			g_timer_start( iwindow_hglass_update_timer );
+
+			shape += 1;
+			if( shape > IWINDOW_SHAPE_HGLASS8 )
+				shape = IWINDOW_SHAPE_HGLASS1;
+		}
 	}
 }
 
@@ -334,8 +357,17 @@ set_hourglass( void )
 
 	iwindow_hglass_count += 1;
 
-	if( iwindow_hglass_count == 1 ) 
+	if( iwindow_hglass_count == 1 ) {
+		if( !iwindow_hglass_timer )
+			iwindow_hglass_timer = g_timer_new();
+		if( !iwindow_hglass_update_timer )
+			iwindow_hglass_update_timer = g_timer_new();
+
+		g_timer_start( iwindow_hglass_timer );
+		g_timer_start( iwindow_hglass_update_timer );
+
 		animate_hourglass();
+	}
 }
 
 void
@@ -345,7 +377,7 @@ set_pointer( void )
 
 	iwindow_hglass_count -= 1;
 
-	if( !iwindow_hglass_count )
+	if( !iwindow_hglass_count ) 
 		iwindow_map_all( (iWindowMapFn) iwindow_cursor_set, 
 			(void *) IWINDOW_SHAPE_NONE );
 }
@@ -356,7 +388,7 @@ iwindow_cursor_context_new( iWindow *iwnd, int priority, const char *name )
 	iWindowCursorContext *cntxt = INEW( NULL, iWindowCursorContext );
 
 #ifdef DEBUG
-	printf( "iwindow_cursor_context_new: %s\n", cntxt->name );
+	printf( "iwindow_cursor_context_new: %s\n", name );
 #endif /*DEBUG*/
 
 	cntxt->iwnd = iwnd;
@@ -537,6 +569,22 @@ iwindow_finalize( GObject *gobject )
 		main_quit_test();
 }
 
+static void
+iwindow_destroy( GtkObject *gobject )
+{
+	iWindow *iwnd = IWINDOW( gobject );
+
+#ifdef DEBUG
+	printf( "iwindow_destroy: %s\n", iwnd->title );
+#endif /*DEBUG*/
+
+	/* My instance destroy stuff.
+	 */
+	FREESID( iwnd->parent_unmap_sid, iwnd->parent_window );
+
+	GTK_OBJECT_CLASS( parent_class )->destroy( gobject );
+}
+
 static void
 iwindow_popdown_notify( iWindow *iwnd, iWindowResult result )
 {
@@ -559,14 +607,30 @@ iwindow_popdown_notify( iWindow *iwnd, iWindowResult result )
 		 */
 		iwnd->destroy = FALSE;
 	}
-	else
-		/* Popdown confirmed ... unmap.
+	else {
+#ifdef OS_WIN32
+		/* Pop the parent back up to the top of the window stack. 
+		 * Helps win32 keep window ordering if you alt-tab away from 
+		 * the app while a dialog is being shown.
 		 */
-		gtk_widget_unmap( GTK_WIDGET( iwnd ) );
+		if( iwnd->parent_window && 
+			GTK_WIDGET_MAPPED( iwnd->parent_window ) )
+			gtk_window_present( GTK_WINDOW( iwnd->parent_window ) );
+#endif /*OS_WIN32*/
 
-	call_string_filenamef( (call_string_fn) gtk_accel_map_save,
-		"%s" IM_DIR_SEP_STR "accel_map", get_savedir() );
+		/* Popdown confirmed! Trigger class popdown. _real_popdown()
+		 * does an unmap to hide the window during the rest of the
+		 * destroy.
+		 */
+		IWINDOW_GET_CLASS( iwnd )->popdown( GTK_WIDGET( iwnd ) );
+	}
 
+	calli_string_filenamef( (calli_string_fn) gtk_accel_map_save,
+		"%s" G_DIR_SEPARATOR_S "accel_map", get_savedir() );
+
+	/* If this is the final pending response and ->destroy is true, this 
+	 * will destroy the window.
+	 */
 	iwindow_notify_return( iwnd );
 }
 
@@ -597,7 +661,7 @@ static gboolean
 iwindow_configure_event( GtkWidget *widget, GdkEventConfigure *event )
 {
 	iWindow *iwnd = IWINDOW( widget );
-	
+
 	if( iwnd->width_pref ) {
 		/* Save window size in global prefs.
 		 */
@@ -609,6 +673,23 @@ iwindow_configure_event( GtkWidget *widget, GdkEventConfigure *event )
 		configure_event( widget, event ) );
 }
 
+/* Our parent has been destroyed, kill us too.
+ */
+static void
+iwindow_parent_unmap_cb( GtkWidget *par, iWindow *iwnd )
+{
+#ifdef DEBUG
+	printf( "iwindow_parent_unmap_cb: %s\n", iwnd->title );
+#endif /*DEBUG*/
+
+	/* Here for dead parent ... if parent is dead, we won't need to remove
+	 * the dead-dad signal. 
+	 */
+	iwnd->parent_unmap_sid = 0;
+
+	iwindow_kill( iwnd );
+}
+
 static void 
 iwindow_real_build( GtkWidget *widget )
 {
@@ -644,28 +725,54 @@ iwindow_real_build( GtkWidget *widget )
 			IM_MIN( height, gdk_screen_get_height( screen ) ) );
 	}
 
+	/* Link to parent.
+	 */
+        if( iwnd->parent_window ) {
+		/* Child windows locked in front of parents?
+		 */
+		if( WINDOW_INFRONT )
+			gtk_window_set_transient_for( GTK_WINDOW( iwnd ),
+				GTK_WINDOW( iwnd->parent_window ) );
+
+		/* We watch our parent's "unmap" rather than "destroy" since
+		 * we use gtk_widget_unmap() to hide killed windows during
+		 * popdown (see iwindow_popdown_notify()).
+		 */
+		iwnd->parent_unmap_sid = gtk_signal_connect( 
+			GTK_OBJECT( iwnd->parent_window ), 
+			"unmap",
+			GTK_SIGNAL_FUNC( iwindow_parent_unmap_cb ), iwnd );
+	}
+
         gtk_widget_show( iwnd->work );
 }
 
+static void 
+iwindow_real_popdown( GtkWidget *widget )
+{
+	gtk_widget_unmap( widget );
+}
+
 static void
 iwindow_class_init( iWindowClass *class )
 {
-	GObjectClass *gobject_class;
-	GtkWidgetClass *widget_class;
-
-	gobject_class = (GObjectClass *) class;
-	widget_class = (GtkWidgetClass *) class;
+	GObjectClass *object_class = (GObjectClass *) class;
+	GtkObjectClass *gobject_class = (GtkObjectClass *) class;
+	GtkWidgetClass *widget_class = (GtkWidgetClass *) class;
 
 	parent_class = g_type_class_peek_parent( class );
 
 	/* Init methods.
 	 */
-	gobject_class->finalize = iwindow_finalize;
+	object_class->finalize = iwindow_finalize;
+
+	gobject_class->destroy = iwindow_destroy;
 
 	widget_class->delete_event = iwindow_delete_event;
 	widget_class->configure_event = iwindow_configure_event;
 
 	class->build = iwindow_real_build;
+	class->popdown = iwindow_real_popdown;
 
 	/* Create signals.
 	 */
@@ -682,10 +789,12 @@ iwindow_init( iWindow *iwnd )
 	printf( "iwindow_init: %s\n", iwnd->title );
 #endif /*DEBUG*/
 
-	/* Init our instance fields.
-	 */
 	iwnd->work = NULL;
 
+	iwnd->parent = NULL;
+	iwnd->parent_window = NULL;
+        iwnd->parent_unmap_sid = 0;
+
 	/* Might as well.
 	 */
 	iwnd->accel_group = gtk_accel_group_new();
@@ -713,10 +822,10 @@ iwindow_init( iWindow *iwnd )
 GtkType
 iwindow_get_type( void )
 {
-	static GtkType iwindow_type = 0;
+	static GtkType type = 0;
 
-	if (!iwindow_type) {
-		static const GtkTypeInfo iwnd_info = {
+	if( !type ) {
+		static const GtkTypeInfo info = {
 			"iWindow",
 			sizeof( iWindow ),
 			sizeof( iWindowClass ),
@@ -727,10 +836,10 @@ iwindow_get_type( void )
 			(GtkClassInitFunc) NULL,
 		};
 
-		iwindow_type = gtk_type_unique( GTK_TYPE_WINDOW, &iwnd_info );
+		type = gtk_type_unique( GTK_TYPE_WINDOW, &info );
 	}
 
-	return( iwindow_type );
+	return( type );
 }
 
 GtkWidget *
@@ -756,12 +865,10 @@ iwindow_set_title( iWindow *iwnd, const char *title, ... )
         (void) im_vsnprintf( buf, 1024, title, ap );
         va_end( ap );
 
-	SETSTR( iwnd->title, buf );
-
-	/* If the window has been realised, update title now.
-	 */
-	if( GTK_WIDGET_REALIZED( GTK_WIDGET( iwnd ) ) )
+	if( !iwnd->title || strcmp( iwnd->title, buf ) != 0 ) {
+		IM_SETSTR( iwnd->title, buf );
 		gtk_window_set_title( GTK_WINDOW( iwnd ), iwnd->title );
+	}
 }
 
 void 
@@ -796,6 +903,21 @@ iwindow_set_work_window( iWindow *iwnd, GdkWindow *work_window )
 	iwindow_cursor_update( iwnd );
 }
 
+void 
+iwindow_set_parent( iWindow *iwnd, GtkWidget *parent )
+{
+	assert( !iwnd->parent );
+
+	iwnd->parent = parent;
+
+	/* Get parent window now, we sometimes need it after parent has been
+	 * destroyed.
+	 */
+	if( parent )
+		iwnd->parent_window = 
+			IWINDOW( iwindow_get_root( GTK_WIDGET( parent ) ) );
+}
+
 void *
 iwindow_kill( iWindow *iwnd )
 {
@@ -837,7 +959,7 @@ iwindow_build( iWindow *iwnd )
 	IWINDOW_GET_CLASS( iwnd )->build( GTK_WIDGET( iwnd ) );
 }
 
-/* Get the root window for a widget.
+/* Get the enclosing window for a widget.
  */
 GtkWidget *
 iwindow_get_root( GtkWidget *widget )
@@ -853,6 +975,22 @@ iwindow_get_root( GtkWidget *widget )
 
 		return( iwindow_get_root( parent ) );
 	}
-
-	return( toplevel );
+	else
+		return( toplevel );
 }
+
+/* Get the enclosing no-parent window for a widget.
+ */
+GtkWidget *
+iwindow_get_root_noparent( GtkWidget *widget )
+{
+	GtkWidget *toplevel = iwindow_get_root( widget );
+
+	/* If this is a transient, get the window we popped up from.
+	 */
+	if( IS_IWINDOW( toplevel ) && IWINDOW( toplevel )->parent ) 
+		return( iwindow_get_root( IWINDOW( toplevel )->parent ) );
+	else
+		return( toplevel );
+}
+
