@@ -141,10 +141,15 @@ typedef struct {
 #define IWINDOW_SUSP( X ) ((iWindowSusp *) (X))
 
 struct _iWindow {
-	GtkWindow parent;
+	GtkWindow parent_object;
 
-	/* My instance vars.
+	/* Parent window. Used for (eg.) image displays windows which we need
+	 * to float over the main workspace.
 	 */
+	GtkWidget *parent;		/* Our parent widget */
+	iWindow *parent_window;		/* Our parent window */
+	guint parent_unmap_sid; 	/* Watch parent death here */
+
 	GtkWidget *work;
 	GtkAccelGroup *accel_group;
 
@@ -181,9 +186,10 @@ struct _iWindow {
 typedef struct _iWindowClass {
 	GtkWindowClass parent_class;
 
-	/* Per class build function.
+	/* Per class build/popdown functions.
 	 */
 	void (*build)( GtkWidget * );
+	void (*popdown)( GtkWidget * );
 } iWindowClass;
 
 int iwindow_number( void );
@@ -214,17 +220,20 @@ void iwindow_notify_null( void *client, iWindowResult result );
 GtkType iwindow_get_type( void );
 
 GtkWidget *iwindow_new( GtkWindowType );
-void iwindow_set_title( iWindow *, const char *, ... );
+void iwindow_set_title( iWindow *, const char *, ... )
+	__attribute__((format(printf, 2, 3)));
 void iwindow_set_build( iWindow *, iWindowBuildFn, void *, void *, void * );
 void iwindow_set_popdown( iWindow *, iWindowFn, void * );
 void iwindow_set_size_prefs( iWindow *, const char *, const char * );
 void iwindow_set_work_window( iWindow *iwnd, GdkWindow *work_window );
+void iwindow_set_parent( iWindow *, GtkWidget *par );
 
 void iwindow_build( iWindow * );
 void *iwindow_kill( iWindow * );
 void iwindow_kill_action_cb( GtkAction *action, iWindow *iwnd );
 
 GtkWidget *iwindow_get_root( GtkWidget *widget );
+GtkWidget *iwindow_get_root_noparent( GtkWidget *widget );
 
 #ifdef __cplusplus
 }
