@@ -164,7 +164,7 @@ struct _Reduce {
 
 /* Util.
  */
-void reduce_throw( Reduce *rc );
+void reduce_throw( Reduce *rc ) __attribute__((noreturn));
 
 typedef void *(*reduce_safe_pointer_fn)( Reduce *rc, PElement *, 
 	void *, void *, void *, void * );
@@ -190,18 +190,18 @@ int reduce_get_matrix( Reduce *rc,
 	PElement *base, double *buf, int n, int *xsize, int *ysize );
 void reduce_get_matrix_size( Reduce *rc, 
 	PElement *base, int *xsize, int *ysize );
-gboolean reduce_iselist( Reduce *rc, PElement *base );
-gboolean reduce_islist( Reduce *rc, PElement *base );
-gboolean reduce_isstring( Reduce *rc, PElement *base );
-gboolean reduce_isfinitestring( Reduce *rc, PElement *base );
-gboolean reduce_isrealvec( Reduce *rc, PElement *base );
-gboolean reduce_ismatrix( Reduce *rc, PElement *base );
-gboolean reduce_isclass( Reduce *rc, PElement *klass );
+gboolean reduce_is_elist( Reduce *rc, PElement *base );
+gboolean reduce_is_list( Reduce *rc, PElement *base );
+gboolean reduce_is_string( Reduce *rc, PElement *base );
+gboolean reduce_is_finitestring( Reduce *rc, PElement *base );
+gboolean reduce_is_realvec( Reduce *rc, PElement *base );
+gboolean reduce_is_matrix( Reduce *rc, PElement *base );
+gboolean reduce_is_class( Reduce *rc, PElement *klass );
 int reduce_list_length( Reduce *rc, PElement *base );
 void reduce_list_index( Reduce *rc, PElement *base, int n, PElement *out );
-gboolean reduce_isinstanceof_exact( Reduce *rc, 
+gboolean reduce_is_instanceof_exact( Reduce *rc, 
 	const char *name, PElement *instance );
-gboolean reduce_isinstanceof( Reduce *rc, 
+gboolean reduce_is_instanceof( Reduce *rc, 
 	const char *name, PElement *instance );
 
 /* Main.
