@@ -436,6 +436,98 @@ row_dispose( GObject *gobject )
 	G_OBJECT_CLASS( parent_class )->dispose( gobject );
 }
 
+static void *
+row_add_parent_name( Link *link, BufInfo *buf )
+{
+	Row *row;
+
+	if( link->parent->expr && (row = link->parent->expr->row) ) {
+		row_qualified_name_relative( link->child, row, buf );
+		buf_appends( buf, " " );
+	}
+
+	return( NULL );
+}
+
+static void *
+row_add_child_name( Link *link, BufInfo *buf )
+{
+	Row *row;
+
+	if( link->child->expr && (row = link->child->expr->row) ) {
+		row_qualified_name_relative( link->parent, row, buf );
+		buf_appends( buf, " " );
+	}
+
+	return( NULL );
+}
+
+static void *
+row_add_dirty_child_name( Link *link, BufInfo *buf )
+{
+	if( link->child->dirty ) {
+		symbol_qualified_name( link->child, buf );
+		buf_appends( buf, " " );
+	}
+
+	return( NULL );
+}
+
+static void
+row_info( iObject *iobject, BufInfo *buf )
+{
+	Row *row = ROW( iobject );
+
+	buf_appends( buf, _( "Name" ) );
+	buf_appends( buf, ": " );
+	row_qualified_name( row, buf );
+	buf_appends( buf, "\n" );
+
+	if( row->expr ) 
+		iobject_info( IOBJECT( row->expr ), buf );
+	if( row->child_rhs->itext ) 
+		iobject_info( IOBJECT( row->child_rhs->itext ), buf );
+	if( row->child_rhs->graphic ) 
+		iobject_info( IOBJECT( row->child_rhs->graphic ), buf );
+	if( row->top_row->sym ) {
+		if( row->top_row->sym->topchildren ) {
+			row_qualified_name( row, buf );
+			buf_appends( buf, " " );
+			/* Expands to eg. "B1 refers to: B2, B3".
+			 */
+			buf_appends( buf, _( "refers to" ) );
+			buf_appends( buf, ": " );
+			slist_map( row->top_row->sym->topchildren, 
+				(SListMapFn) row_add_child_name, buf );
+			buf_appends( buf, "\n" );
+		}
+		if( row->top_row->sym->topparents ) {
+			row_qualified_name( row, buf );
+			buf_appends( buf, " " );
+			/* Expands to eg. "B1 is referred to by: B2, B3".
+			 */
+			buf_appends( buf, _( "is referred to by" ) );
+			buf_appends( buf, ": " );
+			slist_map( row->top_row->sym->topparents, 
+				(SListMapFn) row_add_parent_name, buf );
+			buf_appends( buf, "\n" );
+		}
+	}
+	if( row == row->top_row && row->sym->dirty ) {
+		Symbol *sym = row->sym;
+
+		if( sym->ndirtychildren ) {
+			row_qualified_name( row, buf );
+			buf_appends( buf, " " );
+			buf_appends( buf, _( "is blocked on" ) );
+			buf_appends( buf, ": " );
+			slist_map( sym->topchildren,
+				(SListMapFn) row_add_dirty_child_name, buf );
+			buf_appends( buf, "\n" );
+		}
+	}
+}
+
 static Rhs *
 row_get_rhs( Row *row )
 {
@@ -532,6 +624,31 @@ row_parent_remove( iContainer *child )
 	ICONTAINER_CLASS( parent_class )->parent_remove( child );
 }
 
+static View *
+row_view_new( Model *model, View *parent )
+{
+	return( rowview_new() );
+}
+
+static void
+row_scrollto( Model *model, ModelScrollPosition position )
+{
+	Row *row = ROW( model );
+	Column *col = row->top_col;
+
+	/* If our column is closed, there won't be a view to scrollto, ouch!
+	 * Need to open the column first, then scroll to that column. We can't
+	 * scroll to the exact row, since there's no view for it, and won't be
+	 * for a while after we hit the idle loop again.
+	 */
+	if( !col->open ) {
+		column_set_open( col, TRUE );
+		model_scrollto( MODEL( col ), position );
+	}
+
+	MODEL_CLASS( parent_class )->scrollto( model, position );
+}
+
 static gboolean
 row_load( Model *model, 
 	ModelLoadState *state, Model *parent, xmlNode *xnode )
@@ -543,22 +660,19 @@ row_load( Model *model,
 
 	assert( IS_SUBCOLUMN( parent ) );
 
-	/* "name" is loaded by model ... we need it here for the link to the
-	 * sym, grab early.
-	 */
-	if( !get_sprop( xnode, "name", name, 256 ) )
+	if( !get_sprop( xnode, "name", name, 256 ) ) 
 		return( FALSE );
+	IM_SETSTR( IOBJECT( row )->name, name );
 
 #ifdef DEBUG
-	printf( "row_load: loading row %s (xmlNode 0x%x)\n", 
-		name, (unsigned int) xnode );
+	printf( "row_load: loading row %s (xmlNode %p)\n", name, xnode );
 #endif /*DEBUG*/
 
 	/* Popup is optional (top level only)
 	 */
 	(void) get_bprop( xnode, "popup", &row->popup );
 
-	if( scol->istop ) {
+	if( scol->is_top ) {
 		Column *col = scol->top_col;
 		Workspace *ws = col->ws;
 
@@ -569,23 +683,11 @@ row_load( Model *model,
 		(void) compile_new_local( sym->expr );
 		row_link_symbol( row, sym, NULL );
 
-		/* Note this sym on the workspace ... last_symbol is used as 
-		 * part of the print-last-item-on-quit thing.
-		 */
-		ws->last_symbol = sym;
-
 		/* We can't symbol_made() here, we've not parsed our value
 		 * yet. See below ... we just make sure we're on the recomp
 		 * lists.
 		 */
 	}
-	else {
-		/* For locals, note the name in the row and do 
-		 * row_link_symbol() on 1st recomp. See
-		 * subcolumn_class_update_model_sub().
-		 */
-		iobject_set( IOBJECT( row ), name, NULL );
-	}
 
 	if( !MODEL_CLASS( parent_class )->load( model, state, parent, xnode ) )
 		return( FALSE );
@@ -593,7 +695,7 @@ row_load( Model *model,
 	/* If we've loaded a complete row system, mark this row plus any 
 	 * edited subrows dirty, and make sure this sym is dirty too.
 	 */
-	if( scol->istop ) {
+	if( scol->is_top ) {
 		row_dirty_set( row, TRUE );
 		expr_dirty( row->sym->expr, link_serial_new() );
 	}
@@ -601,7 +703,7 @@ row_load( Model *model,
 	/* If we've loaded a toplevel into a closed column, make sure display
 	 * is off.
 	 */
-	if( scol->istop && !scol->top_col->open )
+	if( scol->is_top && !scol->top_col->open )
 		model_set_display( MODEL( row ), FALSE );
 
 	return( TRUE );
@@ -658,6 +760,8 @@ row_save( Model *model, xmlNode *xnode )
 	if( row->top_row == row ) 
 		if( !set_prop( xthis, "popup", bool_to_char( row->popup ) ) )
 			return( NULL );
+	if( !set_sprop( xthis, "name", IOBJECT( row )->name ) )
+		return( NULL );
 
 	return( xthis );
 }
@@ -778,7 +882,7 @@ row_new_heap( Heapmodel *heapmodel, PElement *root )
 			model_set_display( MODEL( row ), FALSE );
 		/* Hide top-level rows in closed columns.
 		 */
-		else if( row->scol->istop && !row->top_col->open )
+		else if( row->scol->is_top && !row->top_col->open )
 			model_set_display( MODEL( row ), FALSE );
 		else
 			model_set_display( MODEL( row ), TRUE );
@@ -796,21 +900,21 @@ row_new_heap( Heapmodel *heapmodel, PElement *root )
 
 	/* Class display only for non-param classes.
 	 */
-	row->isclass = PEISCLASS( root ) && row->sym->type != SYM_PARAM;
+	row->is_class = PEISCLASS( root ) && row->sym->type != SYM_PARAM;
 
 	/* Set the default vis level.
 	 */
 	if( row->child_rhs && row->child_rhs->vislevel == -1 ) {
 		PElement member;
 		double value;
-		gboolean isclass;
+		gboolean is_class;
 
-		if( !heap_isclass( root, &isclass ) )
+		if( !heap_is_class( root, &is_class ) )
 			return( row );
 
 		/* If it's a class with a vis hint, use that.
 		 */
-		if( isclass && 
+		if( is_class && 
 			class_get_member( root, MEMBER_VISLEVEL, 
 				NULL, &member ) &&
 			heap_get_real( &member, &value ) ) 
@@ -841,6 +945,7 @@ static void
 row_class_init( RowClass *class )
 {
 	GObjectClass *gobject_class = (GObjectClass *) class;
+	iObjectClass *iobject_class = (iObjectClass *) class;
 	iContainerClass *icontainer_class = (iContainerClass *) class;
 	ModelClass *model_class = (ModelClass *) class;
 	HeapmodelClass *heapmodel_class = (HeapmodelClass *) class;
@@ -854,11 +959,14 @@ row_class_init( RowClass *class )
 	 */
 	gobject_class->dispose = row_dispose;
 
+	iobject_class->info = row_info;
+
 	icontainer_class->child_add = row_child_add;
 	icontainer_class->parent_add = row_parent_add;
 	icontainer_class->parent_remove = row_parent_remove;
 
-	model_class->view_new = rowview_new;
+	model_class->view_new = row_view_new;
+	model_class->scrollto = row_scrollto;
 	model_class->load = row_load;
 	model_class->save = row_save;
 	model_class->save_test = row_save_test;
@@ -890,7 +998,7 @@ row_init( Row *row )
 	row->err = FALSE;
 
 	row->selected = FALSE;
-	row->isclass = FALSE;
+	row->is_class = FALSE;
 	row->popup = POPUP_NEW_ROWS;
 	row->to_save = FALSE;
 
@@ -967,16 +1075,14 @@ row_link_symbol( Row *row, Symbol *sym, PElement *root )
 Row *
 row_new( Subcolumn *scol, Symbol *sym, PElement *root )
 {
-	Row *row;
+	Row *row = g_object_new( TYPE_ROW, NULL );
 
 #ifdef DEBUG_NEW
 	printf( "row_new: " );
 	dump_tiny( sym );
-	printf( ", pos = %d\n", pos );
+	printf( "\n" );
 #endif /*DEBUG_NEW*/
 
-	row = ROW( g_object_new( TYPE_ROW, NULL ) );
-
 	/* Don't make a display or a RHS for invisible rows.
 	 */
 	if( !row_is_displayable( sym ) )
@@ -1109,7 +1215,7 @@ row_link_build( Row *row )
 
 	/* Build new recomp list. Only for class displays.
 	 */
-	if( !row->scol->istop && row->expr && 
+	if( !row->scol->is_top && row->expr && 
 		row_link_build2( row->expr, row ) )
 		return( row );
 
@@ -1516,6 +1622,8 @@ row_recomp_row( Row *row )
 			return( FALSE );
 	}
 
+	animate_hourglass();
+
 	return( TRUE );
 }
 
@@ -1528,9 +1636,11 @@ row_recomp_all( Row *top_row )
 		Row *dirty_row = ROW( top_row->recomp->data );
 
 #ifdef DEBUG_ROW
-		printf( "row_recomp_all: starting for row " );
-		row_name_print( dirty_row );
-		printf( "\n" );
+		static GTimer *timer = NULL;
+
+		if( !timer )
+			timer = g_timer_new();
+		g_timer_reset( timer );
 #endif /*DEBUG_ROW*/
 
 		if( !row_recomp_row( dirty_row ) ) 
@@ -1540,6 +1650,12 @@ row_recomp_all( Row *top_row )
 		else
 			row_dirty_clear( dirty_row );
 
+#ifdef DEBUG_ROW
+		printf( "row_recomp_all: done row " );
+		row_name_print( dirty_row );
+		printf( " - %gs\n", g_timer_elapsed( timer, NULL ) );
+#endif /*DEBUG_ROW*/
+
 #ifdef DEBUG
 		printf( "row_recomp_all: after row recomp, top value now " );
 		pgraph( &top_row->expr->root );
@@ -1576,7 +1692,7 @@ row_recomp( Row *row )
 
 	/* Remove all top-level dependencies.
 	 */
-	(void) symbol_link_destroy( top_row->sym );
+	symbol_link_destroy( top_row->sym );
 
 	/* Remove any row recomp links we have.
 	 */
@@ -1630,24 +1746,29 @@ row_recomp( Row *row )
 	 * recalc again.
 	 */
 	if( slist_equal( top_row->recomp_save, top_row->recomp ) ) {
-		/* Can just mark all rows clean.
+		/* Provided we didn't abandon recomp on an error, we can 
+		 * just mark all rows clean.
 		 */
-		slist_map( top_row->recomp, 
-			(SListMapFn) row_dirty_clear, NULL );
+		if( !top_row->err )
+			slist_map( top_row->recomp, 
+				(SListMapFn) row_dirty_clear, NULL );
 	}
 	else {
 #ifdef DEBUG_DIRTY
 		printf( "row_recomp: recomp list has changed ... pass 2\n" );
 #endif /*DEBUG_DIRTY*/
 
-		/* Rebuild all dirty rows.
+		/* Rebuild all dirty rows in a second pass.
 		 */
 		row_recomp_all( top_row );
 	}
 
 	IM_FREEF( g_slist_free, top_row->recomp_save );
 
-	symbol_dirty_clear( top_row->sym );
+	/* The symbol can be cleared as well.
+	 */
+	if( !top_row->err )
+		symbol_dirty_clear( top_row->sym );
 
 	/* Now we're clean, all models can update from the heap. Rows
 	 * containing errors can have bad pointers in, so careful.
@@ -1697,6 +1818,15 @@ row_deselect( Row *row )
 	ws->selected = g_slist_remove( ws->selected, row );
 	row->selected = FALSE;
 
+	/* Hack: if this is a matrix with selected cells, deselect the matrix
+	 * sellection too. We should really have a row method for this I
+	 * guess :-( See also workspace_selected_names_sub().
+	 */
+	if( row->child_rhs && row->child_rhs->graphic &&
+		IS_MATRIX( row->child_rhs->graphic ) &&
+		MATRIX( row->child_rhs->graphic )->selected ) 
+		matrix_deselect( MATRIX( row->child_rhs->graphic ) );
+
 	iobject_changed( IOBJECT( row ) );
 	iobject_changed( IOBJECT( ws ) );
 
@@ -1719,6 +1849,21 @@ row_select2( Row *row )
 	}
 }
 
+/* Make sure a row is selected ... used for (eg.) select changed on gktsheet.
+ * No deselection.
+ */
+void *
+row_select_ensure( Row *row )
+{
+	row_select2( row );
+
+	/* Note for extend select.
+	 */
+	row->top_col->last_select = row;
+
+	return( NULL );
+}
+
 /* Select a row, deselecting others first.
  */
 void *
@@ -1851,7 +1996,7 @@ row_show_dependants( Row *row )
 void
 row_hide_dependants( Row *row )
 {
-	Symbol *topsym = row->top_row->sym;
+	Symbol *topsym;
 
 #ifdef DEBUG
 	printf( "row_hide_dependants: " );
@@ -1859,7 +2004,7 @@ row_hide_dependants( Row *row )
 	printf( "\n" );
 #endif /*DEBUG*/
 
-	if( topsym ) {
+	if( row->top_row && (topsym = row->top_row->sym) ) {
 		slist_map( topsym->topparents,
 			(SListMapFn) row_show_parent, 
 			GUINT_TO_POINTER( ROW_SHOW_NONE ) );
@@ -1869,6 +2014,43 @@ row_hide_dependants( Row *row )
 	}
 }
 
+/* Set help for a row. Used by rowview and itextview etc. on mouseover.
+ */
+void
+row_set_status( Row *row )
+{
+	Expr *expr = row->expr;
+
+	char str[MAX_LINELENGTH];
+	BufInfo buf;
+
+	/* No symbol? eg. on load error.
+	 */
+	if( !expr )
+		return;
+
+	buf_init_static( &buf, str, MAX_LINELENGTH );
+
+	row_qualified_name( row, &buf );
+
+	if( expr->err ) {
+		buf_appends( &buf, ": " );
+		buf_appends( &buf, expr->error_top );
+	}
+	else if( row->child_rhs->itext ) {
+		iText *itext = ITEXT( row->child_rhs->itext );
+
+		buf_appends( &buf, " = " );
+
+		if( row->ws->mode != WORKSPACE_MODE_FORMULA )
+			buf_appends( &buf, NN( itext->formula ) );
+		else 
+			buf_appends( &buf, buf_all( &itext->value ) );
+	}
+
+	workspace_set_status( row->ws, "%s", buf_firstline( &buf ) );
+}
+
 /* Sub fn of below ... search inside a row hierarcy. Context is (eg.) row
  * "A1", path is (eg.) "super.name".
  */
@@ -1967,5 +2149,3 @@ row_parse_name( Symbol *context, const char *path )
 
 	return( row_parse_name( sym, tail ) );
 }
-
-
