@@ -70,10 +70,6 @@ Symbol *symbol_root = NULL;
  */
 static GSList *symbol_leaf_set = NULL;
 
-/* Set if we are currently recalculating.
- */
-gboolean symbol_busy = FALSE;
-
 static FilemodelClass *parent_class = NULL;
 
 /* Apply a function to a symbol ... and any locals.
@@ -232,7 +228,7 @@ symbol_clear( Symbol *sym )
 	sym->expr = NULL;
 
 	sym->base.type = ELEMENT_NOVAL;
-	sym->base.ele = (void *) 15;
+	sym->base.ele = (void *) 15;	/* handy for debugging */
 
 	sym->dirty = FALSE;
 	sym->parents = NULL;
@@ -277,9 +273,12 @@ symbol_root_init( void )
 /* Should a symbol be in the leaf set?
  */
 static gboolean
-symbol_isleafable( Symbol *sym )
+symbol_is_leafable( Symbol *sym )
 {
-	if( is_top( sym ) && sym->dirty && sym->expr && !sym->expr->err && 
+	if( is_top( sym ) && 
+		sym->dirty && 
+		sym->expr && 
+		!sym->expr->err && 
 		sym->ndirtychildren == 0 )
 		return( TRUE );
 
@@ -298,9 +297,9 @@ symbol_sanity( Symbol *sym )
 				symbol_name( sym ) );
 	}
 
-	if( symbol_isleafable( sym ) && !sym->leaf )
+	if( symbol_is_leafable( sym ) && !sym->leaf )
 		error( "sanity failure #2 for sym \"%s\"", symbol_name( sym ) );
-	if( !symbol_isleafable( sym ) && sym->leaf )
+	if( !symbol_is_leafable( sym ) && sym->leaf )
 		error( "sanity failure #3 for sym \"%s\"", symbol_name( sym ) );
 	if( sym->leaf && !g_slist_find( symbol_leaf_set, sym ) )
 		error( "sanity failure #6 for sym \"%s\"", symbol_name( sym ) );
@@ -428,26 +427,22 @@ symbol_made( Symbol *sym )
 }
 
 static void *
-symbol_not_defined_sub( Link *link, BufInfo *buf, gboolean *first )
+symbol_not_defined_sub( Link *link, BufInfo *buf )
 {
-	if( !*first )
-		buf_appendf( buf, "\n" );
-	*first = FALSE;
-
-	buf_appendf( buf, "\"" );
 	symbol_qualified_name( link->parent, buf );
-	buf_appendf( buf, "\"" );
+	buf_appends( buf, " " );
 
 	if( link->parent->tool ) {
 		Tool *tool = link->parent->tool;
 
-		buf_appendf( buf, ", " );
+		buf_appends( buf, "(" );
 		buf_appendf( buf, _( "defined in toolkit \"%s\"" ),
 			IOBJECT( tool->kit )->name );
 		if( tool->lineno != -1 ) {
-			buf_appendf( buf, ", " );
+			buf_appends( buf, ", " );
 			buf_appendf( buf, _( "line %d" ), tool->lineno );
 		}
+		buf_appends( buf, ") " );
 	}
 
 	return( NULL );
@@ -464,17 +459,16 @@ symbol_not_defined( Symbol *sym )
 
 	error_top( _( "Not found." ) );
 	buf_init_static( &buf, txt, 256 );
-	buf_appendf( &buf, _( "Symbol \"%s\" is not defined." ), 
+	buf_appendf( &buf, _( "Symbol %s is not defined." ), 
 		symbol_name( sym ) );
+	buf_appends( &buf, "\n" );
 	if( sym->topparents ) {
-		gboolean first;
-
-		buf_appendf( &buf, "\n" );
-		buf_appendf( &buf, _( "Referred to by:" ) );
-		buf_appendf( &buf, "\n" );
-		first = TRUE;
+		buf_appendf( &buf, _( "%s is referred to by" ),
+			symbol_name( sym ) );
+		buf_appends( &buf, ": " );
 		slist_map2( sym->topparents,
-			(SListMap2Fn) symbol_not_defined_sub, &buf, &first );
+			(SListMap2Fn) symbol_not_defined_sub, &buf, NULL );
+		buf_appends( &buf, "\n" );
 	}
 	error_sub( buf_all( &buf ) );
 }
@@ -504,7 +498,7 @@ symbol_dispose( GObject *gobject )
 #ifdef DEBUG_MAKE
 	printf( "symbol_dispose: " );
 	symbol_name_print( sym );
-	printf( " (0x%x)\n", (unsigned int) sym );
+	printf( " (%p)\n", sym );
 #endif /*DEBUG_MAKE*/
 
 	/* Note the impending destruction of this symbol. It's a state
@@ -605,7 +599,7 @@ symbol_init( Symbol *sym )
 	symbol_clear( sym );
 
 #ifdef DEBUG_MAKE
-	printf( "symbol_init: (0x%x)\n", (unsigned int) sym );
+	printf( "symbol_init: (%p)\n", sym );
 #endif /*DEBUG_MAKE*/
 }
 
@@ -652,7 +646,7 @@ symbol_new( Compile *compile, const char *name )
 #ifdef DEBUG_MAKE
 		printf( "symbol_new: redefining " );
 		symbol_name_print( sym );
-		printf( " (0x%x)\n", (unsigned int) sym );
+		printf( " (%p)\n", sym );
 #endif /*DEBUG_MAKE*/
 	}
 	else {
@@ -664,7 +658,7 @@ symbol_new( Compile *compile, const char *name )
 #ifdef DEBUG_MAKE
 		printf( "symbol_new: creating " );
 		symbol_name_print( sym );
-		printf( " (0x%x)\n", (unsigned int) sym );
+		printf( " (%p)\n", sym );
 #endif /*DEBUG_MAKE*/
 	}
 
@@ -827,7 +821,7 @@ symbol_parameter_builtin_init( Symbol *sym )
 
 /* Get the next dirty leaf symbol.
  */
-Symbol *
+static Symbol *
 symbol_leaf_next( void ) 
 {
 	if( symbol_leaf_set )
@@ -836,21 +830,46 @@ symbol_leaf_next( void )
 		return( NULL ); 
 }
 
+/* Are there symbols we can recalculate? Used to display "Calculating ..."
+ * status.
+ */
+gboolean 
+symbol_busy( void )
+{
+	return( symbol_leaf_set != NULL );
+}
+
 /* Set leaf state.
  */
 static void
 symbol_set_leaf( Symbol *sym, gboolean leaf )
 {
 	if( sym->leaf != leaf ) {
+		gboolean changed;
+
 		sym->leaf = leaf;
 
-		if( leaf )
+		changed = FALSE;
+		if( leaf ) {
+			if( !symbol_leaf_set )
+				changed = TRUE;
+
 			symbol_leaf_set = 
 				g_slist_prepend( symbol_leaf_set, sym );
-		else
+		}
+		else {
+			assert( symbol_leaf_set );
+
 			symbol_leaf_set = 
 				g_slist_remove( symbol_leaf_set, sym );
 
+			if( !symbol_leaf_set )
+				changed = TRUE;
+		}
+
+		if( changed )
+			iobject_changed( IOBJECT( reduce_context->heap ) );
+
 		if( sym->expr && sym->expr->row )
 			iobject_changed( IOBJECT( sym->expr->row ) );
 	}
@@ -865,7 +884,7 @@ symbol_state_change( Symbol *sym )
 
 	/* Used to do more ... now we just set leaf.
 	 */
-	symbol_set_leaf( sym, symbol_isleafable( sym ) );
+	symbol_set_leaf( sym, symbol_is_leafable( sym ) );
 }
 
 /* Recalculate a symbol. We know we are dirty and have no dirty ancestors.
@@ -873,10 +892,14 @@ symbol_state_change( Symbol *sym )
 static gboolean
 symbol_recalculate_sub( Symbol *sym )
 {
+	gboolean result = TRUE;
+
 #ifdef DEBUG_TIME
-	printf( "symbol_recalculate_sub: " );
-	symbol_name_print( sym );
-	printf( "\n" );
+	static GTimer *timer = NULL;
+
+	if( !timer )
+		timer = g_timer_new();
+	g_timer_reset( timer );
 #endif /*DEBUG_TIME*/
 
 	if( is_value( sym ) ) {
@@ -886,7 +909,7 @@ symbol_recalculate_sub( Symbol *sym )
 			 */
 			row_recomp( sym->expr->row );
 			if( sym->expr->row->err )
-				return( FALSE );
+				result = FALSE;
 		}
 		else if( sym->expr->compile->nparam > 0 ) {
 			/* This is a function ... just mark it clean and wait
@@ -897,23 +920,33 @@ symbol_recalculate_sub( Symbol *sym )
 			expr_new_value( sym->expr );
 		}
 		else {
-			gboolean res;
+			if( reduce_regenerate( sym->expr, &sym->expr->root ) ) {
+				symbol_dirty_clear( sym );
+				expr_new_value( sym->expr );
+			}
+			else
+				result = FALSE;
 
-			res = reduce_regenerate( sym->expr, &sym->expr->root );
-
-			if( !res )
-				return( FALSE );
-
-			symbol_dirty_clear( sym );
-			expr_new_value( sym->expr );
 		}
 	}
 	else
 		symbol_dirty_clear( sym );
 
-	return( TRUE );
+#ifdef DEBUG_TIME
+	printf( "symbol_recalculate_sub: " );
+	symbol_name_print( sym );
+	printf( " - %gs\n", g_timer_elapsed( timer, NULL ) );
+#endif /*DEBUG_TIME*/
+
+	return( result );
 }
 
+/* We can get called recursively .. eg. we do an im_tiff2vips(), that
+ * pops a progress box, that triggers idle, that tries to recalc a
+ * leaf again.
+ */
+static gboolean symbol_running = FALSE;
+
 /* Recalc a symbol ... with error checks.
  */
 void *
@@ -930,26 +963,31 @@ symbol_recalculate_check( Symbol *sym )
 		symbol_dirty_clear( sym );
 		return( NULL );
 	}
+	if( symbol_running )
+		return( NULL );
 
 	reduce_context->heap->filled = FALSE;
-	if( !symbol_recalculate_sub( sym ) || reduce_context->heap->filled ) {
+	symbol_running = TRUE;
+	if( !symbol_recalculate_sub( sym ) || 
+		reduce_context->heap->filled ) {
 		expr_error_set( sym->expr );
+		symbol_running = FALSE;
 		return( sym );
 	}
+	symbol_running = FALSE;
 
 	return( NULL );
 }
 
-/* Recompute the leaf set.
+/* Recalculate a symbol. FALSE if no symbols can be recalced.
  */
-static void
-symbol_recalculate_leaves( void )
+static gboolean
+symbol_recalculate_leaf( void )
 {
+	gboolean recalculated;
 	Symbol *sym;
 
-	/* Read all text widgets off the screen.
-	 */
-	(void) view_scan_all();
+	recalculated = FALSE;
 
 #ifdef DEBUG
 	printf( "symbol_recalculate_leaves: Leaf set: " );
@@ -959,7 +997,7 @@ symbol_recalculate_leaves( void )
 
 	/* Grab stuff off the leaf set.
 	 */
-	while( (sym = symbol_leaf_next()) ) {
+	if( (sym = symbol_leaf_next()) ) {
 		/* Should be dirty with no dirty children. Unless it's a
 		 * function, in which case dirty kids are OK.
 		 */
@@ -972,62 +1010,76 @@ symbol_recalculate_leaves( void )
 		 */
 		(void) symbol_recalculate_check( sym );
 
-		/* Process some more repaints.
-
-			FIXME 
-
-		if( mainw_countdown_animate( 99 ) )
-			return;
+		/* Note a pending GC.
 		 */
-		animate_hourglass();
+		(void) heap_gc_request( reduce_context->heap );
+
+		/* We have recalculated a symbol.
+		 */
+		recalculated = TRUE;
 	}
 
-	/* Note a pending GC.
-	 */
-	(void) heap_gc_request( reduce_context->heap );
+	return( recalculated );
 }
 
-/* Recalculate the symbol table, even if auto recomp is off.
+/* Our idle recomp callback. 
+ */
+static gint symbol_idle_id = 0;
+
+static gboolean
+symbol_recalculate_idle_cb( void )
+{
+	gboolean run_again;
+
+	run_again = TRUE;
+
+	if( !symbol_running ) {
+		set_hourglass();
+
+		if( !symbol_recalculate_leaf() ) {
+			/* Nothing more to do: shut down idle recomp.
+			 */
+			symbol_idle_id = 0;
+
+			run_again = FALSE;
+		}
+
+		set_pointer();
+	}
+
+	return( run_again );
+}
+
+/* Recalculate ... either nudge the idle recomp, or in batch mode, do a recomp
+ * right now.
  */
 void
-symbol_recalculate_all_force( void )
+symbol_recalculate_all_force( gboolean now )
 {
-#ifdef DEBUG_TIME
-	static GTimer *compute_timer = NULL;
-
-	if( !compute_timer )
-		compute_timer = g_timer_new();
-#endif /*DEBUG_TIME*/
-
-	/* Make sure we cannot get nested calls. 
-	 */
-	if( symbol_busy )
-		return;
-
 #ifdef DEBUG
-	/* Walk the whole table, doing a sanity check.
-	 */
 	icontainer_map( ICONTAINER( symbol_root->expr->compile ), 
 		(icontainer_map_fn) symbol_sanity, NULL, NULL );
 #endif /*DEBUG*/
 
-	/* Lock and calculate.
+	/* In case we're called directly.
 	 */
-	set_hourglass();
-	symbol_busy = TRUE;
-	reduce_total_recomputations = 0;
-#ifdef DEBUG_TIME
-	g_timer_reset( compute_timer );
-#endif /*DEBUG_TIME*/
-	symbol_recalculate_leaves();
-	symbol_busy = FALSE;
-	set_pointer();
+	(void) view_scan_all();
 
-#ifdef DEBUG_TIME
-	printf( "symbol_recalculate_all: %d reductions in %g secs\n",
-		reduce_total_recomputations,
-		g_timer_elapsed( compute_timer, NULL ) );
-#endif /*DEBUG_TIME*/
+	if( symbol_running )
+		/* Do nothing.
+		 */
+		;
+	else if( main_option_batch || now ) {
+		set_hourglass();
+
+		while( symbol_recalculate_leaf() )
+			;
+
+		set_pointer();
+	}
+	else if( !symbol_idle_id ) 
+		symbol_idle_id = g_idle_add( 
+			(GSourceFunc) symbol_recalculate_idle_cb, NULL );
 }
 
 /* Recalculate the symbol table.
@@ -1040,19 +1092,6 @@ symbol_recalculate_all( void )
 	 */
 	(void) view_scan_all();
 
-	if( CALC_RECOMP )
-		symbol_recalculate_all_force();
-}
-
-/* Do a general recalc, then repeat for a particular sym to pick up any
- * errors.
- */
-void *
-symbol_recalculate( Symbol *sym )
-{
-	if( symbol_busy )
-		return( NULL );
-
-	symbol_recalculate_all();
-	return( symbol_recalculate_check( sym ) );
+	if( mainw_auto_recalc )
+		symbol_recalculate_all_force( FALSE );
 }
