@@ -70,6 +70,12 @@ toolkitgroup_changed( iObject *iobject )
 	IOBJECT_CLASS( parent_class )->changed( iobject );
 }
 
+static View *
+toolkitgroup_view_new( Model *model, View *parent )
+{
+	return( toolkitgroupview_new() );
+}
+
 static void
 toolkitgroup_class_init( ToolkitgroupClass *class )
 {
@@ -88,7 +94,7 @@ toolkitgroup_class_init( ToolkitgroupClass *class )
 
 	iobject_class->changed = toolkitgroup_changed;
 
-	model_class->view_new = toolkitgroupview_new;
+	model_class->view_new = toolkitgroup_view_new;
 }
 
 static void
@@ -129,7 +135,8 @@ toolkitgroup_link( Toolkitgroup *kitg, Symbol *root )
 	assert( root );
 
 	kitg->root = root;
-	im_snprintf( buf, 256, "toolkits_local_to_%s",
+
+	im_snprintf( buf, 256, _( "Toolkits for %s" ),
 		IOBJECT( root )->name );
 	iobject_set( IOBJECT( kitg ), buf, NULL );
 }
