@@ -75,7 +75,8 @@ int trace_get_mark( void );
 GtkType trace_get_type( void );
 Trace *trace_new( void );
 
-void trace_text( TraceFlags flags, const char *fmt, ... );
+void trace_text( TraceFlags flags, const char *fmt, ... )
+	__attribute__((format(printf, 2, 3)));
 
 void trace_pelement( PElement *pe );
 void trace_node( HeapNode *node );
