@@ -82,21 +82,27 @@ struct _Workspace {
 	GSList *selected;	/* Rows selected in this workspace */
 	GSList *errors;		/* Rows with errors */
         WorkspaceMode mode;	/* Display mode */
-	gboolean compatibility;	/* In compatibility mode */
+
+	/* Compatibility. 7.8.x used different names for Marks, and had strange
+	 * position behaviour
+	 */
+	gboolean compatibility;	/* In 7.8.x compatibility mode */
+
+	/* Other versions (7.10 etc.) need special compat toolkits. 0 here for
+	 * no compat toolkits loaded.
+	 */
+	int compat_major;
+	int compat_minor;
 
 	/* Control load/save for this ws.
 	 */
 	WorkspaceLoadType load_type;
 	WorkspaceSaveType save_type;
 
-	/* Set during ws load as hints for display.
-	 */
-	Symbol *last_symbol;
-	Column *last_column;
-	gboolean print_last;	/* Print last sym to stdout on close */
-
 	Rect area;		/* Rect enclosing the set of columns */
 	Rect vp;		/* Viewport hint ... set by views */
+	int window_width;	/* Enclosing window size ... set by views */
+	int window_height;
 
 	guint auto_save_timeout;/* Timeout for next autosave */
 
@@ -180,7 +186,8 @@ gboolean workspace_selected_recalc( Workspace *ws );
 void workspace_selected_remove_yesno( Workspace *ws, GtkWidget *parent );
 gboolean workspace_selected_ungroup( Workspace *ws );
 
-void workspace_set_status( Workspace *ws, const char *fmt, ... );
+void workspace_set_status( Workspace *ws, const char *fmt, ... )
+	__attribute__((format(printf, 2, 3)));
 
 void workspace_set_mode( Workspace *ws, WorkspaceMode mode );
 
